/*
 *                        main.cpp
 *                           -------------------
 *  begin                : mer dic  4 22:54:15 CET 2002
 *  copyright            : (C) 2002 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * 
 */
static char* cvs_id = "$Id: main.cpp,v 1.7 2005/01/22 14:31:51 gbertani Exp $";

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#include <kcmdlineargs.h>
#include <kurl.h>
#include <kaboutdata.h>
#include "config.h"
#include "serlook.h"


static const char *description = 
{
    "SerLooK is a GUI tool for serial\n"
    "lines data traffic inspection\n"
    "and debugging"
}; 
    
    
static KCmdLineOptions options[] =
{
    { "+[File]", I18N_NOOP("file to open"), 0 },
    { 0, 0, 0 }
    // INSERT YOUR COMMANDLINE OPTIONS HERE
};


/**
 * Debug messages handler, 
 * directly from QT2 documentation
 */ 
void myMessageOutput( QtMsgType type, const char *msg )
{
#ifndef QT_NO_DEBUG
    switch ( type ) {
        case QtDebugMsg:
            fprintf( stderr, "Debug: %s\n", msg );
            break;
            
        case QtWarningMsg:
            //I got rid of "QGArray::find: Index X out of range"
            //messages, because this is a normal behavior in
            //terminal views.
            if ( strstr( msg, "out of range" ) == 0 )
            {
                fprintf( stderr, "Warning: %s\n", msg );
            }
            break;
        case QtFatalMsg:
            fprintf( stderr, "Fatal: %s\n", msg );
            abort();                        // dump core on purpose
    }
#endif
}


int main(int argc, char *argv[])
{
    qInstallMsgHandler( myMessageOutput );
    

   
    KAboutData aboutData( "serlook", 
                        I18N_NOOP("SerLooK"),
                        VERSION, 
                        description, 
                        KAboutData::License_GPL,
                        "(c) 2003, giuseppe massimo bertani", 
                        0, 
                        "http://serlook.sunsite.dk", 
                        "gmbertani@serlook.sunsite.dk");
    
    aboutData.addAuthor("giuseppe massimo bertani",
                        "\nAuthor and maintainer\n\n" 
                        "This is work in progress. Let me know what do you think\n"
                        "about it - wishes, suggestions, etc.\n" 
                        "Contributions will be highly appreciated :-)",
                        "gmbertani@serlook.sunsite.dk",
                        "http://www.geocities.com/gmbertani"
                        );

                        
    aboutData.addCredit("espen sand", 
                "\nContributor\n\n"
                "SerLooK reuses some classes from his khexedit\n"
                "program for dump views and data buffers" , 
                "espensa@online.no"
                );

    aboutData.addCredit("lars doelle",
                "\nContributor\n\n"
                "SerLooK reuses the VT102 emulation engine from his\n"
                "konsole for terminal views" ,
                "lars.doelle@on-line.de"
                );    
    
    /* code snippet to set the app icon adapted from Krusader */
    int i = 0;
    char* argx[argc+2];
    static char* iconArg =  "-icon";
    static char* iconFileArg = "lo32-app-serlook";
    CHECK_PTR(argx);
    for (i = 0; i < argc; i++)
    {
        argx[i] = argv[i];
    }
    argx[argc] = iconArg;
    argx[++argc] = iconFileArg;
    argx[++argc] = 0;
    argv = argx;
    
    KCmdLineArgs::init( argc, argv, &aboutData );
    KCmdLineArgs::addCmdLineOptions( options ); // Add our own options.

    KApplication app;
 
    if (app.isRestored())
    {
        RESTORE(SerLooKApp);
    }
    else 
    {
        /*
         *
         * SerLooK is a strange MDI application: only one document (session)
         * can be opened in the same time, with multiple views (MVI)
         */
        
        SerLooKApp *mviApp = new SerLooKApp();
        mviApp->show();
        
        
        //serlook starts correctly when the GUI
        //is already displayed before opening the default
        //untitled session. If someone knows a smarter
        //way to do that I'll be happy to implement it.
        app.processEvents();

        KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

        
        
        if (args->count() == 1)
        {
            /*
             * Loading the session file from the command line
             */

            KURL base( QDir::currentDirPath() );
            KURL url( base, args->arg(0) );
            mviApp->openDocumentFile( url );
        }
        else
        {
            KURL url;   
            mviApp->openDocumentFile( url );
        }
        
        //args->clear(); it's really needed??
    }


    
    return app.exec();
}  


