/*
 *                        serlook.cpp
 *                           -------------------
 *  begin                : mer dic  4 22:54:15 CET 2002
 *  copyright            : (C) 2002 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * 
 */

static char* cvs_id = "$Id: serlook.cpp,v 1.14 2006/01/19 10:19:54 gbertani Exp $";

#include "slsession.h"
#include "slview.h"
#include "slsessiondialog.h"
#include "sldumpview.h"
#include "slterminalview.h"
#include "sleventview.h"
#include "slhexbuffer.h"
#include "slmodemleds.h"
#include "serlook.h"


SerLooKApp::SerLooKApp():KMainWindow(0,"SerLooK")
{
    KIconLoader *loader = KGlobal::iconLoader();
    QPixmap icon;
    icon = loader->loadIcon("lo16-app-serlook", KIcon::User);
    setIcon(icon);
    
    initializing = true;
    acquisition = false;
    
        
    config=kapp->config();
   
    //printer configuration
    printer = new CHexPrinter();
    CHECK_PTR( printer );
    printer->setScaleToFit(true);
    printer->setAll(true);
    printer->setPageMarginMM(0,0,0,0);
    untitledCount=0;

    pDoc = 0;

    //main window construction
    initView();
    initStatusBar();
    initActions();
    initViewToolBar();
    initMacroToolBar();
    readOptions();

    documentActionsEnabling();
}

SerLooKApp::~SerLooKApp()
{
    delete printer;
}

void SerLooKApp::initActions()
{
    //the action names come from serlookui.rc - that file contains
    //the new actions defined by serlook - the standard actions like file_open
    //are placed in ui_standards.rc instead

    //file menu
    fileNew = KStdAction::openNew(this, SLOT(slotFileNew()), actionCollection());
    fileNew->setStatusText(i18n("Creates a new session"));
    fileOpen = KStdAction::open(this, SLOT(slotFileOpen()), actionCollection());

    fileOpen->setStatusText(i18n("Opens an existing session file"));
    fileOpenRecent = KStdAction::openRecent(this, SLOT(slotFileOpenRecent(const KURL&)), actionCollection());
    fileOpenRecent->setStatusText(i18n("Opens a recently used session file"));
    fileSave = KStdAction::save(this, SLOT(slotFileSave()), actionCollection());
    fileSave->setStatusText(i18n("Saves the actual session on a file"));
    fileSaveAs = KStdAction::saveAs(this, SLOT(slotFileSaveAs()), actionCollection());
    fileSaveAs->setStatusText(i18n("Saves the actual session as..."));
    filePrint = KStdAction::print(this, SLOT(slotFilePrint()), actionCollection());
    filePrint ->setStatusText(i18n("Prints out the session settings and the active view"));
    fileQuit = KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());
    fileQuit->setStatusText(i18n("Quits the application"));

    //adding nonstandard menu entry under standard file menu (needed serlookui.rc):
    fileSend = new KAction(i18n("s&End..."), "slsend.png", 0, this, SLOT(slotFileSend()), actionCollection(),"file_send");
    fileSend->setStatusText(i18n("Sends a binary file"));
    fileCleanLogs = new KAction(i18n("clean &Log"), "slcleanlogs.png" , 0, this, SLOT(slotFileCleanLogs()), actionCollection(),"file_cleanlogs");
    fileCleanLogs->setStatusText(i18n("Clean all the opened log filesp"));
    
    //edit menu
    //Cannot delete neither cut data from rx/tx serial buffers !!!
    //
    editCopy = KStdAction::copy(this, SLOT(slotEditCopy()), actionCollection());
    editCopy->setStatusText(i18n("Copies the selected section to the clipboard"));
    editPaste = KStdAction::paste(this, SLOT(slotEditPaste()), actionCollection());
    editPaste->setStatusText(i18n("Pastes the clipboard contents to actual position (only in output panes)"));

    //settings menu
    setViewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection());
    setViewToolBar->setStatusText(i18n("Enables/disables the toolbar"));
    setViewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection());
    setViewStatusBar->setStatusText(i18n("Enables/disables the statusbar"));
    setGlobalPreferences = KStdAction::preferences(this, SLOT(slotGlobalPreferences()), actionCollection());
    setGlobalPreferences->setStatusText(i18n("Global parameters dialog box"));

    //adding nonstandard entry under settings menu (needed serlookui.rc):
    setViewMacroBar = new KToggleAction(i18n("show &Macro toolbar"), 0, 0, this, SLOT(slotViewMacroBar()), actionCollection(), "options_show_macrobar");
    setViewMacroBar->setStatusText(i18n("Enables/disables the macro toolbar"));
    
    setSessionSettings = new KAction(i18n("configure &Session..."), "slsession.png", 0, this, SLOT(slotSessionSettings()), actionCollection(),"session_settings");
    setSessionSettings->setStatusText(i18n("Opens the session parameters dialog box"));

    viewProperties = new KAction(i18n("active &View properties"), "slview.png", 0, this, SLOT(slotChangeViewProperties()), actionCollection(), "view_properties");
    viewProperties->setStatusText(i18n("Opens the active view settings dialog"));
    
    //static nonstandard run menu (needs serlookui.rc) 
    //runMenu  = new KActionMenu(i18n("&Run"), actionCollection(), "run");
    runStop  = new KRadioAction(i18n("&Stop"), "slstop.png", 0, this, SLOT(slotRunStop()), actionCollection(),"run_stop");
    runStop->setStatusText(i18n("Stopping communication traffic acquisition"));
    runStop->setExclusiveGroup("run");
    runStop->setChecked(true);
    
    runRecord = new KRadioAction(i18n("&Record"), "slrecord.png" , 0, this, SLOT(slotRunRecord()), actionCollection(),"run_record");
    runRecord->setStatusText(i18n("Starting communication traffic acquisition"));
    runRecord->setExclusiveGroup("run");


    
    //
    //static nonstandard views menu
    //the new view...tile and cascade entries are fixed in serlookui.rc, while the
    //submenu with the list of the opened views is built dynamically 
    //(no XML serlookui.rc involved)
    
    viewMenu = new KActionMenu(i18n("&Views"), actionCollection(), "views");
    
    viewNewDumpView = new KAction(i18n("new &Dump view..."), "slnewdumpview.png" , 0, this, SLOT(slotViewNewDumpView()), actionCollection(),"view_new_dump_view");
    viewNewDumpView->setStatusText(i18n("Creates a new dump view in the current session"));
    
    viewNewTerminalView = new KAction(i18n("new t&Erminal view..."), "slnewtermview.png" , 0, this, SLOT(slotViewNewTerminalView()), actionCollection(),"view_new_terminal_view");
    viewNewTerminalView->setStatusText(i18n("Creates a new terminal view in the current session"));
    
    viewNewEventView = new KAction(i18n("new e&Vent log view..."), "slneweventview.png" , 0, this, SLOT(slotViewNewEventView()), actionCollection(),"view_new_event_view");
    viewNewEventView->setStatusText(i18n("Creates a new event log view in the current session"));
    
    
    viewTile = new KAction(i18n("&Tile"), 0, this, SLOT(slotViewTile()), actionCollection(),"view_tile");
    viewTile->setStatusText(i18n("Tiling views"));
    viewCascade = new KAction(i18n("&Cascade"), 0, this, SLOT(slotViewCascade()), actionCollection(),"view_cascade");
    viewCascade->setStatusText(i18n("Cascading views"));
    
    viewList = new KActionMenu(i18n("&Opened views"), actionCollection(), "opened_views");
    viewList->setStatusText(i18n("Displays the opened views submenu"));

    connect(viewList->popupMenu(), SIGNAL(aboutToShow()), this, SLOT(slotViewListAboutToShow()));

    //The XML file serlookui.rc must be placed into
    //share/apps/serlook otherwise the session menu won't be visible
    createGUI();
}

void SerLooKApp::initViewToolBar()
{    
    //BUG#1: TOOLBARS MOVING DOES NOT WORK, neither the following code can
    //enable this feature under kde2. Under mdk9 (kde3) the MDI skeleton generated
    //by kdevelop works ok. I must look in some kde apps sources to see how to
    //solve this problem
    //probably because

    toolBar( "mainToolBar" )->hide();

    toolBar( "baseToolBar" )->show();
    
}

void SerLooKApp::initMacroToolBar()
{
    macroToolBar = new KToolBar(this, Qt::DockLeft, false, "macroToolBar", true, false);
    CHECK_PTR( macroToolBar );
/*
    spostare nella slsession dopo aver definito
    il dialog box di configurazione dei tasti in
    slsessiondialog
*/    
    //Macro bar keys
    QString buttName,buttNr;
    for (int id=1; id <= TOT_MACROS; id++)
    {
        buttName.sprintf(i18n("macro key %u"),id);
        buttNr.sprintf(i18n("%u"),id);
        macroToolBar->insertButton("slfkey.png", id, false, buttName);
    }

    connect(macroToolBar, SIGNAL(clicked(int)), this, SLOT(slotMacroSend(int)));

}


void SerLooKApp::initStatusBar()
{
    //in the future the status bar should be enclosed
    //in a dedicated class

    
    QString sampleLine;
    int maxHeight;
    
    modeLabel = new QLabel( statusBar(), "modeLabel");
    CHECK_PTR( modeLabel );
    sampleLine = "Snoop";
    modeLabel->setMinimumWidth( modeLabel->fontMetrics().width( sampleLine ) );
    maxHeight = modeLabel->fontMetrics().height();
    modeLabel->setMaximumHeight( maxHeight );
    modeLabel->setAlignment( Qt::AlignHCenter | Qt::AlignVCenter );
    QToolTip::add( modeLabel, i18n("acquisition mode") );
    
    paramLabel = new QLabel( statusBar(), "paramLabel");
    CHECK_PTR( paramLabel );
    sampleLine = "115200,8,N,1,XANY/XOFF";
    paramLabel->setMinimumWidth( paramLabel->fontMetrics().width( sampleLine ) );
    paramLabel->setMaximumHeight( maxHeight );
    paramLabel->setAlignment( Qt::AlignHCenter | Qt::AlignVCenter );
    QToolTip::add( paramLabel, i18n("communication params") );

    priRXLabel = new QLabel( statusBar(), "priRXLabel");
    CHECK_PTR( priRXLabel );
    sampleLine = "PriRX:1234567";
    priRXLabel->setMinimumWidth( priRXLabel->fontMetrics().width( sampleLine ) );
    priRXLabel->setMaximumHeight( maxHeight );
    priRXLabel->setAlignment( Qt::AlignHCenter | Qt::AlignVCenter );
    QToolTip::add( priRXLabel, i18n("number of bytes received through the primary port") );


    priTXLabel = new QLabel( statusBar(), "priTXLabel");
    CHECK_PTR( priTXLabel );
    sampleLine = "PriTX:1234567";
    priTXLabel->setMinimumWidth( priTXLabel->fontMetrics().width( sampleLine ) );
    priTXLabel->setMaximumHeight( maxHeight );
    priTXLabel->setAlignment( Qt::AlignHCenter | Qt::AlignVCenter );
    QToolTip::add( priTXLabel, i18n("number of bytes sent through the primary port") );

    
    secRXLabel = new QLabel( statusBar(), "secRXLabel");
    CHECK_PTR( secRXLabel );
    sampleLine = "SecRX:1234567";
    secRXLabel->setMinimumWidth( secRXLabel->fontMetrics().width( sampleLine ) );
    secRXLabel->setMaximumHeight( maxHeight );
    secRXLabel->setAlignment( Qt::AlignHCenter | Qt::AlignVCenter );
    QToolTip::add( secRXLabel, i18n("number of bytes received through the secondary port") );

    lineLabel = new QLabel( statusBar(), "lineLabel");
    CHECK_PTR( lineLabel );
    sampleLine = "Line:1234567";
    lineLabel->setMinimumWidth( lineLabel->fontMetrics().width( sampleLine ) );
    lineLabel->setMaximumHeight( maxHeight );
    lineLabel->setAlignment( Qt::AlignHCenter | Qt::AlignVCenter );
    QToolTip::add( lineLabel, i18n("number of text lines (LF terminated) displayed in the active widget") );
    
    progress = new KProgress( statusBar(), "progress" );
    CHECK_PTR( progress );
    progress->setMaximumHeight( maxHeight );

    //progress->setPercentageVisible(false); ONLY KDE3
    //progress->setBarStyle(KProgress::Blocked); ONLY KDE2

    primaryLeds = new SLModemLeds( false, statusBar(), i18n( "primary port" ) );
    CHECK_PTR( primaryLeds );
    //QToolTip::add( primaryLeds, i18n("modem status leds, primary port") );
    primaryLeds->setMaximumHeight( maxHeight );

    secondaryLeds = new SLModemLeds( true, statusBar(), i18n( "secondary port" ) );
    CHECK_PTR( secondaryLeds );
    //QToolTip::add( secondaryLeds, i18n("modem status leds, secondary port") );
    secondaryLeds->setMaximumHeight( maxHeight );

    statusBar()->addWidget( progress, 0, true );
    statusBar()->addWidget( modeLabel, 0, true );
    statusBar()->addWidget( paramLabel, 0, true );
    statusBar()->addWidget( priTXLabel, 0, true );
    statusBar()->addWidget( priRXLabel, 0, true );
    statusBar()->addWidget( secRXLabel, 0, true );
    statusBar()->addWidget( lineLabel, 0, true );
    statusBar()->addWidget( primaryLeds, 0, true );
    statusBar()->addWidget( secondaryLeds, 0, true );

    //showed only with snooper mode
    secondaryLeds->hide();
    secRXLabel->hide();

    //normally hidden
    progress->hide();

    progressOwner = 0;
    
    readyMsg();
}


void SerLooKApp::initView()
{
    // here the main view of the KMainWindow is created by a background box and
    // the QWorkspace instance for MDI view.
    view_back = new QVBox( this );
    CHECK_PTR ( view_back );
    view_back->setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
    setCentralWidget(view_back);
    
}


void SerLooKApp::documentActionsEnabling()
{

    if (pDoc == 0)
    {
        // disables the widgets that works with session documents
        // when no session are opened 
        fileNew->setEnabled(true);
        fileOpenRecent->setEnabled(true);
        fileSend->setEnabled(false);
        fileCleanLogs->setEnabled(false);
        fileSaveAs->setEnabled(false);
        fileOpen->setEnabled(true);
        filePrint->setEnabled(false);
        fileSave->setEnabled(false);
        editCopy->setEnabled(false);
        editPaste->setEnabled(false);
        setSessionSettings->setEnabled(false);
        viewMenu->setEnabled(false);
        viewNewDumpView->setEnabled(false);
        viewNewTerminalView->setEnabled(false);
        viewNewEventView->setEnabled(false);
        viewProperties->setEnabled(false);
        runStop->setEnabled(false);
        runRecord->setEnabled(false);
        setGlobalPreferences->setEnabled(false);
        
        for (int i=1; i <= TOT_MACROS; i++)
        {
            macroToolBar->setItemEnabled( i, false );
        }
        
    }
    else
    {
        //session opened and no acquisition running
        fileNew->setEnabled(true);
        fileOpenRecent->setEnabled(true);
        fileSave->setEnabled(true);
        fileOpen->setEnabled(true);
        fileSaveAs->setEnabled(true);
        filePrint->setEnabled(true); 
        //filePrint->setEnabled(false);//TBD
        fileSend->setEnabled(false);
        fileCleanLogs->setEnabled(true);
        editCopy->setEnabled(true);
        editPaste->setEnabled(true);
        setSessionSettings->setEnabled(true);
        viewMenu->setEnabled(true);
        viewNewDumpView->setEnabled(true);
        viewNewTerminalView->setEnabled(true);
        viewNewEventView->setEnabled(true);
        viewProperties->setEnabled(true);
        viewCascade->setEnabled(true);
        viewTile->setEnabled(true);
        runStop->setEnabled(false);
        runRecord->setEnabled(true);
        //setGlobalPreferences->setEnabled(true);
        for (int i=1; i <= TOT_MACROS; i++)
        {
            macroToolBar->setItemEnabled( i, false );
        }
       
        if (acquisition == true)
        {
            fileNew->setEnabled(false);
            fileOpenRecent->setEnabled(false);
            fileSave->setEnabled(false);
            fileOpen->setEnabled(false);
            fileSaveAs->setEnabled(false);
            filePrint->setEnabled(false);
            fileCleanLogs->setEnabled(false);
            editCopy->setEnabled(true);
            editPaste->setEnabled(true);
            setSessionSettings->setEnabled(false);
            viewMenu->setEnabled(true);
            viewNewDumpView->setEnabled(false);
            viewNewTerminalView->setEnabled(false);
            viewNewEventView->setEnabled(false);
            viewProperties->setEnabled(false);
            viewCascade->setEnabled(true);
            viewTile->setEnabled(true);
            runStop->setEnabled(true);
            runRecord->setEnabled(false);
            setGlobalPreferences->setEnabled(false);

            SLSessionDialog* dlg = pDoc->getDialog();
            switch( dlg->getLineType() )
            {
                case SLSessionDialog::typeSerialP2P:
                    fileSend->setEnabled(true);
                    break;

                case SLSessionDialog::typeSerialTT:
                    fileSend->setEnabled(false);
                    break;
            }
            
            for (int i=1; i <= TOT_MACROS; i++)
            {
                macroToolBar->setItemEnabled( i, true );
            }

        }

        //Updates the views-related buttons
        slotViewActivated( pDoc->getActiveView() );
    }
}


void SerLooKApp::addClient( SLView* newView )
{
   
    // creates immediately the QWorkspaceChild for this view 
    QApplication::sendPostedEvents( pDoc->workspace(), QEvent::ChildInserted );
    
    SLView* currentView = pDoc->getActiveView();
    if (currentView != 0)
    {
         //Inheriting the position and size from the previous active view window
        
        QSize s = currentView->size();
        QPoint p = currentView->pos();
        
         // Avoids to overlap entirely the original view.
         //
         //I'd like to know how to dinamically detect the titlebar height
         //of a view in order to avoid to use these hardcoded constants:
          
        p.setX( p.x() + 20 );
        p.setY( p.y() + 20 );

        if(!s.isEmpty())
        {
            newView->resize( s );
        }

        newView->move( p );
    }
   
    newView->installEventFilter(this);
    newView->setIcon(kapp->miniIcon());
/*
    if ( pDoc->noViews() ) // show the very first view in maximized mode 
    {
        newView->showMaximized();
    }
*/
        
    connect( pDoc, SIGNAL( updateAllViews( BufferLine ) ), newView, SLOT( slotUpdateView( BufferLine ) ) );
    //kapp->processEvents();
    
    newView->show();
    newView->raise();
    newView->setActiveWindow();
    slotViewActivated( newView );
    pDoc->slotChangedViewList();
    
}


void SerLooKApp::openDocumentFile(const KURL& url)
{
    slotStatusMsg(i18n("Opening file..."));
    
    //Checks is file exists before trying to open it
    QString tmpfile;
    KIO::NetAccess::download( url, tmpfile );

    if ( !QFile::exists( tmpfile ) )
    {
        KMessageBox::sorry( kapp->mainWidget(), i18n( "The given session file does not exist: " ) + tmpfile );
        return;
    }

    /*
     * opening a session file requires to close the previous opened session,
     * if there is one.
     * A session can be closed only opening or creating another one
     *
     */ 
    
    if (pDoc != 0)
    {
        qDebug( "%s:%u - opening the given session",__FILE__,__LINE__ );
        
        // check, if the same session is already open. If yes, set the focus to the first view
        if (pDoc->URL() == url)
        {
            SLView* view = pDoc->firstView();
            view->setFocus();
            readyMsg();
            return;
        }
        
        // otherwise closes the active session doc before opening the new one
        queryClose();
        delete pDoc;
        pDoc = 0;

        documentActionsEnabling();
    }
      
    ASSERT(pDoc == 0);

    if(url.isEmpty())
    {
        //creates a new untitled session

        qDebug( "%s:%u - creating untitled session",__FILE__,__LINE__  );
        untitledCount++;
        
        QDir cwd;
        QString fileName=QString(i18n("Untitled%1.ser")).arg(untitledCount);
        QString pathName = cwd.cleanDirPath( cwd.absFilePath( fileName ) );
        
        KURL myUrl(pathName);
        pDoc = new SLSession(view_back, myUrl);
        CHECK_PTR (pDoc);
        
        setCaption(myUrl.fileName());
        emit documentLoaded();

        slotUpdateStatusBar();
   }
    else
    {
        //create the session...
        pDoc = new SLSession(view_back, url);
        CHECK_PTR (pDoc);

        //Automatic creation of views while loading session files:
        connect( pDoc, SIGNAL( createDumpView() ), this, SLOT( slotViewNewDumpView() ) );
        connect( pDoc, SIGNAL( createTerminalView() ), this, SLOT( slotViewNewTerminalView() ) );
        connect( pDoc, SIGNAL( createEventView() ), this, SLOT( slotViewNewEventView() ) );
        
        //...then it loads an already existing session file
        //and opens all the views described inside that
     
        initializing = true;
        
        if(!pDoc->loadDocument(url))
        {
            KMessageBox::error (this,i18n("Could not open session !"), i18n("Error !"));
            delete pDoc;
            pDoc = 0;
            documentActionsEnabling();
            slotStatusMsg(i18n("No active session"));
            return;    
        }
        
        //fileOpenRecent->addURL(url); already made in the slot
        setCaption(url.path());
        pDoc->slotChangedViewList();
        emit documentLoaded();
    }

    //Updating views and status bar while sending/receiving data:
    connect( pDoc, SIGNAL( updateAllViews( BufferLine ) ), this, SLOT( slotUpdateStatusBar() ) );
    connect( pDoc, SIGNAL( setProgressBarRange( int ) ), progress, SLOT( setTotalSteps(int) ) );
    connect( pDoc, SIGNAL( updateProgressBar( int, void* ) ), this, SLOT( slotUpdateProgressBar( int, void* ) ) );
    connect( pDoc, SIGNAL( updateStatusBar( const QString& ) ), this, SLOT( slotStatusMsg( const QString& ) ) );
    connect( pDoc, SIGNAL( updateLineAStatusLeds( uint ) ), primaryLeds, SLOT( slotSetStatus( uint ) ) );
    connect( pDoc, SIGNAL( updateLineBStatusLeds( uint ) ), secondaryLeds, SLOT( slotSetStatus( uint ) ) );


    //Applying session parameters changes to views and status bar:
    connect( pDoc->workspace(), SIGNAL( windowActivated( QWidget* ) ), this, SLOT( slotViewActivated( QWidget* ) ) );
    connect( pDoc->getDialog(), SIGNAL( commLineChoice() ), this, SLOT( slotUpdateStatusBar() ) );
    connect( pDoc->getDialog(), SIGNAL( commParamsChoice() ), this, SLOT( slotUpdateStatusBar() ) );

    //The buffers update the progress bar while printing
    connect( pDoc->getBuffer( Primary_Read ), SIGNAL( updateProgressBar( int, void* ) ), this, SLOT( slotUpdateProgressBar( int, void* ) ) );
    connect( pDoc->getBuffer( Primary_Write ), SIGNAL( updateProgressBar( int, void* ) ), this, SLOT( slotUpdateProgressBar( int, void* ) ) );
    connect( pDoc->getBuffer( Secondary_Read ), SIGNAL( updateProgressBar( int, void* ) ), this, SLOT( slotUpdateProgressBar( int, void* ) ) );
    
    documentActionsEnabling();
    pDoc->setModified( false );
    initializing = false;
    
    readyMsg();
}


void SerLooKApp::saveOptions()
{    
    qDebug( "%s:%u - saving options",__FILE__,__LINE__  );
    
    config->setGroup("General Options");
    config->writeEntry("Main Window Position", pos());
    config->writeEntry("Main Window Size", size());
    config->writeEntry("Show ToolBar", toolBar("baseToolBar")->isVisible());
    config->writeEntry("Show Statusbar",statusBar()->isVisible());
    config->writeEntry("ToolBarPos", (int) toolBar("baseToolBar")->barPos());

    config->writeEntry("Show MacroBar", macroToolBar->isVisible());
    config->writeEntry("MacroBarPos", (int) macroToolBar->barPos());


    fileOpenRecent->saveEntries(config,"Recent Files");


}


void SerLooKApp::readOptions()
{

    qDebug( "%s:%u - reading options: ",__FILE__,__LINE__  );
    
    config->setGroup("General Options");

    // main window geometry and position
    QSize s = config->readSizeEntry("Main Window Size");
    if(!s.isEmpty())
    {
        qDebug( "%s:%u - restoring window size",__FILE__,__LINE__  );
        resize( s );
    }
    
    QPoint p = config->readPointEntry("Main Window Position");

    qDebug( "%s:%u - restoring window position",__FILE__,__LINE__  );
    move( p );
    
    // tool bar status settings
    bool bViewToolbar = config->readBoolEntry("Show Toolbar", true);
    setViewToolBar->setChecked(bViewToolbar);

    qDebug( "%s:%u - restoring toolbar",__FILE__,__LINE__  );
    slotViewToolBar();

    bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
    setViewStatusBar->setChecked(bViewStatusbar);

    qDebug( "%s:%u - restoring status bar",__FILE__,__LINE__  );
    if( bViewStatusbar == false )
    {
        statusBar()->hide();
    }
    else
    {
        statusBar()->show();
    }

    // tool bar position settings
    KToolBar::BarPosition toolBarPos;
    toolBarPos=(KToolBar::BarPosition) config->readNumEntry("ToolBarPos", KToolBar::Top);

    qDebug( "%s:%u - restoring toolbar position",__FILE__,__LINE__ );
    toolBar("baseToolBar")->setBarPos(toolBarPos);

    // macro bar status settings
    bool bViewMacroBar = config->readBoolEntry("Show MacroBar", true);
    setViewMacroBar->setChecked(bViewMacroBar);
    qDebug( "%s:%u - restoring macrobar",__FILE__,__LINE__  );
    slotViewMacroBar();

    // macro bar position settings
    KToolBar::BarPosition macroBarPos;
    macroBarPos=(KToolBar::BarPosition) config->readNumEntry("MacroBarPos", KToolBar::Top);

    qDebug( "%s:%u - restoring macrobar position",__FILE__,__LINE__ );
    macroToolBar->setBarPos(macroBarPos);
                                                                           

    // initialize the recent file list
    qDebug( "%s:%u - restoring recent files list",__FILE__,__LINE__ );
    fileOpenRecent->loadEntries(config,"Recent Files");

}

/*
void SerLooKApp::saveProperties(KConfig *_cfg)
{
}


void SerLooKApp::readProperties(KConfig* _cfg)
{
}
*/

bool SerLooKApp::queryClose()
{

    QStringList saveFiles;
    if(pDoc == 0)
    {
        return true;
    }
    
    if(pDoc->isModified())
    {
        saveFiles.append(pDoc->URL().fileName());
    }
    
    if(!saveFiles.isEmpty())
    {
        switch (KMessageBox::questionYesNoList(this,
            i18n("The communication session have been modified.\nSave changes before closing?"),saveFiles))
        {
            case KMessageBox::Yes:
                if(pDoc->URL().fileName().contains(i18n("Untitled")))
                {
                    slotFileSaveAs();
                }
                else
                {
                    if(!pDoc->saveDocument(pDoc->URL()))
                    {
                        KMessageBox::error (this,i18n("Could not save the current session !"), i18n("I/O Error !"));
                        return false;
                    }
                }
                return true;
            
            case KMessageBox::No:
            default:
                return true;
        }
    }
    return true;
}

bool SerLooKApp::queryExit()
{
    saveOptions();
    return true;
}
bool SerLooKApp::eventFilter(QObject* object, QEvent* event)
{
    if (object->inherits("SLView") == true)
    {
        /* considers only the closing events from
         * SLView objects
         */

        SLView* pView = static_cast <SLView*>(object);
        switch(event->type())
        {
//The cases must be enclosed in separate blocks because of Q*Event *e initialization
            case QEvent::Close:
                {
                    QCloseEvent* e = static_cast <QCloseEvent*>(event);
                    if(pDoc->canCloseFrame())
                    {
                        //deletes the view
                        //true is returned to avoid
                        //further event propagation
                        //with obj null
                        pDoc->workspace()->closeActiveWindow();
                        delete pView;
                        pView = 0;
                        pDoc->slotChangedViewList();
                        e->accept();

                        return true;
                    }
                    else
                    {
                        e->ignore();
                    }
                }
                break;

            default:
                break;

        }
    }

    return QWidget::eventFilter( object, event );    // standard event processing
}



/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////


void SerLooKApp::slotFileNew()
{
    slotStatusMsg(i18n("Creating new session..."));

    KURL url;
    openDocumentFile(url);

    slotUpdateStatusBar();
    readyMsg();
}

void SerLooKApp::slotFileOpen()
{
    slotStatusMsg(i18n("Opening file..."));

    KURL url=KFileDialog::getOpenURL(QString::null,
      i18n("*.ser|SerLooK session files\n*|All files"), this, i18n("Open File..."));
    if(!url.isEmpty())
    {
        openDocumentFile(url);
        fileOpenRecent->addURL( url );
    }

    slotUpdateStatusBar();
    readyMsg();
}

void SerLooKApp::slotFileOpenRecent(const KURL& url)
{
    slotStatusMsg(i18n("Opening file..."));
      
    openDocumentFile(url);
    
    slotUpdateStatusBar();
    readyMsg();
}

void SerLooKApp::slotFileSave()
{
    slotStatusMsg(i18n("Saving file..."));

    if(pDoc->URL().fileName().contains(i18n("Untitled")))
    {
        slotFileSaveAs();
    }
    else if( !pDoc->saveDocument(pDoc->URL()))
    {
        KMessageBox::error (this,i18n("Could not save the current session !"), i18n("I/O Error !"));
    }

    slotUpdateStatusBar();
    readyMsg();
}

void SerLooKApp::slotFileSaveAs()
{
    slotStatusMsg(i18n("Saving file with a new filename..."));

    KURL url=KFileDialog::getSaveURL(":" /*QString::null*/ ,
        i18n("*.ser|SerLooK session files\n*|All files"), this, i18n("Save as..."));
    
    if(!url.isEmpty())
    {
        //automatic .ser extension
        QString name = url.fileName();
        if (name.right(4) != ".ser")
        {
            url.setFileName(name + ".ser");
        }
        
        if(!pDoc->saveDocument(url))
        {
            KMessageBox::error (this,i18n("Could not save the current session with that name!"), i18n("I/O Error !"));
            return;
        }

        delete pDoc;
        pDoc = 0;
        openDocumentFile(url);
        fileOpenRecent->addURL( url );
    }

    slotUpdateStatusBar();
    readyMsg();
}

void SerLooKApp::slotFileClose()
{
    slotStatusMsg(i18n("Closing file..."));
    if (queryClose() == true)
    {
        delete pDoc;
        pDoc = 0;
    }

    documentActionsEnabling();
    slotUpdateStatusBar();
    readyMsg();
}

void SerLooKApp::slotFilePrint()
{
    slotStatusMsg(i18n("Printing..."));

    SLView* m = pDoc->getActiveView();
    if ( m )
    {
        m->print( printer );
    }

    readyMsg();
}

void SerLooKApp::slotFileQuit()
{
    slotStatusMsg(i18n("Exiting..."));
    queryClose();
    saveOptions();

    if (pDoc != 0)
    {
        delete pDoc;
        pDoc = 0;
    }
    
    slotUpdateStatusBar();
    slotStatusMsg(i18n("Bye..."));
    QApplication::exit(0);

}

void SerLooKApp::slotFileSend()
{
    slotStatusMsg(i18n("Sending file..."));

    if (pDoc != 0)
    {
        KURL url=KFileDialog::getOpenURL(":" /*QString::null*/ ,
            i18n("*|All files"), this, i18n("Send file..."));
        
        if(!url.isEmpty())
        {
            pDoc->sendFile(url);
            emit documentLoaded();
        }
    }

    slotUpdateStatusBar();
    readyMsg();
}

void SerLooKApp::slotMacroSend(int id)
{
    QString tmp;
    tmp.sprintf( i18n("Pressed macro key %u"), id);
    slotStatusMsg(tmp);

    qDebug("%s:%u - slotMacroSend(), macro [%d]",
                __FILE__, __LINE__, id);
    slotUpdateStatusBar();
}



void SerLooKApp::slotFileCleanLogs()
{
    slotStatusMsg(i18n("Cleaning log files..."));
    if (pDoc != 0)
    {
        pDoc->slotCleanBuffers();
    }
    slotUpdateStatusBar();
    readyMsg();
}


void SerLooKApp::slotEditUndo()
{
    slotStatusMsg(i18n("Reverting last action..."));
    
    SLView* m = pDoc->getActiveView();
    if ( m )
    {
        //m->undo();
    }

    readyMsg();
}

void SerLooKApp::slotEditCut()
{
    slotStatusMsg(i18n("Cutting selection..."));

    SLView* m = pDoc->getActiveView();
    if ( m )
    {
        m->cut();    
    }
    readyMsg();
}

void SerLooKApp::slotEditCopy()
{
    slotStatusMsg(i18n("Copying selection to clipboard..."));

    SLView* m = pDoc->getActiveView();
    if ( m )
    {
        m->copy();
    }
    readyMsg();
}

void SerLooKApp::slotEditPaste()
{
    slotStatusMsg(i18n("Inserting clipboard contents..."));

    SLView* m = pDoc->getActiveView();
    if ( m )
    {
        m->paste();
    }
        
    readyMsg();
}

void SerLooKApp::slotViewToolBar()
{
    slotStatusMsg(i18n("Toggle the base toolbar..."));
    ///////////////////////////////////////////////////////////////////
    // turn Toolbar on or off
    if(!setViewToolBar->isChecked())
    {
        toolBar("baseToolBar")->hide();
    }
    else
    {
        toolBar("baseToolBar")->show();
    }        

    readyMsg();
}

void SerLooKApp::slotViewMacroBar()
{
    slotStatusMsg(i18n("Toggle the macro toolbar..."));
    ///////////////////////////////////////////////////////////////////
    // turn Macrobar on or off
    if(!setViewMacroBar->isChecked())
    {
        macroToolBar->hide();
    }
    else
    {
        macroToolBar->show();
    }

    readyMsg();
}


void SerLooKApp::slotViewStatusBar()
{
    slotStatusMsg(i18n("Toggle the statusbar..."));

    if(!setViewStatusBar->isChecked())
    {
        statusBar()->hide();
    }
    else
    {
        statusBar()->show();
    }
    pDoc->setModified( true );
    readyMsg();
}

void SerLooKApp::slotSessionSettings()
{
    slotStatusMsg(i18n("Session parameters dialog box"));
    pDoc->getDialog()->show();
    pDoc->setModified( true );
    readyMsg();
}


void SerLooKApp::slotGlobalPreferences()
{
    slotStatusMsg(i18n("Global parameters dialog box"));


    pDoc->setModified( true );
    readyMsg();
}

void SerLooKApp::slotChangeViewProperties()
{
    
    SLView* v = static_cast<SLView*>( pDoc->workspace()->activeWindow() );
    if ( v != 0 )
    {
        slotStatusMsg(i18n("Editing current view settings."));
        v->slotPropsDialog();
    }
}

void SerLooKApp::slotRunRecord()
{
    slotStatusMsg(i18n("Starting communication traffic acquisition"));
    pDoc->slotStartAcquisition();
    acquisition = true;    
    documentActionsEnabling();
    pDoc->setModified( true );
    slotStatusMsg(i18n("Acquisition..."));
}

void SerLooKApp::slotRunPlay()
{
}


void SerLooKApp::slotRunStop()
{
    slotStatusMsg(i18n("Stopping communication traffic acquisition"));
    pDoc->slotStopAcquisition();
    acquisition = false;
    documentActionsEnabling();
    pDoc->setModified( true );
    readyMsg();
}


void SerLooKApp::slotViewNewDumpView()
{
    slotStatusMsg(i18n("Opening a new dump view..."));
    CHECK_PTR( pDoc );
   
    SLDumpView* v = new SLDumpView( pDoc, Primary_Read );
    CHECK_PTR( v );
    

    addClient(v);

    QPixmap p = viewNewDumpView->iconSet().pixmap(QIconSet::Small, QIconSet::Normal);
    v->setIcon(p);
        
    pDoc->setModified( true );
    
    connect(this, SIGNAL( documentLoaded() ), v, SLOT( slotDocumentLoaded() ));
    connect(v, SIGNAL( updateProgressBar( int, void* ) ), this, SLOT( slotUpdateProgressBar( int, void* ) ) );
    connect(v, SIGNAL( cursorChanged() ), this, SLOT( slotUpdateStatusBar() ));
    
    if (initializing == false)
    {
        //session already loaded
        emit documentLoaded();
    }
    
    readyMsg();
}

void SerLooKApp::slotViewNewTerminalView()
{
    slotStatusMsg(i18n("Opening a new terminal view..."));
    CHECK_PTR( pDoc );
   
    SLTerminalView* v = new SLTerminalView( pDoc, Primary_Read );
    CHECK_PTR( v );
 
    addClient(v);
    
    QPixmap p = viewNewTerminalView->iconSet().pixmap(QIconSet::Small, QIconSet::Normal);
    v->setIcon(p);

    pDoc->setModified( true );
   
    connect(this, SIGNAL( documentLoaded() ), v, SLOT( slotDocumentLoaded() ));
    connect(v, SIGNAL( updateProgressBar( int, void* ) ), this, SLOT( slotUpdateProgressBar( int, void* ) ) );
    connect(v, SIGNAL( cursorChanged() ), this, SLOT( slotUpdateStatusBar() ));
   
    if (initializing == false)
    {
        //session already loaded
        emit documentLoaded();
    }
    
    readyMsg();
}

void SerLooKApp::slotViewNewEventView()
{
    slotStatusMsg(i18n("Opening a new event log view..."));
    CHECK_PTR( pDoc );
   
    SLEventView* v = new SLEventView( pDoc );
    CHECK_PTR( v );
 
    addClient(v);
    
    QPixmap p = viewNewEventView->iconSet().pixmap(QIconSet::Small, QIconSet::Normal);
    v->setIcon(p);

    pDoc->setModified( true );
   
    connect(this, SIGNAL( documentLoaded() ), v, SLOT( slotDocumentLoaded() ));
    connect(v, SIGNAL( updateProgressBar( int, void* ) ), this, SLOT( slotUpdateProgressBar( int, void* ) ) );
    connect(v, SIGNAL( cursorChanged() ), this, SLOT( slotUpdateStatusBar() ));
   
    if (initializing == false)
    {
        //session already loaded
        emit documentLoaded();
    }
    
    readyMsg();
}

    
void SerLooKApp::slotViewTile()
{
    pDoc->workspace()->tile();
    pDoc->setModified( true );
}

void SerLooKApp::slotViewCascade()
{
    pDoc->workspace()->cascade();
    pDoc->setModified( true );

}


void SerLooKApp::slotViewActivated(QWidget* w)
{
    
    SLView* pView = 0;
    
    if (w != 0)
    {
        pView = static_cast<SLView*>(w);
    }
    else
    {
        return;
    }


    pView->slotUpdateView( pView->getLine() );

    pDoc->slotChangedViewList();
    slotUpdateStatusBar();
}

    
void SerLooKApp::slotUpdateStatusBar()
{
    if (pDoc != 0)
    {
        QString buf;
        SLSessionDialog* dlg = pDoc->getDialog();
        SLView* active = pDoc->getActiveView();
        uint offset = 0;
        uint lineNr = 0;
        
        if ( active != 0 )
        {
            SLHexBuffer* curr = pDoc->getBuffer( active->getLine() );

            if ( curr != 0 )
            {
                offset = curr->documentSize();
                lineNr = curr->getLF( offset );
            }
        }
        
        switch( dlg->getLineType() )
        {
            case SLSessionDialog::typeSerialP2P:
                modeLabel->setText( "PtP" );
                paramLabel->setText( dlg->getSyntheticParams().latin1() );
                
                buf.sprintf( "PriTX:%u", pDoc->getBuffer(Primary_Write)->documentSize() );
                priTXLabel->setText( buf );
                
                buf.sprintf( "PriRX:%u", pDoc->getBuffer(Primary_Read)->documentSize() );
                priRXLabel->setText( buf );
                
                buf.sprintf( "Line:%u", lineNr );
                lineLabel->setText( buf );
                
                secondaryLeds->hide();
                secRXLabel->hide();
                priTXLabel->show();
                break;

            case SLSessionDialog::typeSerialTT:
                modeLabel->setText( "Snoop" );
                paramLabel->setText( dlg->getSyntheticParams().latin1() );
                
                buf.sprintf( "PriRX:%u", pDoc->getBuffer(Primary_Read)->documentSize() );
                priRXLabel->setText( buf );
                
                buf.sprintf( "SecRX:%u", pDoc->getBuffer(Secondary_Read)->documentSize() );
                secRXLabel->setText( buf );
                
                buf.sprintf( "Line:%u", lineNr );
                lineLabel->setText( buf );
                
                secondaryLeds->show();
                secRXLabel->show();
                priTXLabel->hide();
                break;

        }
    }

}


    
void SerLooKApp::slotStatusMsg(const QString &text)
{
    statusBar()->message(text);
    kapp->processEvents();
}

void SerLooKApp::readyMsg()
{
    if ( initializing == false )
    {
        kapp->processEvents();
        statusBar()->message( i18n( "Ready." ));
    }
    //QApplication::restoreOverrideCursor();
}




void SerLooKApp::slotViewMenuActivated( int id )
{
    QWidget* w = pDoc->workspace()->windowList().at( id );
    if ( w )
    {
        w->setFocus();
    }
}

void SerLooKApp::slotViewListAboutToShow()
{
    KPopupMenu* m = viewList->popupMenu();

    
    //rebuild the view list
    m->clear();

    QWidgetList views = pDoc->workspace()->windowList();
    for ( int i = 0; i < int(views.count()); ++i )
    {
        int id = m->insertItem(
                QString("&%1 ").arg(i+1)+views.at(i)->caption(), 
                this, 
                SLOT( slotViewMenuActivated( int ) ) );
        
        qDebug("%s:%u - added view (%d), id=%d",__FILE__,__LINE__, i,  id);

        m->setItemParameter( id, i );
        m->setItemChecked( id, pDoc->workspace()->activeWindow() == views.at(i) );
    }
}

void SerLooKApp::slotUpdateProgressBar(int value, void* requester)
{
    if ( progressOwner == 0 && requester != 0 && value != -1 )
    {
        //changing bar owner
        progressOwner = static_cast<QObject*> (requester);
        qDebug( "%s:%u - Changing progress bar owner to %s:%p ...",__FILE__,__LINE__, 
                progressOwner->className(), (void*)progressOwner ); 
        
    }
            
    if ( value == -1 && (requester == progressOwner || progressOwner == 0 ))
    {
        //the owner releases the bar
        qDebug( "%s:%u - Progress bar released by %s:%p",__FILE__,__LINE__,  progressOwner->className(), (void*)progressOwner );
        progressOwner = 0;
        value = 0;
    }

    //Once an object takes the progress bar control, it becomes 
    //the unique that can update it, until the release (value == -1)
    if ( requester == progressOwner || progressOwner == 0 )
    {
        progress->setValue(value);
        if (value == 0)
        {
            progress->hide();
        }
        else
        {
            progress->show();
        }
        kapp->processEvents();
    }
}

    
