/*
 *                        serlook.h
 *                           -------------------
 *  begin                : mer dic  4 22:54:15 CET 2002
 *  copyright            : (C) 2002 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB.h> skips=4
 *  <TAB.h> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * $Id: serlook.h,v 1.13 2006/01/19 10:19:12 gbertani Exp $ 
 */


#ifndef SERLOOK_H
#define SERLOOK_H
 

// include files for Qt
#include <qdir.h>
#include <qprinter.h>
#include <qtooltip.h>
#include <qvbox.h>
#include <qwhatsthis.h>
#include <qtooltip.h>
#include <qtoolbutton.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qstrlist.h>
#include <qworkspace.h>
#include <qlabel.h>
#include <qcombobox.h>

// include files for KDE
#include <kapp.h>
#include <kmainwindow.h>
#include <kstatusbar.h>
#include <kaction.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <klocale.h>
#include <kconfig.h>
#include <kstdaction.h>
#include <kstddirs.h>
#include <kpopupmenu.h>
#include <kprogress.h>
#include <ktoolbarbutton.h>

#include "khexedit/hexprinter.h"

//overriding the definition in config.h
//#undef VERSION
//#define VERSION "0.3.2"

//session file .ser format version
#define SER_FILE_VER 9

//default macro keys 
#define TOT_MACROS  20


// forward declaration of the SerLooK classes
class SLSession;
class SLView;
class SLModemLeds;

/**
  * The base class for SerLooK application windows. It sets up the main
  * window and reads the config file as well as providing a menubar, toolbar
  * and statusbar. In initView(), your main view is created as the MDI child view manager.
  * Child views are created in createClient(), which gets a document instance as it's document to
  * display whereby one document can have several views.The MDI child is an instance of SLView,
  * the document an instance of SLSession.
  * SerLooKApp reimplements the methods that KTMainWindow provides for main window handling and supports
  * full session management as well as keyboard accelerator configuration by using KAccel.
  * @see KMainWindow
  * @see KApplication
  * @see KConfig
  * @see KAccel
  *
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
  * @version KDevelop version 1.3 code generation
  */
class SerLooKApp : public KMainWindow
{
  Q_OBJECT

public:
    /** constructor of SerLooKApp, calls all init functions to create the application.
     * @see initMenuBar initToolBar
     */
    SerLooKApp();
    ~SerLooKApp();
    
    /** opens a file specified by commandline option
     */
    void openDocumentFile(const KURL& url); //attempt to fix bug #4 #5

    /** default window size */
    QSize sizeHint() const
    {
        return QSize(640, 480);
    }


private:
    /** queryClose is called by KTMainWindow on each closeEvent of a view. Against the
     * default implementation (only returns true), this overridden function retrieves all modified documents
     * from the open document list and asks the user to select which files to save before exiting the application.
     * @see KTMainWindow#queryClose
     * @see KTMainWindow#closeEvent
     */
    virtual bool queryClose();
    /** queryExit is called by KTMainWindow when the last view of the application is going to be closed during the closeEvent().
     * Against the default implementation that just returns true, this calls saveOptions() to save the settings of the last view's    
     * properties.
     * @see KTMainWindow#queryExit
     * @see KTMainWindow#closeEvent
     */
    virtual bool queryExit();

    /** saves the view properties for each open view during session end to the session config file, including saving the currently
     * opened file by a temporary filename provided by KApplication.
     * @see KTMainWindow#saveProperties
     */
    //virtual void saveProperties(KConfig *_cfg);

    /** reads the session config file and restores the application's state including the last opened files and documents by reading the
     * temporary files saved by saveProperties()
     * @see KTMainWindow#readProperties
     */
    //virtual void readProperties(KConfig *_cfg);

    
    /** event filter to catch close events for MDI child views and is installed in createClient() on every child view.
     * Closing a view calls the eventFilter first which removes the view from the connected documents' view list. If the
     * last view is going to be closed, the eventFilter() tests if the document is modified; if yes, it asks the user to
     * save the document. If the document title contains "Untitled", slotFileSaveAs() gets called to get a save name 
     * and path.
     *
     * GMB:
     * SerLooK use it in a different way, because the data showed in the view 
     * are always
     * recorded. The real "document" managed by SerLooK is the session, that
     * is not showed in a view and only one session at once can be opened. The
     * session remains alive also without views.
     */
    virtual bool eventFilter(QObject* object, QEvent* event);
    
    /** 
     * adds a new child view in the session
     */
    void addClient( SLView* newView );


    /** save general Options like all bar positions and status as well as the geometry and the recent file list to the configuration
     * file
     */     
    void saveOptions();

    /** read general Options again and initialize all variables like the recent file list
     */
    void readOptions();

    /** initializes the KActions of the application */
    void initActions();

    /** sets up the statusbar for the main view by initialzing a statuslabel.
     */
    void initStatusBar();

    /** Creates the main view of the KTMainWindow instance and initializes the MDI view area including any needed
        * connections.
     */
    void initView();

    /** Creates the view toolbar (uses XML GUI framework)*/
    void initViewToolBar();

    /** Creates the macro toolbar (no XML GUI) */
    void initMacroToolBar();

    
    /** enable/disable the document-related actions and toolbars */
    void documentActionsEnabling();

    /** the configuration object of the application */
    KConfig *config;
    
    /** the printer instance */
    CHexPrinter *printer;
   
    /** client area base */
    QVBox* view_back;

    
    /** a counter that gets increased each time the user creates a new document with "File"-.h>"New" */
    int untitledCount;

    /** this flag is true while loading a session from file */
    bool initializing;

    /** true while acquiring data from the serial lines */
    bool acquisition;
    
    /** communication session currently open */
    SLSession* pDoc;    

    /** progress bar owner object */
    QObject* progressOwner;

    // File menu KActions
    KAction*            fileNew;
    KAction*            fileOpen;
    KRecentFilesAction* fileOpenRecent;
    KAction*            fileSave;
    KAction*            fileSaveAs;
    KAction*            filePrint;
    KAction*            fileQuit;
    KAction*            fileSend;
    KAction*            fileCleanLogs;
    
    // Edit menu KActions
    KAction*            editCopy;
    KAction*            editPaste;
    
    // View menu KActions
    KActionMenu*        viewMenu;
    KAction*            viewNewDumpView;
    KAction*            viewNewTerminalView;
    KAction*            viewNewEventView;
    KAction*            viewProperties;
    KAction*            viewTile;
    KAction*            viewCascade;
    KActionMenu*        viewList;
   
    // Settings menu KActions
    KToggleAction*      setViewToolBar;
    KToggleAction*      setViewMacroBar;
    KToggleAction*      setViewStatusBar;
    KAction*            setGlobalPreferences;
    KAction*            setSessionSettings;

    // Run menu KActions
    //KActionMenu*        runMenu;
    KRadioAction*       runRecord;
    KRadioAction*       runStop;

    // Help menu KActions
    KAction*            helpContents;

    KToolBar*           macroToolBar;
    
    //Status bar items
    QLabel*             modeLabel;
    QLabel*             paramLabel;
    QLabel*             priRXLabel;
    QLabel*             priTXLabel;
    QLabel*             secRXLabel;
    QLabel*             lineLabel;
    KProgress*          progress;
    SLModemLeds*        primaryLeds;
    SLModemLeds*        secondaryLeds;


signals:
    /** */
    void documentLoaded();


private slots:
    /** clears the document in the actual view to reuse it as the new document */
    void slotFileNew();

    /** open a file and load it into the document*/
    void slotFileOpen();
    
    /** opens a file from the recent files menu */
    void slotFileOpenRecent(const KURL& url);
    
    /** save a document */
    void slotFileSave();
    
    /** save a document by a new filename*/
    void slotFileSaveAs();
    
    /** asks for saving if the file is modified, then closes the actual file and view*/
    void slotFileClose();
    
    /** print the actual file */
    void slotFilePrint();
    
    /** closes all documents and quits the application.*/
    void slotFileQuit();
    
    /** sends a binary file */
    void slotFileSend();

    /** sends a binary string (macro) */
    void slotMacroSend(int id);
    
    /** clean all the logfiles of the active session */
    void slotFileCleanLogs();
    
    /** reverts the last user action for the active view */
    void slotEditUndo();
    
    /** put the marked text/object into the clipboard and remove
     *    it from the document
     */
    void slotEditCut();
    
    /** put the marked text/object into the clipboard
     */
    void slotEditCopy();
    
    /** paste the clipboard into the document
     */
    void slotEditPaste();
    
    /**
     * Opens the session configuration dialog box
     */ 
    void slotSessionSettings();
   
    /**
     *
     */
    void slotGlobalPreferences();
    
    /** called when pressing RX/TX or numerical base buttons 
     * in the toolbar */
    void slotChangeViewProperties();
        
    /** toggles the toolbar
     * 
     */
    void slotViewToolBar();

    /** toggles the macrobar
     *
     */
    void slotViewMacroBar();

            
    /** toggles the statusbar
     */
    void slotViewStatusBar();
    
    /** 
     * updates status bar 
     */
    void slotUpdateStatusBar();
    
    /** 
     * updates progress bar 
     */
    void slotUpdateProgressBar(int value, void* requester);
    
    /**
     * Starts communication traffic acquisition
     */ 
    void slotRunRecord();

    /**
     * Stops communication traffic acquisition
     */ 
    void slotRunStop();
    
    /**
     * 
     * Starts communication traffic replay
     */ 
    void slotRunPlay();

    /** gets called when the view list submenu is activated; 
     * recreates the submenu with all opened view titles. */
    void slotViewListAboutToShow();
    
    /** activates the MDI child widget when it gets selected from the view menu. */
    void slotViewMenuActivated( int id );
    
    /** */
    void slotViewNewDumpView();
    
    /** */
    void slotViewNewTerminalView();
    
    /** */
    void slotViewNewEventView();
    
    /**  */
    void slotViewTile();
    /**  */
    void slotViewCascade();

    /** 
     * Changes the statusbar contents for the standard label permanently, used to indicate current actions.
     * Please call readyMsg() to display "Ready."
     * @param text the text that is displayed in the statusbar
     */
    void slotStatusMsg(const QString &text);
    
    /** to be called when changing the focus
     * between the views to update the 
     * viewToolbar
     */ 
    void slotViewActivated(QWidget* w);

    /**
     * sometimes the menu commands slots are called
     * directly (for example while loading the session file
     * the views are recreated by calling directly the slot
     * functions). This causes an action nesting.
     * To avoid displaying "ready" on the status bar
     * too early, this function checks the nesting
     * counter and displays "ready" only when it reaches
     * the zero
     */ 
    void readyMsg();
};




#endif // SERLOOK_H
