/*
 *                        sldumpdialog.cpp
 *                           -------------------
 *  begin                : gio ago 28 2003
 *  copyright            : (C) 2003 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * 
 */
static char* cvs_id = "$Id";


#include "sldumpdialog.h"

SLDumpDialog::SLDumpDialog(QWidget *parent, const char *name) 
    : SLViewDialogBase(i18n("Dump view properties"), parent, name, false)
{
    setupSourceBufferPage();
    setupDataFormatPage();
}

SLDumpDialog::~SLDumpDialog()
{
}


void SLDumpDialog::setupSourceBufferPage()
{

    QFrame* page = addPage( i18n( "Source" ), i18n( "Data source selection" ));
    QGridLayout* myLayout = new QGridLayout( page ); 
    myLayout->setSpacing(10);
    
//row 0
    mSourceBuffer.sourceBufferLabel = new QLabel( page );
    mSourceBuffer.sourceBufferLabel->setText( i18n( "Source of the data displayed in the view" ) );
    myLayout->addWidget( mSourceBuffer.sourceBufferLabel, 0, 0 );


    mSourceBuffer.sourceBufferSelector = new QComboBox( false, page );
    mSourceBuffer.sourceBufferSelector->insertItem( i18n( "Input from primary line" ), Primary_Read );
    mSourceBuffer.sourceBufferSelector->insertItem( i18n( "Input from secondary line" ), Secondary_Read );
    mSourceBuffer.sourceBufferSelector->insertItem( i18n( "Output to primary line" ), Primary_Write );
    myLayout->addWidget( mSourceBuffer.sourceBufferSelector, 0, 1 );

//row 1
    mSourceBuffer.syncViewLabel = new QLabel( page );
    mSourceBuffer.syncViewLabel->setText( i18n( "This view accepts synchronization signals" ) );
    myLayout->addWidget( mSourceBuffer.syncViewLabel, 1, 0 );

    mSourceBuffer.syncViewBox = new QCheckBox( page );
    myLayout->addWidget( mSourceBuffer.syncViewBox, 1, 1 );

    int idx = pageIndex( page );
    qDebug( "%s:%u - dump dialog source buffer page %p index: %d", __FILE__, __LINE__, (void*)page, idx );

}


void SLDumpDialog::setupDataFormatPage()
{

    QFrame* page = addPage( i18n( "Format" ), i18n( "Displayed data format" ));
    QGridLayout* myLayout = new QGridLayout( page ); 
    myLayout->setSpacing(10);
    
//row 0
    mDataFormat.dataFormatLabel = new QLabel( page );
    mDataFormat.dataFormatLabel->setText( i18n( "Displayed data format" ) );
    myLayout->addWidget( mDataFormat.dataFormatLabel, 0, 0 );

    mDataFormat.dataFormatSelector = new QComboBox( false, page );
    mDataFormat.dataFormatSelector->insertItem( i18n( "Hexadecimal" ),    Hexadecimal);
    mDataFormat.dataFormatSelector->insertItem( i18n( "Decimal" ),        Decimal);
    mDataFormat.dataFormatSelector->insertItem( i18n( "Octal" ),          Octal);
    mDataFormat.dataFormatSelector->insertItem( i18n( "Binary" ),         Binary);
    mDataFormat.dataFormatSelector->insertItem( i18n( "7 bit ASCII" ),    Ascii);
    myLayout->addWidget( mDataFormat.dataFormatSelector, 0, 1 );

//row 1
    mDataFormat.offsetFormatLabel = new QLabel( page );
    mDataFormat.offsetFormatLabel->setText( i18n( "Display offsets in hex instead of decimal" ) );
    myLayout->addWidget( mDataFormat.offsetFormatLabel, 1, 0 );

    mDataFormat.offsetFormatBox = new QCheckBox( page );
    myLayout->addWidget( mDataFormat.offsetFormatBox, 1, 1 );

    
    int idx = pageIndex( page );
    qDebug( "%s:%u - dump dialog data format page: %p index: %d", __FILE__, __LINE__, (void*)page, idx );

}

void SLDumpDialog::slotAllDefaults()
{
    mSourceBuffer.sourceBufferSelector->setCurrentItem(Primary_Read);
    mSourceBuffer.syncViewBox->setDown( true );
    mDataFormat.dataFormatSelector->setCurrentItem(Hexadecimal);
    mDataFormat.offsetFormatBox->setDown( false );
}

void SLDumpDialog::slotDefault()
{
    switch( activePageIndex() )
    {
            
        case pageSourceBuffer:
            mSourceBuffer.sourceBufferSelector->setCurrentItem(Primary_Read);
            mSourceBuffer.syncViewBox->setDown( true );
            break;
            
        case pageDataFormat:
            mDataFormat.dataFormatSelector->setCurrentItem(Hexadecimal);
            mDataFormat.offsetFormatBox->setDown( false );
            break;
            
        default:
            break;
    }
    
}

void SLDumpDialog::slotApply()
{
    SLViewDialogBase::slotApply();
    
    switch( activePageIndex() )
    {
        case pageDataFormat:
            emit dataFormatChoice();
            break;
            
        default:
            break;
    }
}

SLDumpDialog::DataFormat SLDumpDialog::getDataFormat() const
{
    return (DataFormat) mDataFormat.dataFormatSelector->currentItem();
}

void SLDumpDialog::setDataFormat( DataFormat f )
{
    mDataFormat.dataFormatSelector->setCurrentItem( f );
}

bool SLDumpDialog::getOffsetFormat() const
{
    return (DataFormat) mDataFormat.offsetFormatBox->isChecked();
}

void SLDumpDialog::setOffsetFormat( bool f )
{
    mDataFormat.offsetFormatBox->setDown( f );
}


void SLDumpDialog::save(KConfig* data)
{
    SLViewDialogBase::save(data);
    data->writeEntry("Data Format", getDataFormat());
    data->writeEntry("Offset Format", getOffsetFormat());
}

void SLDumpDialog::load(KConfig const* data)
{
    setDataFormat( (SLDumpDialog::DataFormat) data->readUnsignedNumEntry("Data Format") );
    setOffsetFormat( data->readBoolEntry("Offset Format") );
    emit dataFormatChoice();
    SLViewDialogBase::load(data);
}






