/*
 *                        sldumpview.cpp
 *                           -------------------
 *  begin                : Tue Apr 15 2003
 *  copyright            : (C) 2003 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * 
 */

static char* cvs_id = "$Id: sldumpview.cpp,v 1.12 2005/01/21 08:40:58 gbertani Exp $";

#include <iostream.h>

#include "slsession.h"
#include "slview.h"
#include "slsessiondialog.h"
#include "slhexviewwidget.h"
#include "slhexbuffer.h"
#include "sldumpdialog.h"
#include "sldumpview.h"


SLDumpView::SLDumpView(SLSession* const pDoc, BufferLine line) :
    SLView(pDoc, line)   
{
    qDebug( "%s:%u - building sldumpview", __FILE__, __LINE__ );  
    //----- viewer-------------
   
    QHBoxLayout* layout = new QHBoxLayout(this); 
  
    bool ro = (line != Primary_Write);
    
    view = new SLHexViewWidget( this, 0, doc->getBuffer( currentLine ), ro );
    CHECK_PTR( view );

    layout->addWidget( view );

    props = new SLDumpDialog(this);
    CHECK_PTR( props );
    baseProps = props;

    isOffsetHex = true;
           
    //default props
    props->setSourceBuffer( currentLine );
    props->setDataFormat( SLDumpDialog::Hexadecimal );

    
    connect(view, SIGNAL( cursorChanged( SCursorState& )), this, SLOT( slotCursorChanged( SCursorState& ) ));
    connect(view, SIGNAL( scrollBarChanged(uint)), this, SLOT( slotScrollBarChanged(uint) ));
    connect(view, SIGNAL( updateBuffer( SLHexBuffer* ) ), doc, SLOT( slotUpdateOutputBuffer( SLHexBuffer* ) ));
    connect(view, SIGNAL( mouseRightClick() ), this, SLOT( slotPropsDialog() ));
    
    connect(doc,  SIGNAL( synchronizeViews( const SLView*, BufferLine, uint )), this, SLOT( slotGoToOffset( const SLView*, BufferLine, uint ) ));
    connect(props, SIGNAL( sourceBufferChoice() ), this, SLOT( slotChangeBuffer() ));
    connect(props, SIGNAL( dataFormatChoice() ), this, SLOT( slotChangeDataFormat() ));
    connect(doc,  SIGNAL( selectChunk( BufferLine, uint, uint )), this, SLOT( slotSelectChunk( BufferLine, uint, uint ) ));

    qDebug( "%s:%u - sldumpview ok", __FILE__, __LINE__  );

    ready = false;
    isSyncing = false;
}

SLDumpView::~SLDumpView()
{
    qDebug( "%s:%u - destroying SLDumpView", __FILE__, __LINE__   );
    delete props;
    delete view;
}



void SLDumpView::cut()
{
    view->cut();    
}


void SLDumpView::copy()
{
    view->copy();    
}

void SLDumpView::paste()
{
    view->paste();    
}


void SLDumpView::save(KConfig* data)
{
    data->writeEntry("Class", QString("SLDumpView") );
    props->save(data);
    SLView::save(data);
}

void SLDumpView::load(KConfig const* data)
{
    //The "Class" field is already read
    //by SLSession

    SLView::load(data);
    props->load(data);

}

void SLDumpView::slotUpdateView(BufferLine line)
{    
    if (line != currentLine)
    {
        return;
    }


    if (view != 0)
    {
        //--------------( redraw, fixCursor )
        view->updateView( true, false );
    }

}

bool SLDumpView::slotChangeDataFormat() 
{ 
    
    SLDumpDialog::DataFormat b = props->getDataFormat();

    switch (b) 
    { 
        case SLDumpDialog::Hexadecimal:
            view->setBase( "hexadecimal" );
            view->setBytesPerLine(16);
            isOffsetHex = true;
            break;
            
        case SLDumpDialog::Decimal: 
            view->setBase( "decimal" ); 
            view->setBytesPerLine(10);
            isOffsetHex = false;
            break;
    
        case SLDumpDialog::Octal: 
            view->setBase( "octal" ); 
            view->setBytesPerLine(8); 
            isOffsetHex = true;
            break;
        
        case SLDumpDialog::Binary: 
            view->setBase( "binary" ); 
            view->setBytesPerLine(8);
            isOffsetHex = true;
            break;
    
        case SLDumpDialog::Ascii: 
            view->setBase( "textOnly" ); 
            view->setBytesPerLine(64);
            isOffsetHex = false;
            break;

        default: 
            return false; 
    }
    view->setOffsetBase( props->getOffsetFormat() );

    
    view->updateView( true, true );
    return true;   
}

void SLDumpView::print(CHexPrinter *pPrinter)
{
    if (pPrinter->setup( this ))
    {
        view->print( *pPrinter );
    }

}


void SLDumpView::setCaption( const QString& num, const QString& dev, 
    const QString& enable ) 
{
    QString caption = "(" + num + ") " + dev + " ";
    
    switch( currentLine )
    {
        case Primary_Read:
            caption += i18n("Read-Only") + enable;
            break;

        case Primary_Write:
            caption += i18n("Write-Only") + enable;
            break;
            
        case Secondary_Read:
            caption += i18n("Read-Only") + enable;
            break;
    }

    QWidget::setCaption(caption);
    
}

void SLDumpView::slotChangeBuffer()
{
    BufferLine line = props->getSourceBuffer();

    //view->clear();
    currentLine = line;
    view->setBuffer( doc->getBuffer( currentLine ) );

    bool ro = (currentLine != Primary_Write);
    
    if (ro == false && doc->isSnooper() == true)
    {
        ro = true;
    }
    
    view->setMute(ro);
    view->updateView(true, true);
    doc->slotChangedViewList();
}

void SLDumpView::slotSelectChunk(BufferLine line, uint offset, uint length)
{
    if (line != currentLine ||
        props->getSyncView() == false ||
        ready == false )
    {
        //qDebug( "%s:%u - slotSelectChunk() nothing to do", __FILE__, __LINE__ );
        return;
    }
    isSyncing = true;
    qDebug( "%s:%u - slotSelectChunk(line=%u,offset=%u,length=%u)", __FILE__, __LINE__,
        line, offset, length);

 
    view->selectDataChunk( offset, length );
    isSyncing = false;
}

void SLDumpView::slotScrollBarChanged(uint offset)
{
    if (ready == true && isSyncing == false)
    {
        qDebug( "%s:%u - slotScrollBarChanged(offset=%u)", __FILE__, __LINE__, offset);
        doc->forceSynchronizeViews( this, currentLine, offset );
        emit cursorChanged();
    }
 
}

void SLDumpView::slotCursorChanged( SCursorState& cs )
{
    //warning: issuing cursorChanged() causes scroll bar updating
    //it must be issued after synchronizing the views otherwise
    //the cs.offset is forced to zero (15/6/2004)
    if (ready == true && isSyncing == false)
    {
        qDebug( "%s:%u - slotCursorChanged(offset=%u)", __FILE__, __LINE__, cs.offset);
        doc->forceSynchronizeViews( this, currentLine, cs.offset );
        emit cursorChanged();
    }
}

void SLDumpView::slotGoToOffset( const SLView* master, BufferLine line, uint offset )
{ 
    if (line != currentLine || 
        master == this ||
        props->getSyncView() == false ||
        ready == false )
    {
        //qDebug( "%s:%u - slotGoToOffset(offset=%u) nothing to do", __FILE__, __LINE__, offset );
        return;
    }
    isSyncing = true;
    qDebug( "%s:%u - slotGoToOffset(offset=%u)", __FILE__, __LINE__, offset);
    view->showOffset( offset );
    isSyncing = false;
}

void SLDumpView::slotDocumentLoaded()
{
    ready = true;
    view->showBufferEnd();
}

/*
void SLDumpView::closeEvent(QCloseEvent* e)
{
    qDebug( "%s:%u - closing SLDumpView", __FILE__, __LINE__   );
    delete this;
}
*/
