/*
 *                        sldumpview.h
 *                           -------------------
 *  begin                : Tue Apr 15 2003
 *  copyright            : (C) 2003 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * $Id: sldumpview.h,v 1.9 2005/01/21 08:40:58 gbertani Exp $ 
 */


#ifndef SLDUMPVIEW_H
#define SLDUMPVIEW_H

#include <assert.h>

// include files for Qt
#include <qlayout.h>


// includes for KDE
#include <klocale.h>
//#include <ktoolbar.h>
//#include <ktoolbarbutton.h>

#include "slsession.h"
#include "slview.h"

class SLView;
class SLSession;
class SLHexViewWidget;
class SLDumpDialog;
class SLHexBuffer;
struct SDisplayState;
struct SCursorState;

/**This view shows the input data as 
hexadecimal/octal/binary dump.
Most of this work is done by the
CHexViewWidget class  
from KHexEdit (c)Espen Sand.  

  *@author giuseppe massimo bertani
  */

class SLDumpView : public SLView  
{
   
    Q_OBJECT
       
public: 
    /** 
     * Creates a view that shows received or sent data 
     * By default the data are showed as hexdump, 16 bytes per line
     * with offset at the left pane and ASCII dump in the right pane.
     * 
     * */
    SLDumpView(SLSession* const pDoc, BufferLine line);
	
    /** */
    virtual ~SLDumpView();
    
    /** */

    
    /** */
    void save(KConfig* data);

    /** */
    void load(KConfig const* data);
   
    
    /** default window size */
    QSize sizeHint() const
    {
        return QSize(600, 200);
    }
    
    /** */
    void cut();

    /** */
    void copy();

    /** */
    void paste();
    
    /** contains the implementation for printing functionality and gets called by SerLooKApp::slotFilePrint() */
    virtual void print(CHexPrinter *pPrinter);
    
    /**
     * Sets the caption text, adding a personalization
     * after a base text
     */
    virtual void setCaption( const QString& num, const QString& dev, 
            const QString& enable ); 
    

    
public slots:
   /** synchronizes the view with the buffer contents.
     * If the line is different from the actual one,
     * nothing happens.
     */
    virtual void slotUpdateView( BufferLine line );

    /** */    
    virtual void slotScrollBarChanged( uint value );
    
    /**  */
    virtual void slotSelectChunk(BufferLine line, uint offset, uint length);
    
protected slots:
    /**
     * Synchronizes the view with the buffer contents
     * when the source buffer changes in the relative
     * properties dialog.
     * If the line is different from the actual one,
     * the view must completely refresh and display the
     * new data */
    virtual void slotChangeBuffer();
        
    /** */
    virtual void slotCursorChanged( SCursorState& cs );

    /** 
     * Updates the hex view widget settings after changing
     * them in the relative properties dialog.
     */
    virtual bool slotChangeDataFormat();
    
    /**
     * if bl is equal to currentLine
     * and master is not equal to this, it
     * synchronizes the view
     * to show the data starting from offset
     */
    virtual void slotGoToOffset( const SLView* master, BufferLine line, uint offset );

    /**
     * things to do immediately after a new session
     * is created or loaded
     */
    virtual void slotDocumentLoaded();

protected:
    //virtual void closeEvent(QCloseEvent* e);
 
    
private:
    
    /** viewer */
    SLHexViewWidget*    view;
    
    /** View properties dialog box */
    SLDumpDialog* props;

    /** cursor offset format */
    bool isOffsetHex;


};

#endif
