/*
 *                        sleventdialog.cpp  -  description
 *                           -------------------
 *  begin                : lun feb 9 2004
 *  copyright            : (C) 2004 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * 
 */
static char* cvs_id = "$Id";


#include "sleventdialog.h"

SLEventDialog::SLEventDialog(QWidget *parent, const char *name  ) 
    : SLViewDialogBase(i18n("Event view properties"), parent, name, false)
{
    setupSourceBufferPage();
    setupSettingsParamsPage();
}

SLEventDialog::~SLEventDialog()
{
    
}

void SLEventDialog::setupSourceBufferPage()
{
    QFrame* page = addPage( i18n( "Source" ), i18n( "Sync selection" ));
    QGridLayout* myLayout = new QGridLayout( page ); 
    myLayout->setSpacing(10);

//row 0
    mSourceBuffer.sourceBufferLabel = new QLabel( page );
    mSourceBuffer.sourceBufferLabel->setText( i18n( "Source of the data displayed in the view" ) );
    //myLayout->addWidget( mSourceBuffer.sourceBufferLabel, 0, 0 );


    mSourceBuffer.sourceBufferSelector = new QComboBox( false, page );

//Primary_Read omitted, since Primary_Write is used -
//meaning bidirectional behavior (Input+Output)

    mSourceBuffer.sourceBufferSelector->insertItem( i18n( "Input/Output to primary line" ), Primary_Read );
    mSourceBuffer.sourceBufferSelector->insertItem( i18n( "Input from secondary line" ), Secondary_Read );
    //myLayout->addWidget( mSourceBuffer.sourceBufferSelector, 0, 1 );

//row 1
    mSourceBuffer.syncViewLabel = new QLabel( page );
    mSourceBuffer.syncViewLabel->setText( i18n( "This view accepts synchronization signals" ) );
    myLayout->addWidget( mSourceBuffer.syncViewLabel, 1, 0 );

    mSourceBuffer.syncViewBox = new QCheckBox( page );
    myLayout->addWidget( mSourceBuffer.syncViewBox, 1, 1 ); 
        
    int idx = pageIndex( page );
    qDebug( "%s:%u -terminal dialog source buffer page %p index: %d", __FILE__, __LINE__, (void*)page, idx );
}

void SLEventDialog::setupSettingsParamsPage()
{

    QFrame* page = addPage( i18n( "Settings" ), i18n( "Event view settings" ));
    QGridLayout* myLayout = new QGridLayout( page ); 
    myLayout->setSpacing(10);
//row 0
    mSettingsParams.skipLabel = new QLabel( page );
    mSettingsParams.skipLabel->setText( i18n( "Skip modem lines status " ) );
    myLayout->addWidget( mSettingsParams.skipLabel, 0, 0 );

    mSettingsParams.skipBox = new QCheckBox( page );
    myLayout->addWidget( mSettingsParams.skipBox, 0, 1 );
 
    int idx = pageIndex( page );
    qDebug( "%s:%u --event view parameters page %p index: %d", __FILE__, __LINE__, (void*)page, idx );

}


void SLEventDialog::slotAllDefaults()
{
    mSettingsParams.skipBox->setChecked(false);
}

void SLEventDialog::slotDefault()
{
    switch( activePageIndex() )
    {
        case pageSettingsParams:
            mSettingsParams.skipBox->setChecked(false);
            break;
            
        default:
            break;
    }
    
}

void SLEventDialog::slotApply()
{
    SLViewDialogBase::slotApply();
    
    switch( activePageIndex() )
    {
        case pageSettingsParams:
            emit settingsChoice();
            break;

        default:
            break;
            
    }
    
}

bool SLEventDialog::getSkip() const
{
    return mSettingsParams.skipBox->isChecked();
}

void SLEventDialog::setSkip( bool skip )
{
    mSettingsParams.skipBox->setChecked( skip );
}

void SLEventDialog::save(KConfig* data)
{
    SLViewDialogBase::save(data);
    data->writeEntry("Skip Modem Status", getSkip()); 
}

void SLEventDialog::load(KConfig const* data)
{
    setSkip( data->readBoolEntry("Skip Modem Status") );
    emit settingsChoice();
    SLViewDialogBase::load(data);
}






