/*
 *                        sleventrecord.cpp  -  description
 *                           -------------------
 *  begin                : lun nov 24 2003
 *  copyright            : (C) 2003 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * 
 */
static char* cvs_id = "$Id: sleventrecord.cpp,v 1.7 2005/01/21 08:40:58 gbertani Exp $";


#include "slhexbuffer.h"
#include "sleventrecord.h"

SLEventRecord::SLEventRecord( QString& name )
{
    qDebug( "%s:%u - Creating event recorder", __FILE__, __LINE__ );

    evRead.progNr = 0;
    evWrite.progNr = 0;
    fileName = name;
    
    evBuf = new SLHexBuffer;
    CHECK_PTR( evBuf );

    if (evBuf->loadBuffer( fileName ) == true)
    {
        evWrite.progNr = getTotalEvents();
    }
    else
    {
        qDebug( "%s:%u - Event buffer file %s not found, it will be created from scratch", __FILE__, __LINE__, fileName.latin1() );
    }

}

SLEventRecord::~SLEventRecord()
{
    qDebug( "%s:%u - Destroying SLEventRecord", __FILE__, __LINE__ );

    evBuf->closeFile();
    delete evBuf;
}

ulong SLEventRecord::getTotalEvents() const
{
    return static_cast<ulong>(evBuf->documentSize() / sizeof(SLEvent));
}

void SLEventRecord::addEvent( EventKind k, long off, long size, uint modem )
{
    timeval timeStamp;

    switch( k )
    {
        case EV_STAT1IN:
        case EV_STAT1OUT:
        case EV_STAT2:
            evWrite.offset = off;
            evWrite.chunkSize = 0;
            break;

        case EV_RX1:
        case EV_RX2:
        case EV_TX:
            evWrite.offset = off;
            evWrite.chunkSize = size;
            break;

        default:
            //does nothing
            return;
    }

    gettimeofday( &timeStamp, 0 );
    setEventTimeval( evWrite, timeStamp );
    evWrite.kind = k;
    evWrite.mStatus = modem;
    evWrite.progNr++;

    //moves the input cursor of the buffer after
    //the last event written
    SCursorConfig cc;
    cc.emulateControlButton( true );
    evBuf->cursorEnd( cc.controlButton() );

    QByteArray ba(sizeof(SLEvent));
    memcpy( ba.data(), &evWrite, sizeof(SLEvent) );
    
    int errCode = evBuf->inputAtCursor( ba, 0 );
    if( errCode != Err_Success  )
    {
        qDebug( "%s:%u - addEvent() - inputAtCursor failed, errCode=%d", __FILE__, __LINE__, errCode );
    }
}


bool SLEventRecord::getEvent( ulong prog, SLEvent& ev )
{
    ulong pos = (prog-1)*sizeof(SLEvent);
    
    if (pos < evBuf->documentSize())
    {
        memcpy(&evRead, &evBuf->data()[pos], sizeof(SLEvent));
        ev = evRead;
        return true;
    }
    
    return false;
}

int SLEventRecord::getEvent( const timeval& ts, uint id, SLEvent& ev )
{
bool found = false;
int retval = -1;
timeval timeStamp;
long begin, end, curr;
    
    getEventTimeval( evRead, timeStamp );
    
    if ( timeDiff (ts, timeStamp) == 0 )
    {
        //requested the same already read
        ev = evRead;
        retval = 0;
        return retval;
    }
    
    
    if ( timeDiff (ts, timeStamp) > 0 )
    {
        //The event searched happened after the last
        //record read: searches forward from the last event 
        //read to the last recorded 
        //(remember progr IDs start from 1)

        begin = evRead.progNr;
        end = getTotalEvents();
    }
    else
    {
        //Otherwise searches from the first
        //record to the last one read
        begin = 1;
        end = evRead.progNr;
    }
    
    //browses the buffer until the timestamp is different
    //than requested
    for (curr = begin; curr <= end; curr++)
    {
        if ( getEvent( curr, evRead ) == false )
        {
            //it should never happen
            qDebug( "%s:%u - getEvent() returned false", __FILE__, __LINE__ );
            break;
        }
        getEventTimeval( evRead, timeStamp );
        if ( timeDiff (ts, timeStamp) == 0 )
        {
            found = true;
            break;
        }
    }
        
    if (found == true)
    {
        //Checks the <id> argument

        if (id == 0)
        {
            ev = evRead;
            retval = 0;
            // returns the first record that matches the timestamp
        }
        else
        {
            found = false;
            curr += id;
            if ( getEvent( curr, evRead ) == false )
            {
                //record at curr+id not found
                retval = -1;
            }
            getEventTimeval( evRead, timeStamp );
            if ( timeDiff (ts, timeStamp) == 0 )
            {
                //record at curr+id has different timestamp
                retval = -2;
            }
        }

    }
    else
    {
        //timestamp not found at all
        retval = -3;
    }

    return retval;
}

ulong SLEventRecord::getEventProg( EventKind k, long off )
{
int retval = -1;
long lastOff = -1;
long begin, end, curr;

    if ( evRead.offset == off )
    {
        //requested the same already read
        retval = evRead.progNr;
        return retval;
    }
    
    if ( evRead.offset < off )
    {
        //The event searched happened after the last
        //record read: searches forward from the last event 
        //read to the last recorded 
        //(remember progr IDs start from 1)

        begin = evRead.progNr;
        end = getTotalEvents();
    }
    else
    {
        //Otherwise searches from the first
        //record to the last one read
        begin = 1;
        end = evRead.progNr;
    }

    for (curr = begin; curr <= end; curr++)
    {
        if ( getEvent( curr, evRead ) == false )
        {
            //it should never happen
            qDebug( "%s:%u - getEvent() returned false", __FILE__, __LINE__ );
            break;
        }
        if( evRead.kind == k )
        {
            lastOff = (evRead.offset + evRead.chunkSize - 1);
            if (off >= evRead.offset && off <= lastOff)
            {
                return evRead.progNr;
            }
        }
    }
        
    return retval;

}
    

long SLEventRecord::timeDiff( const timeval& a, const timeval& b )
{
long l_diff;


    l_diff = (UINT32) (a.tv_sec - b.tv_sec);
    l_diff *= 1000000;
    l_diff += (UINT32)(a.tv_usec - b.tv_usec);

    return l_diff;
}



void SLEventRecord::getEventTimeval( const SLEvent& ev, timeval& tv ) const
{
    tv.tv_sec = ev.tvSec;
    tv.tv_usec = ev.tvUSec;
}

void SLEventRecord::setEventTimeval( SLEvent& ev, const timeval& tv ) const
{
    ev.tvSec = tv.tv_sec;
    ev.tvUSec = tv.tv_usec;
}

/**
 * flushes the buffer
 */
void SLEventRecord::clear()
{
    evBuf->clear();
    evRead.progNr = 0;
    evWrite.progNr = 0;

}

/**
 *
 */
bool SLEventRecord::saveBuffer( const QString& newName )
{
    return evBuf->saveBuffer( newName );
}

/**
 *
 */
bool SLEventRecord::loadBuffer( const QString& name )
{
    return evBuf->loadBuffer( name );
}
    
