/*
 *                        sleventview.cpp  -  description
 *                           -------------------
 *  begin                : Wed Dec 31 2003
 *  copyright            : (C) 2003 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * 
 */
static char* cvs_id = "$Id: sleventview.cpp,v 1.10 2005/01/21 08:40:58 gbertani Exp $";


#include "slsession.h"
#include "slview.h"
#include "sleventdialog.h"
#include "slhexbuffer.h"
#include "sleventrecord.h"
#include "slserialport.h"
#include "sllogbrowser.h"
#include "sleventview.h"

#define HEADING_SIZE 2

SLEventView::SLEventView(SLSession* const pDoc) :
    SLView(pDoc)
{
    qDebug( "%s:%u - building sleventview", __FILE__, __LINE__ );  
    //----- viewer-------------
   
    QVBoxLayout* layout = new QVBoxLayout(this); 
   
    //first creates a default term widget 
    view = new SLLogBrowser( this, 0 );
    CHECK_PTR( view );
    view->setHeadingSize( HEADING_SIZE );
    
    layout->addWidget( view );
    
    props = 0;
    
    props = new SLEventDialog(this);
    CHECK_PTR( props );
    baseProps = props;
    
    //default props
    props->setSkip(true);

    hdr = i18n("Event ID  |     Time [s.uS]      |  Event Kind    | Offset   |Chunk Size|TXD|RTS|DTR|RXD|CTS|DSR|DCD| RI ");
//  hdr = i18n("Event ID  |     Time [s.uS]      |  Event Kind    | Offset   |Chunk Size|      Modem status lines        ");
    sep =      "------------------------------------------------------------------------------------------------------";

    
    connect(doc,  SIGNAL( synchronizeViews( const SLView*, BufferLine, uint )), this, SLOT( slotGoToOffset( const SLView*, BufferLine, uint ) ));
    connect(props, SIGNAL( settingsChoice() ), this, SLOT( slotChangeSettings() ));
    connect(view, SIGNAL( mouseRightClick() ), this, SLOT( slotPropsDialog() ));
    connect(view, SIGNAL( clicked(int,int) ), this, SLOT( slotCursorChanged(int,int) ));
    connect(view, SIGNAL( scrollBarChanged(uint) ), this, SLOT( slotScrollBarChanged(uint) ));

    view->append( hdr );
    view->append( sep );
    slotUpdateView( Primary_Write );

    ready = false;
    isSyncing = false;
}


SLEventView::~SLEventView()
{
    qDebug( "%s:%u - destroying SLEventView", __FILE__, __LINE__ );   
    delete props;
    delete view;
}

void SLEventView::copy()
{
    view->copy();
}


void SLEventView::save(KConfig* data)
{
    data->writeEntry("Class", QString("SLEventView") );
    
    props->save(data);
    SLView::save(data);
}

void SLEventView::load(KConfig const* data)
{
    //The "Class" field is already read
    //by SLSession

    SLView::load(data);
    props->load(data);
}

void SLEventView::print(CHexPrinter *pPrinter)
{
    if (pPrinter->setup( this ))
    {
        // create a painter to paint on the printer object
        QPainter paint;
        QString toDraw = view->text();
        QString line;
        uint lineWidth = sep.length();
        SLEventRecord* er = doc->getEventRecorder();
        ulong numLines = er->getTotalEvents();
        
        // start painting
        paint.begin( pPrinter );
        paint.setFont( view->font() );
        
        int interLine = view->getLineHeight();
        
        SPageMargin margin = pPrinter->pageMargin();
        SPageSize size = pPrinter->pageUsableSize();
        paint.setClipRect( margin.left, margin.top, size.width, size.height );

        int headHeight, footHeight, headMargin, footMargin, freeHeight;

        headHeight = footHeight = headMargin = footMargin = 0;
        headHeight = interLine;
        headMargin = interLine / 2;
        footHeight = interLine;
        footMargin = interLine / 2;
        
        freeHeight = size.height - headHeight - footHeight - headMargin - footMargin;
        uint pixLineWidth =  static_cast<unsigned int>(view->getLineWidth( sep ));
        float scale = 1.0;
        if( pixLineWidth > size.width /*&& pPrinter->scaleToFit() == true */)
        {
            scale = (float)size.width / (float)pixLineWidth;
            paint.scale( scale, scale );
        }
        
        uint headerLines = HEADING_SIZE;
        uint linePerPage = (uint) ((float)freeHeight/((float)interLine*scale)) - headerLines;
        uint sx = (uint) ((float)margin.left/scale);
        uint sy = (uint) ((float)(margin.top+headHeight+headMargin)/scale);

        uint remaining = numLines;
        uint onePerCent = numLines / 100;
        
        uint i = 0; 
        uint lineInPage;

        while( remaining > 0 )
        {
            //the view header is skipped because in printing
            //the header is reprinted on each page
            i = numLines - remaining + headerLines;
            
            emit updateProgressBar(i / onePerCent, this);
            
            lineInPage = remaining > linePerPage ? linePerPage : remaining;
            uint y = sy;
        
            paint.drawText(sx, y, hdr, lineWidth); 
            y += interLine;
            paint.drawText(sx, y, sep, lineWidth);
            y += interLine;
            //Each log line has always the same length
            //that is the same length of the header separator,
            //so it's easy to split the log into separate lines
            for( uint count = 0 ; count < lineInPage; y += interLine, count++, i++ )
            {
                line = toDraw.section('\n', i);
                paint.drawText(sx, y, line, lineWidth);
            }
            
            remaining -= lineInPage;
            if( remaining > 0 )
            {
                pPrinter->newPage();
            }


        }
        // stop painting, this will automatically send the print data to the printer
        paint.end();
        emit updateProgressBar( -1, this );

    }
}


void SLEventView::setCaption( const QString& num, const QString& dev,
            const QString& enable )
{
    //<dev> and <enable> parameters intentionally ignored
    QString caption = "(" + num + ") " + "Events log view";
    QWidget::setCaption(caption);
}

void SLEventView::slotPropsDialog()
{
    if (doc->acquisition() == false)
    {
        SLView::slotPropsDialog();

    }
}

void SLEventView::slotChangeSettings()
{
    if (ready == true)
    {
        //avoids to redraw all while loading the buffer
        //the view was already filled by the constructor
        view->clear();
        view->append( hdr );
        view->append( sep );
        slotUpdateView( Primary_Write );
    }
 
}

void SLEventView::buildLogLine( const SLEvent& ev, QString& line )
{

    bool skipOffset = false;
    bool skipModem = false;
    QString kindStr;

    switch(ev.kind )
    {
        case EV_RX1:
            kindStr = i18n("pri RX       ");
            break;

        case EV_RX2:
            kindStr = i18n("sec RX       ");
            break;

        case EV_TX:
            kindStr = i18n("pri TX       ");
            break;

        case EV_STAT1IN:
            kindStr = i18n("pri state in ");
            skipOffset = true;
            break;
            
        case EV_STAT1OUT:
            kindStr = i18n("pri state out");
            skipOffset = true;
            break;

        case EV_STAT2:
            kindStr = i18n("sec state    ");
            skipOffset = true;
            break;

        default:
            kindStr = i18n("unknown      ");
            break;
    }

    QString leds;
    if (props->getSkip() == true)
    {
        //skips the modem status events
        //as required from the event dialog box
        skipModem = true;
        leds = "";
    }

    if ( skipModem == false )
    {

        if ((ev.mStatus & TIOCM_ST) != 0)
        {
            leds.append(" 1 |");
        }
        else
        {
            leds.append(" 0 |");
        }

        if ((ev.mStatus & TIOCM_RTS) != 0)
        {
            leds.append(" 1 |");
        }
        else
        {
            leds.append(" 0 |");
        }

        if ((ev.mStatus & TIOCM_DTR) != 0)
        {
            leds.append(" 1 |");
        }
        else
        {
            leds.append(" 0 |");
        }

        if ((ev.mStatus & TIOCM_SR) != 0)
        {
            leds.append(" 1 |");
        }
        else
        {
            leds.append(" 0 |");
        }

        if ((ev.mStatus & TIOCM_CTS) != 0)
        {
            leds.append(" 1 |");
        }
        else
        {
            leds.append(" 0 |");
        }

        if ((ev.mStatus & TIOCM_DSR) != 0)
        {
            leds.append(" 1 |");
        }
        else
        {
            leds.append(" 0 |");
        }

        if ((ev.mStatus & TIOCM_CD) != 0)
        {
            leds.append(" 1 |");
        }
        else
        {
            leds.append(" 0 |");
        }

        if ((ev.mStatus & TIOCM_RI) != 0)
        {
            leds.append(" 1 ");
        }
        else
        {
            leds.append(" 0 ");
        }
    }

    if ( skipOffset == false && skipModem == false )
    {
        line.sprintf("%-.9lu |% .9ld.%-9ld |%.13s   |%9ld |%9ld |%s",
                ev.progNr,
                ev.tvSec,
                ev.tvUSec,
                kindStr.latin1(),
                ev.offset,
                ev.chunkSize,
                leds.latin1()
                );
    }
    else if ( skipOffset == false && skipModem == true )
    {
        line.sprintf("%-.9lu |% .9ld.%-9ld |%.13s   |%9ld |%9ld |%s",
                ev.progNr,
                ev.tvSec,
                ev.tvUSec,
                kindStr.latin1(),
                ev.offset,
                ev.chunkSize,
                leds.latin1()
                );
    }
    else
    {
        line.sprintf("%-.9lu |% .9ld.%-9ld |%.13s   |          |          |%s",
                ev.progNr,
                ev.tvSec,
                ev.tvUSec,
                kindStr.latin1(),
                leds.latin1()
                );

    }

    //add trailing spaces in order to keep
    //the log lines always with the same size

    line = line.leftJustify( sep.length(), ' ', true );

}


void SLEventView::slotUpdateView(BufferLine line)
{

    //<line> parameter intentionally ignored

    if ( doc->acquisition() == false )
    {
        //event table is updated only when the acquisition stops
        //for the following reasons:
        //1-speed up the acquisition
        //2-while updating the table, the table view takes automatically
        // the focus - I don't know how to change this behavior - In fact it's
        // very boring while entering data from the keyboard to another 
        // dump view or terminal view :-(
    
        SLEvent ev;
        SLEventRecord* er = doc->getEventRecorder();
        
        //rows and events are numbered starting from 0, 
        //so the totRows value is also the ID of the next
        //row to be added
        ulong totEventsRecorded = er->getTotalEvents();
        ulong totEventsShowed = view->getShowedEvents();        
       
        if (totEventsRecorded == 0 &&  totEventsShowed > 0) 
        {
            qDebug( "%s:%u - slotUpdateView() redrawing heading, totEventsRecorded=%lu, totEventsShowed=%lu",
                __FILE__, __LINE__, totEventsRecorded, totEventsShowed );
            //the event log was cleaned, so the table must
            //be flushed before adding new events
            view->clear();

            //header
            view->append( hdr );
            view->append( sep );
        }
        
        ulong eventsToAdd = (totEventsRecorded - totEventsShowed);

        if (eventsToAdd > 0)
        {
            int retval;
            ulong i;
           
            QString line;
            qDebug( "%s:%u - slotUpdateView() appending totEventsRecorded=%lu",
                __FILE__, __LINE__, totEventsRecorded );
            
            for(i = totEventsShowed + 1; i <= totEventsRecorded; i++)
            {
                retval = er->getEvent( i, ev );
                if (er == 0)
                {
                    //debug( "%s:%u - slotUpdateView() no more events: %lu",
                    // __FILE__, __LINE__, i );
                    break;
                }

                //qDebug( "%s:%u - slotUpdateView() appending %lu",
                //    __FILE__, __LINE__, i );
     
                buildLogLine( ev, line );
                view->append( line );
            }
            view->showEvent(i);
        }
    }
    
}

void SLEventView::slotCursorChanged( int para, int pos )
{
    if (ready == false || isSyncing == true)
    {
        return;
    }
    
    SLEvent ev;
    SLEventRecord* er = doc->getEventRecorder();

    //the first two paragraphs (paragraph==line)
    //contains the heading line. The line count
    //in the event recorder starts from 1
    long prog = para - HEADING_SIZE + 1;
    ASSERT(prog >= 0);

    if (doc->acquisition() == false)
    {
        view->showEvent(prog);
    }

    er->getEvent( prog, ev );

    qDebug( "%s:%u - slotCursorChanged(line=%ld) offset=%lu", __FILE__, __LINE__, prog, ev.offset);
    
    switch (ev.kind)
    {
        case EV_RX1:
            doc->forceSynchronizeViews( this, Primary_Read, ev.offset );
            doc->forceSelectChunk(Primary_Read, ev.offset, ev.chunkSize);
            break;
            
        case EV_RX2:
            doc->forceSynchronizeViews( this, Secondary_Read, ev.offset );
            doc->forceSelectChunk(Secondary_Read, ev.offset, ev.chunkSize);
            break;
            
        case EV_TX:
            doc->forceSynchronizeViews( this, Primary_Write, ev.offset );
            doc->forceSelectChunk(Primary_Write, ev.offset, ev.chunkSize);
            break;

            //the chunk size for state changes is 1
        case EV_STAT1IN:
            doc->forceSynchronizeViews( this, Primary_Read, ev.offset );
            doc->forceSelectChunk(Primary_Read, ev.offset, 1);
            break;
            
        case EV_STAT1OUT:
            doc->forceSynchronizeViews( this, Primary_Write, ev.offset );
            doc->forceSelectChunk(Primary_Write, ev.offset, 1);
            break;
            
        case EV_STAT2:
            doc->forceSynchronizeViews( this, Secondary_Read, ev.offset );
            doc->forceSelectChunk(Secondary_Read, ev.offset, 1);
            break;

        default:
            return;
    }
}

void SLEventView::slotScrollBarChanged( uint para )
{
    if (ready == false || isSyncing == true)
    {
        return;
    }

    SLEvent ev;
    SLEventRecord* er = doc->getEventRecorder();

    //the first two paragraphs (paragraph==line)
    //contains the heading line. The line count
    //in the event recorder starts from 1
    long prog = para - HEADING_SIZE + 1;
    ASSERT(prog >= 0);

    er->getEvent( prog, ev );

    qDebug( "%s:%u - slotScrollBarChanged(line=%ld) offset=%lu", __FILE__, __LINE__, prog, ev.offset);

    switch (ev.kind)
    {
        case EV_RX1:
            doc->forceSynchronizeViews( this, Primary_Read, ev.offset );
            break;

        case EV_RX2:
            doc->forceSynchronizeViews( this, Secondary_Read, ev.offset );
            break;

        case EV_TX:
            doc->forceSynchronizeViews( this, Primary_Write, ev.offset );
            break;

        case EV_STAT1IN:
            doc->forceSynchronizeViews( this, Primary_Read, ev.offset );
            break;

        case EV_STAT1OUT:
            doc->forceSynchronizeViews( this, Primary_Write, ev.offset );
            break;

        case EV_STAT2:
            doc->forceSynchronizeViews( this, Secondary_Read, ev.offset );
            break;

        default:
            return;
    }
}



void SLEventView::slotGoToOffset( const SLView* master, BufferLine line, uint offset )
{

    if (ready == false ||
        props->getSyncView() == false ||
        master == this)
    {
        //qDebug( "%s:%u - slotGoToOffset(offset=%u) nothing to do", __FILE__, __LINE__, offset);
        return;
    }

    isSyncing = true;
    SLEventRecord* er = doc->getEventRecorder();
    long prog;

    switch( line )
    {
        case Primary_Write:
            prog = er->getEventProg( EV_TX, offset );
            break;

        case Primary_Read:
            prog = er->getEventProg( EV_RX1, offset );
            break;

        case Secondary_Read:
            prog = er->getEventProg( EV_RX2, offset );
            break;

        default:
            /* modem state change events ignored */
            prog = -1;
            break;
    }

    if (prog == -1)
    {
        /* requested event not found, ignore request */
        qDebug( "%s:%u - slotGoToOffset(offset=%u) event not found, ignoring", __FILE__, __LINE__, offset);
        isSyncing = false;
        return;
    }

    view->showEvent(prog);

    SLEvent ev;
    er->getEvent(prog, ev);
    doc->forceSelectChunk(line, offset, ev.chunkSize);
    
    qDebug( "%s:%u - slotGoToOffset(offset=%u) setting position at line %ld", __FILE__, __LINE__, offset, prog );
    isSyncing = false;
}


void SLEventView::slotChangeBuffer()
{}

void SLEventView::slotDocumentLoaded()
{
    ready = true;
}

/*
void SLEventView::closeEvent(QCloseEvent* e)
{
    qDebug( "%s:%u - closing SLEventView, __FILE__, __LINE__" );
    delete this;
}
*/



