/*
 *                        sleventview.h  -  description
 *                           -------------------
 *  begin                : Wed Dec 31 2003
 *  copyright            : (C) 2003 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * $Id: sleventview.h,v 1.8 2005/01/21 08:40:58 gbertani Exp $
 */

#ifndef SLEVENTVIEW_H
#define SLEVENTVIEW_H


// include files for Qt
#include <qlayout.h>
#include <qtextbrowser.h>


// includes for KDE
#include <klocale.h>


#include "slsession.h"
#include "slview.h"

//WORK IN PROGRESS -------------------------------------------

class SLView;
class SLEventDialog;
class SLSession;
class SLHexBuffer;
class SLLogBrowser;
class SLEvent;

/**
 * displays low level communication
events (chunks of data sent or 
received and modem lines status
changes)
  *@author giuseppe massimo bertani
  */

class SLEventView : public SLView  
{
   Q_OBJECT
       
public: 
    /** */
	SLEventView(SLSession* const pDoc);

    /** */
	virtual ~SLEventView();

    /** */
    void save(KConfig* data);

    /** */
    void load(KConfig const* data);
   
    /** default window size */
    QSize sizeHint() const
    {
        return QSize(790, 340);
    }

    /** contains the implementation for printing functionality and gets called by SerLooKApp::slotFilePrint() */
    virtual void print(CHexPrinter *pPrinter);

    /**
     * Sets the caption text, adding a personalization
     * after a base text
     */
    virtual void setCaption( const QString& num, const QString& dev, 
            const QString& enable ); 
    
    /** */
    virtual void cut() { qDebug( "cut() not implemented"); };

    /** */
    virtual void copy();

    /** */
    virtual void paste() { qDebug( "paste() not implemented"); };

public slots:
    /** synchronizes the view with the buffer contents.
     * If the line is different from the actual one,
     * nothing happens.
     */
    virtual void slotUpdateView( BufferLine line );

    /**  */
    void slotGoToOffset( const SLView* master, BufferLine line, uint offset );

    /** called when the user clicks on a event line */
    void slotCursorChanged( int para, int pos );

    /** called while moving the scrollbar */
    void slotScrollBarChanged( uint para );
    
protected slots:

    /** */
    virtual void slotPropsDialog();

    /**
     * Synchronizes the view with the buffer contents
     * when the source buffer changes in the relative
     * properties dialog.
     * If the line is different from the actual one,
     * the view must completely refresh and display the
     * new data */
    virtual void slotChangeBuffer();
    
    /** */
    virtual void slotChangeSettings();
        
    /**
     * things to do immediately after a new session
     * is created or loaded
     */
    virtual void slotDocumentLoaded();

signals:
  
protected:
    //virtual void closeEvent(QCloseEvent* e);
    
private:
   
    /** */
    void buildLogLine( const SLEvent& ev, QString& line );
    
    /** */
    SLLogBrowser* view;
    
    /** Event view properties dialog box */
    SLEventDialog* props;

    /** report header */
    QString hdr;
    
    /** header separator */
    QString sep;
    
};


#endif
