/*
 *                        slhexbuffer.cpp
 *                           -------------------
 *  begin                : Wed May 7 2003
 *  copyright            : (C) 2003 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * 
 */

static char* cvs_id = "$Id: slhexbuffer.cpp,v 1.8 2004/12/10 14:16:25 gbertani Exp $";

#include "slhexbuffer.h"



SLHexBuffer::SLHexBuffer( SLHexBuffer* baseBuffer )
{

    clear();
    takeSharedBlockFrom( baseBuffer );
}

SLHexBuffer::SLHexBuffer()
{
}

SLHexBuffer::~SLHexBuffer()
{
    //array_data* my_shd = sharedBlock();
    //ASSERT( my_shd == shared_data );
    //array_data* my_shd = srcBuffer->sharedBlock();
    //ASSERT( my_shd == shared_data );
    //shared_data->deref();
}

void SLHexBuffer::alignBuffers( SLHexBuffer* buf )
{
    if ( buf == 0 )
    {
        setDocumentSize( srcBuffer->documentSize() );
        setLineRef( srcBuffer->getLineRef() );
    }
    else
    {
        setDocumentSize( buf->documentSize() );
        setLineRef( buf->getLineRef() );
    }
    computeNumLines();

}

void SLHexBuffer::takeSharedBlockFrom( SLHexBuffer* baseBuffer )
{
    srcBuffer = baseBuffer;
    
    array_data* my_shd = sharedBlock();
    delete my_shd;
    
    shared_data = srcBuffer->sharedBlock();
    setSharedBlock( shared_data );
    my_shd = sharedBlock();

    ASSERT( my_shd == shared_data );
    
    shared_data->ref();
    setDocumentSize( srcBuffer->documentSize() );
    setLineRef( srcBuffer->getLineRef() );
}


bool SLHexBuffer::inputAtCursor( QChar c )
{
    if( documentPresent() == false || mInputMode.noInput() == true )
    {
        if( mInputMode.noInput() == true ) { inputSound(); }
        return( false );
    }

    if( c.isPrint() == false )
    {
        inputSound();
        return( false );
    }

    unsigned char dest;
    bool insert;
    if( mEditMode == EditReplace || mCursor.curr.cell > 0 )
    {
        if( mCursor.curr.offset >= documentSize() )
        {
            dest     = 0;
            insert = true;
        }
        else
        {
            dest     = (unsigned char)data()[ mCursor.curr.offset ];
            insert = false;
        }
    }
    else
    {
        dest     = 0;
        insert = true;
    }

    if( insert == true && mInputMode.allowResize == false )
    {
        inputSound();
        return( false );
    }

    if( mActiveEditor == edit_primary )
    {
        // ## if( THIS_FPTR(inputCell)( &dest, c.latin1(), mCursor.curr.cell ) 
        //== false )
        if( THIS_FPTR(inputCell)( &dest, QString(c).local8Bit()[0], 
                        mCursor.curr.cell ) == false )
        {
            inputSound();
            return( false );
        }
    }
    else if( mActiveEditor == edit_secondary )
    {
        // ## if( inputAscii( &dest, c.latin1(), mCursor.curr.cell ) == false )
        if( !inputAscii( &dest, QString(c).local8Bit()[0], mCursor.curr.cell ) )
        {
            inputSound();
            return( false );
        }
    }
    else
    {
        return( false );
    }

    computeLF( mCursor.curr.offset, c );

    recordStart( mCursor );
    recordReplace( mCursor, insert == true ? 0 : 1, (char*)&dest, 1 );
    cursorRight( cursorPrimaryEdit() );
    recordEnd( mCursor );

    computeNumLines();

    return( true );
}


int SLHexBuffer::inputAtCursor( const QByteArray &buf, uint oldSize )
{
    if( documentPresent() == false )
    {
        return( Err_NoActiveDocument );
    }
    if( buf.isNull() == true )
    {
        return( Err_EmptyArgument );
    }

    if( mInputMode.noInput() == true )
    {
        inputSound();
        return( Err_WriteProtect );
    }

    if( mInputMode.allowResize == false )
    {
        inputSound();
        return( Err_NoResize );
    }

    computeLF( mCursor.curr.offset, buf );
    
    recordStart( mCursor );
    recordReplace( mCursor, oldSize, (char*)&buf[0], buf.size() );
    cursorStep( buf.size(), true, false );
    recordEnd( mCursor );

    computeNumLines();
    return( Err_Success );
}


void SLHexBuffer::cursorRight( bool cellLevel )
{
    bool ret = false;
    
    if( cellLevel == true && mActiveEditor == edit_primary )
    {
        ret = mCursor.incCell();
    }
    else
    {
        mCursor.addOffset( 1 );
        ret = true;
    }
    cursorCompute();

    if (ret == true)
    {
        emit dataEntered();
    }
}

void SLHexBuffer::cursorStep( uint size, bool forward, bool modulo )
{
    if( forward == true )
    {
        if( modulo == true )
        {
            uint offset = mCursor.getOffset() + size;
            mCursor.setOffset( offset - offset % size );
        }
        else
        {
            mCursor.addOffset( size );
        }
    }
    else
    {
        if( modulo == true )
        {
            uint offset = mCursor.getOffset();
            if( offset % size )
            {
                mCursor.decOffset( offset % size, false );
            }
            else
            {
                mCursor.setOffset( offset < size ? 0 : offset - size );
            }
        }
        else
        {
            mCursor.decOffset( size, false );
        }
    }
    cursorCompute();
    emit dataEntered();

}

void SLHexBuffer::computeLF( uint fromOffset, const QByteArray& ba )
{
    int offset = 0;
    int lines = getLFCount();
    
    do
    {
        offset = ba.find( LF, offset );

        if (offset != -1)
        {
            lineRef.resize( lines+1 );
            lineRef[ lines ] = fromOffset + offset;
            lines++;
            offset++;
        }
            
    }
    while ( offset != -1 ); 
    
}

void SLHexBuffer::computeLF( uint fromOffset, const QChar c )
{
    int lines = getLFCount();
    
    if ( c == QChar( LF ) )
    {
        lineRef.resize( lines+1 );
        lineRef[ lines ] = fromOffset;
    }
}

void SLHexBuffer::computeLF()
{
    int offset = 0;
    int lines = 1;
    
    //rebuild the lineRef array
    lineRef.resize(1);
    
    lineRef[0] = 0;
    
    do
    {
        offset = find( LF, offset );

        if (offset != -1)
        {
            lineRef.resize( lines+1 );
            lineRef[ lines ] = offset;
            lines++;
            offset++;
        }
            
    }
    while ( offset != -1 );
}

uint SLHexBuffer::getLFCount() const
{
    int count = lineRef.size();
    return static_cast<uint>(count);
}

uint SLHexBuffer::getLF( uint offset ) const
{
    int lineOffset = offset;
    int line = 0;
    
    while (lineOffset > 0 && line == 0)
    {
        line = lineRef.find( lineOffset );
        if (line == -1)
        {
            line = 0;
        }
        lineOffset--;
    }

    return static_cast<uint>(line);
}


uint SLHexBuffer::getOffset( uint lineFeeds ) const
{
    int retval = 0;
    uint offset = 0;
    
    if (getLFCount() > lineFeeds)
    {
        retval = lineRef[ lineFeeds ];
        ASSERT( retval >= 0 );
        offset = static_cast<uint>(retval);
        return offset;
    }

    if (getLFCount() == 0)
    {
        retval = lineRef[ 0 ];
        ASSERT( retval >= 0 );
        offset = static_cast<uint>(retval);
        return offset;
    }

    //default: returns the offset of last line
    retval = lineRef[ getLFCount() - 1 ];
    ASSERT( retval >= 0 );
    offset = static_cast<uint>(retval);
    return offset;
}

void  SLHexBuffer::printLineRefs() const
{
    qDebug("%s:%u - Buffer=%p", __FILE__, __LINE__, (void*)this);

    for (uint i = 0; i < getLFCount(); i++)
    {
        uint offset =  lineRef[i];
        qDebug("%s:%u - Line: %u, Offset=%u", __FILE__, __LINE__, i, offset);
    }

}

void SLHexBuffer::clear()
{
    lineRef.resize(1);

    //The line 0 have always offset 0
    //

    lineRef[0] = 0;
    setDocumentSize(0);
}
    
int SLHexBuffer::readFile( QFile &file, const QString &url, CProgress &p )
{
    int retval = CHexBuffer::readFile( file, url, p );
    computeLF();
    return retval;
    
}



int SLHexBuffer::print( CHexPrinter &printer )
{

    printer.setDocName( /*mUrl*/ "NONAME" );

    QPainter paint( &printer );
    paint.setFont( font() );

    SPageMargin margin = printer.pageMargin();
    SPageSize size = printer.pageUsableSize();
    paint.setClipRect( margin.left, margin.top, size.width, size.height );

    //printf("%d,%d,%d,%d\n", margin.left, margin.top, size.width, size.height );

    int headHeight, footHeight, headMargin, footMargin, freeHeight;

    headHeight = footHeight = headMargin = footMargin = 0;
    if( printer.pageHeader().enable == true )
    {
        headHeight = headerHeight( paint );
        headMargin = headerMargin( paint );
    }
    if( printer.pageFooter().enable == true )
    {
        footHeight = headerHeight( paint );
        footMargin = headerMargin( paint );
    }
    freeHeight = size.height - headHeight - footHeight - headMargin - footMargin;

    float scale = 1.0;
    if( (uint)mLineWidth > size.width && printer.scaleToFit() == true )
    {
        scale = (float)size.width / (float)mLineWidth;
        paint.scale( scale, scale );
    }

    uint linePerPage = (uint) ((float)freeHeight/((float)lineHeight()*scale));
    uint sx = (uint) ((float)margin.left/scale);
    uint sy = (uint) ((float)(margin.top+headHeight+headMargin)/scale);

    uint remaining = numLines();
    uint line = 0;

    if( printer.all() == true )
    {
        line = 0;
        remaining = numLines();
    }
    else if( printer.selection() == true )
    {
        if( mSelect.valid() == false )
        {
            return( Err_NoSelection );
        }
        line = calculateLine( mSelect.curr.start );
        remaining = calculateLine( mSelect.curr.stop ) - line + 1;
    }
    else if( printer.range() == true )
    {
        line = calculateLine( printer.startRange() );
        remaining = calculateLine( printer.stopRange() ) - line + 1;
    }
    else
    {
        return( Err_IllegalMode );
    }

    #ifdef PRINTER_TEST
    remaining = remaining > linePerPage * 10 ? linePerPage * 10 : remaining;
    #endif


    SPagePosition pageData( time(0), remaining, linePerPage );
    
    uint totLines = remaining;
    uint i = 0;

    while( remaining > 0 )
    {
        i = totLines - remaining;
        
        uint lineInPage = remaining > linePerPage ? linePerPage : remaining;
        uint y = sy;

        //
        // Draw header and footer. Reset scaling during that operation.
        //
        paint.scale( 1.0/scale, 1.0/scale );
        if( printer.pageHeader().enable == true )
        {
            drawHeader( paint, margin.left, size.width, margin.top, false,
            printer.pageHeader(), pageData );
        }
        if( printer.pageFooter().enable == true )
        {
            drawHeader( paint, margin.left, size.width,
            margin.top+size.height-footHeight, true,
            printer.pageFooter(), pageData );
        }
        paint.scale( scale, scale );

        //
        // Draw actual data
        //
        for( uint i=0; i < lineInPage; i++, line++ )
        {
            drawText( paint, line, sx, mLineWidth, y, printer.printBlackWhite() );
            y += lineHeight();// - mLayout.horzGridWidth; // FIXME not really nice :)

        }

        remaining -= lineInPage;
        if( remaining > 0 )
        {
            printer.newPage();
        }

        pageData.step();
    }

    return( Err_Success );
}
    

int SLHexBuffer::printAscii( CHexPrinter &printer, bool returnCR)
{
    printer.setDocName( /*mUrl*/ "NONAME" );

    QPainter paint( &printer );
    //paint.setFont( font() );
    QFontMetrics fm( KGlobalSettings::fixedFont() );
    QByteArray binaryString;
    QString lineString;
    QChar c;


    paint.setPen( Qt::black );
    paint.setFont( KGlobalSettings::fixedFont() );

    SPageMargin margin = printer.pageMargin();
    SPageSize size = printer.pageUsableSize();
    paint.setClipRect( margin.left, margin.top, size.width, size.height );

    //printf("%d,%d,%d,%d\n", margin.left, margin.top, size.width, size.height );

    int headHeight, footHeight, headMargin, footMargin, freeHeight;

    headHeight = footHeight = headMargin = footMargin = 0;
    if( printer.pageHeader().enable == true )
    {
        headHeight = headerHeight( paint );
        headMargin = headerMargin( paint );
    }
    if( printer.pageFooter().enable == true )
    {
        footHeight = headerHeight( paint );
        footMargin = headerMargin( paint );
    }
    freeHeight = size.height - headHeight - footHeight - headMargin - footMargin;

    float scale = 1.0;

    //Each line must contain 80 fixed-size chars
    lineString.fill( QChar('8'), 80 );
    uint maxLineWidth = fm.width( lineString );

    if( maxLineWidth > size.width && printer.scaleToFit() == true )
    {
        scale = (float)size.width / (float)maxLineWidth;
        paint.scale( scale, scale );
    }

    uint linePerPage = (uint) ((float)freeHeight/((float)lineHeight()*scale));
    uint sx = (uint) ((float)margin.left/scale);
    uint sy = (uint) ((float)(margin.top+headHeight+headMargin)/scale);

    uint remaining = getLFCount();
    uint line = 0;

    if( printer.all() == true )
    {
        line = 0;
        remaining = getLFCount();
    }
    else 
    {
        return( Err_IllegalMode );
    }

    SPagePosition pageData( time(0), remaining, linePerPage );

    uint totLines = remaining;
    uint i = 0;
      
    while( remaining > 0 )
    {
        i = totLines - remaining;
        
        uint lineInPage = remaining > linePerPage ? linePerPage : remaining;
        uint y = sy;

    //
    // Draw header and footer. Reset scaling during that operation.
    //
        paint.scale( 1.0/scale, 1.0/scale );
        if( printer.pageHeader().enable == true )
        {
            drawHeader( paint, margin.left, size.width, margin.top, false,
            printer.pageHeader(), pageData );
        }

        if( printer.pageFooter().enable == true )
        {
            drawHeader( paint, margin.left, size.width,
                    margin.top+size.height-footHeight, true,
                    printer.pageFooter(), pageData );
        }
        paint.scale( scale, scale );

        //
        // Draw actual data
        //
        uint begOff = 0;
        uint endOff = 0;



        for( uint i=0; i < lineInPage; i++, line++ )
        {
            begOff = getOffset( line );
            endOff = getOffset( line + 1 );

            ASSERT(begOff <= endOff);

            binaryString.setRawData( &data()[ begOff ] , endOff - begOff );
            

            //strip away the unprintable chars and
            //builds a QString to be printed

            lineString = "";
            for (uint f = 0; f < binaryString.size(); f++)
            {
                c = binaryString[f];
                if (c.latin1() != 0)
                {
                    lineString.append(c.latin1());
                }
            }
            binaryString.resetRawData( &data()[ begOff ] , endOff - begOff );

            y += lineHeight();
            if (lineString.length() > 0)
            {
                if ( (sx + fm.width( lineString ) ) >= size.width )
                {
                    //paper limit
                    sx = 0;
                }

                paint.drawText( sx, y, lineString );
            }

            if (returnCR == true)
            {
                //carriage return + line feed needs the
                //CR+LF sequence. The LF was already considered
                //as line separators; if the last character of
                //binaryString is a CR, sx = 0. Otherwise sx is
                //incremented by the lineString length (pixels)
                if ( c == QChar(CR) )
                {
                    sx = 0;
                }
                else
                {
                    sx += fm.width( lineString );

                }

            }
            else if (returnCR == false)
            {
                //carriage return + line feed needs the
                //LF only. CRs are ignored
                sx = 0;
            }

        }


        remaining -= lineInPage;
        if( remaining > 0 )
        {
            printer.newPage();
        }

        pageData.step();
    }

    return( Err_Success );
}

uint SLHexBuffer::getCursorOffset()
{
    SCursorState cs = cursorState();

    return cs.offset;
}

/**
 *
 */
bool SLHexBuffer::saveBuffer( const QString& newName )
{
    int status;
    int he;
    CProgress p;

    QString oldName = url();
    QDir dir;

    if (dir.exists(newName) == true)
    {
        //normal saving: the existing file is overwritten.
        //If it does not exist, we are doing a saveAs
        if (dir.remove(newName) == false)
        {
            qDebug("%s:%u - error removing %s", __FILE__, __LINE__, 
                    newName.latin1());
            return false;        
        }
    }
    QFile file( newName );
    file.open( IO_WriteOnly );
    status = file.status();
    if (status != IO_Ok)
    {
        qDebug("%s:%u - creating %s bad file status %d", __FILE__, __LINE__, 
                newName.latin1(), status );
        return false;
    }
    
    setUrl( newName );
    he = writeFile( file, p );
    status = file.status();
    if (status != IO_Ok || he != Err_Success)
    {
        qDebug("%s:%u - writing on %s bad file status %d, hexerror=%d",
                __FILE__, __LINE__, newName.latin1(), status, he );
        file.close();
        
        //the buffer must remain unchanged after
        //this attempt has failed
        setUrl( oldName );
        return false;
    }

    file.flush();
    file.close();

    return true;
}

/**
 *
 */
bool SLHexBuffer::loadBuffer( const QString& name )
{
    CProgress p;
    int he;
    
    newFile( name );
    if ( QFile::exists( name ) )
    {
        QFile file( url() );
        file.open( IO_ReadOnly );
        he = readFile( file, url(), p );
        if (he != Err_Success)
        {
            qDebug( "%s:%u - failed reading buffer file %s, hexerror=%d", __FILE__, __LINE__, name.latin1(), he );
            file.close();
            return false;
        
        }
        file.close();
    }
    else
    {
        qDebug( "%s:%u - buffer file %s not found", __FILE__, __LINE__, name.latin1() );
    }
    return true;
}

