/*
 *                        slhexbuffer.h
 *                           -------------------
 *  begin                : Wed May 7 2003
 *  copyright            : (C) 2003 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * $Id: slhexbuffer.h,v 1.7 2004/12/10 16:19:13 gbertani Exp $ 
 */


#ifndef SLHEXBUFFER_H
#define SLHEXBUFFER_H

using namespace std;


#include <qobject.h>
#include <qdir.h>
#include <kglobalsettings.h>
#include "khexedit/hexbuffer.h"

//ASCII line separator
#define LF 0x0a

//Carriage Return
#define CR 0x0d

/**
 * Variant of CHexBuffer (from khexedit)
 * that can share the same data of another buffer
 * but with private layout
 * 
  *@author giuseppe massimo bertani
  */

class SLHexBuffer :  public QObject, public CHexBuffer
{
    Q_OBJECT

public:
	SLHexBuffer();
    
	SLHexBuffer( SLHexBuffer* baseBuffer );
	
    ~SLHexBuffer();

    /** */
	void takeSharedBlockFrom( SLHexBuffer* baseBuffer );
    
    /** this buffer shares the data area with the base buffer
     * but auxiliary data must be kept aligned by calling this
     * function
     */ 
    void alignBuffers( SLHexBuffer* buf = 0 );
  
    /**
     * flushes the buffer
     */
    void clear();
   
    /**
     *
     */
    bool saveBuffer( const QString& newName );
    
    /**
     *
     */
    bool loadBuffer( const QString& name );
    
    /**
     * returns the total number of lines
     */ 
    uint getLFCount() const;

    /**
     * returns the offset of a given line
     */
    uint getOffset( uint lineFeeds ) const;

    /**
     * returns the line that starts from the offset given 
     */
    uint getLF( uint offset ) const;

    
    /**
     * rebuilds the lineRef array
     */
    void computeLF();
    
    /**
     * updates lineRef by adding the 
     * given QByteArray lines
     */
    void computeLF( uint fromOffset,  const QByteArray& ba ); 
   
    /**
     * if c is a linefeed,
     * updates lineRef by adding the 
     * new line
     */
    void computeLF( uint fromOffset, const QChar c );
 
    /**
     * redefinition 
     */ 
    virtual bool inputAtCursor( QChar c );
    
    /**
     * redefinition 
     */ 
    virtual int  inputAtCursor( const QByteArray &buf, uint oldSize );
   

    /**
     * redefinition 
     */ 
    virtual int  print( CHexPrinter &printer  );
    
    /**
     * ASCII print 
     */ 
    int  printAscii( CHexPrinter& printer, bool returnCR = true );


    
    /** 
     * replaces the original, adding the signal
     * dataEntered() 
     */
    virtual void cursorRight( bool cellLevel );
    
    /** 
     * replaces the original, adding the signal
     * dataEntered() 
     */
    virtual void cursorStep( uint size, bool forward, bool modulo );


    void printLineRefs() const;
    
    /** 
     * replaces the original, adding the LF counting
     * 
     */
    virtual int readFile( QFile &file, const QString &url, CProgress &p );
    
    /**
     *
     */ 
    uint getCursorOffset();
    
protected:

    QArray<uint>& getLineRef()
    {
        return lineRef;
    }
    
    void setLineRef( const QArray<uint>& lf )
    {
        lineRef = lf;
    }

signals:    
    /** */
    void dataEntered();

    /** */
    void updateProgressBar( int progress, void* requester );
    
private:
   

    
    /** */
    SLHexBuffer*    srcBuffer;

    /** */
    array_data*     shared_data; 

    /** couples line numbers with the relative byte offset */
    QArray<uint>    lineRef;
};



#endif
