/*
 *                        slhexviewwidget.cpp
 *                           -------------------
 *  begin                : Mon Apr 28 2003
 *  copyright            : (C) 2003 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * 
 */

static char* cvs_id = "$Id: slhexviewwidget.cpp,v 1.11 2005/01/21 08:40:58 gbertani Exp $";

#include "slhexbuffer.h"
#include "slhexviewwidget.h"

SLHexViewWidget::SLHexViewWidget(QWidget *parent, const char *name, SLHexBuffer *srcBuffer, 
        bool readOnly ) :
    CHexViewWidget( parent, name,  new CHexBuffer ) // <--creates a fake buffer, that is trashed ASAP, since the constructor needs one.
{

    qDebug( "%s:%u - building slhexviewwidget", __FILE__, __LINE__ );      
    privateBuffer = 0;
    sharedBuffer = 0;

    CHECK_PTR( srcBuffer );
    setBuffer( srcBuffer );
          
    //delete fakeBuffer;              

    updateAttributes();
    
    setMute( readOnly );
    setBase( "hexadecimal" );
    setBytesPerLine(16);

    enterOffset = offset();
 
    connect( mVertScroll, SIGNAL(valueChanged(int)), this, SLOT(slotValueChanged(int)) );
    
    qDebug( "%s:%u - slhexviewwidget ok", __FILE__, __LINE__ );      
}

SLHexViewWidget::~SLHexViewWidget()
{
    qDebug( "%s:%u - destroying SLHexViewWidget", __FILE__, __LINE__ );  
    delete privateBuffer;
 
}

void SLHexViewWidget::updateAttributes()
{
    SDisplayColor dc;

    dc.useSystemColor   = true; //???
    dc.offsetBg         = KGlobalSettings::baseColor();
    dc.textBg           = KGlobalSettings::baseColor();
    dc.secondTextBg     = KGlobalSettings::baseColor();
    dc.inactiveBg       = KGlobalSettings::alternateBackgroundColor();
    dc.offsetBg         = KGlobalSettings::baseColor();
    dc.offsetBg         = KGlobalSettings::baseColor();
    dc.selectBg         = KGlobalSettings::highlightColor();
    dc.selectFg         = KGlobalSettings::highlightedTextColor();
    dc.markBg           = KGlobalSettings::highlightColor();
    dc.markFg           = KGlobalSettings::baseColor();
    dc.primaryFg[0]     = KGlobalSettings::textColor();
    dc.primaryFg[1]     = KGlobalSettings::linkColor();
    dc.offsetFg         = KGlobalSettings::visitedLinkColor();
    dc.secondaryFg      = KGlobalSettings::textColor();
    dc.nonPrintFg       = KGlobalSettings::activeTextColor();
    dc.gridFg           = KGlobalSettings::textColor();
    dc.leftSeparatorFg  = KGlobalSettings::textColor();
    dc.rightSeparatorFg = KGlobalSettings::textColor();
    dc.cursorBg         = KGlobalSettings::highlightColor();
    dc.cursorFg         = KGlobalSettings::textColor();
    dc.bookmarkBg       = KGlobalSettings::highlightColor();
    dc.bookmarkFg       = KGlobalSettings::textColor();

    setColor( dc, true );

    qDebug( "%s:%u - loading fixed font", __FILE__, __LINE__ );
    SDisplayFontInfo fi;
    fi.font = KGlobalSettings::fixedFont();
    fi.nonPrintChar = QChar('.');
    setFont( fi, true );
    
}

void SLHexViewWidget::slotBufferChanged()
{
    emit updateBuffer( privateBuffer );
    enterOffset = offset();
 

}

void SLHexViewWidget::selectDataChunk(uint from, uint size)
{
    if (size == 0)
    {
        setSelection( from, false );
        setSelection( from+size, false );
        
        //nothing to select
        qDebug("%s:%u - selectDataChunk(from %u, size %u) clearing selection",
            __FILE__, __LINE__, from, size);
    }
    else
    {
        qDebug( "%s:%u - selectDataChunk(from=%u, size=%u)",
            __FILE__, __LINE__, from, size);
    }
    setSelection( from, true );
    setSelection( from+size, false );
}



void SLHexViewWidget::slotValueChanged(int value)
{
//takes the scrollbar value and calculates
//the relative document offset
    if (value >= 0)
    {
        qDebug( "%s:%u - slotValueChanged(value=%d)", __FILE__, __LINE__, value);
        uint offset = (privateBuffer->documentSize() * mVertScroll->value()) /
            mVertScroll->maxValue();
    
        emit scrollBarChanged(offset);
    }
}
    
void SLHexViewWidget::updateView( bool redraw, bool fixCursor )
{
    /* 
     * every time something is received by the spoolers, it is written
     * in the relative source buffer in SLSession. After that a signal is raised
     * that is handled here. The source buffer and the private buffer
     * shares the same data area but there are some auxiliary fields that must
     * be keep aligned, for example the number of lines
     */ 
    
    privateBuffer->alignBuffers();
    privateBuffer->cursorEnd( true );
    enterOffset = offset();
    CHexViewWidget::updateView( redraw, fixCursor );
}

void SLHexViewWidget::setMute( bool status )
{
    SDisplayInputMode inputMode;
    
    if ( status == true )
    {
        inputMode.readOnly = true;
        inputMode.inputLock = true;
        inputMode.allowResize = false;
    }
    else
    {
        inputMode.readOnly = false;
        inputMode.inputLock = false;
        inputMode.allowResize = true;
    }
    
    setInputMode( inputMode );
    enterOffset = offset();
    isMute = status;
}

void SLHexViewWidget::setBase( const QString& base )
{
    SDisplayLayout layout = currLayout;
    if (base != "")
    {
        layout.setPrimaryMode( base );
    }
    layout.setSecondaryMode( "textOnly" );
    
    setLayout( layout );
    currLayout = layout;
    
    enterOffset = offset();
    updateView(true, true);
}

void SLHexViewWidget::setOffsetBase( bool wantHex )
{
    SDisplayLayout layout = currLayout;

    if(wantHex == true)
    {
        layout.setOffsetMode( "hexadecimal" );
    }
    else
    {
        layout.setOffsetMode( "decimal" );
    }

    setLayout( layout );
    currLayout = layout;

    enterOffset = offset();
    updateView(true, true);
}

void SLHexViewWidget::setBuffer( SLHexBuffer *hexBuffer )
{
    SLHexBuffer* oldBuffer = privateBuffer;
   
    sharedBuffer = hexBuffer;

    qDebug( "%s:%u - setBuffer() hexBuffer = %p", __FILE__, __LINE__, (void*)hexBuffer );
    CHECK_PTR( hexBuffer );
    privateBuffer = new SLHexBuffer( hexBuffer );
    qDebug( "%s:%u - setBuffer() privateBuffer = %p", __FILE__, __LINE__, (void*)privateBuffer );
    CHECK_PTR( privateBuffer );
    
    CHexViewWidget::setBuffer( static_cast<CHexBuffer*>(privateBuffer) );
    
    setMute( isMute );
    setBase( currLayout.primaryModeString() );
    setBytesPerLine( currLayout.lineSize );

    enterOffset = offset();
    updateView(true, true);

    connect(privateBuffer, SIGNAL( dataEntered() ), this, SLOT( slotBufferChanged() ));
    
    delete oldBuffer;
    
}

int SLHexViewWidget::print( CHexPrinter &printer )
{
    return sharedBuffer->print( printer );
}

void SLHexViewWidget::setBytesPerLine( uint bpl )
{
    SDisplayLayout layout = currLayout;
    layout.lineSize = bpl;    
    setLayout( layout );
    currLayout = layout;
    
    enterOffset = offset();
    updateView(true, true);
}

uint SLHexViewWidget::getBytesPerLine()
{
    return currLayout.lineSize;
}

#if 0
void SLHexViewWidget::setCursorPosition(int x, int y, bool init, bool cellLevel)
{

  x += startX();
  y += startY();

  if( mHexBuffer->setCursorPosition( x, y, init, cellLevel ) == false )
  {
    if( init == true )
    {
      unselect();
      unmark();
    }
  }
  else if( init == false )
  {
    SCursorConfig cc;
    cc.setKeepSelection( true );
    updateCursor( cc, false );
  }
  else
  {
    SCursorConfig cc;
    if( mHexBuffer->cursorInsideSelection() == true )
    {
      mDragManager->setup( x - startX(), y - startY() );
      cc.setKeepSelection( true );
      updateCursor( cc, true, false );
    }
    else
    {
      cc.setKeepSelection( false );
      updateCursor( cc, true );
    }
  }
}

#endif

void SLHexViewWidget::mousePressEvent( QMouseEvent *e ) 
{
    if ( e->button() == QMouseEvent::RightButton )
    {
        emit mouseRightClick();
       
    }
    else
    {
        CHexViewWidget::mousePressEvent(e);
    }
}



void SLHexViewWidget::paste( void )
{
    uint curr = offset();
    if (curr != enterOffset && isMute == false) 
    {
        gotoOffset( enterOffset );
    }
    CHexViewWidget::paste();
}




void SLHexViewWidget::keyPressEvent( QKeyEvent *e )  
{
    SCursorConfig cc;
    cc.state = e->state();


    //since the widget should know nothing
    //about serials, sessions, etc. if the 
    //keyboard is under the control of the views
    //we must route to the owner's view the interpretation
    //of nonprintable keystrokes depending from
    //the settings in the session dialog

    if( e->state() & ControlButton )
    {
        switch( e->key() )
        {
            //as in khexedit, ctrl-space toggles from the 
            //hex/oct/dec/bin pane to the ASCII pane
            //at the right and vice-versa.
            case Key_Space:
                e->accept();
                toggleEditor();
                break;

            default:
                //not used by the widget
                e->ignore();
                break;
                
        }
        return;
    }

    if( e->state() & AltButton ) 
    {
        //not used by the widget
        e->ignore();
        return;
    }

    //printable keys are managed always
    //by the widget
    if( (e->text()[0]).isPrint() == true )
    {
        uint curr = offset();
        if (curr != enterOffset && isMute == false) 
        {
            gotoOffset( enterOffset );
        }
        cursorInput( e->text()[0] );
        qDebug("%s:%u - pressed key %d", __FILE__, __LINE__, e->key() );
    }
    else
    {
        //other special keys
        
        switch ( e->key() ) 
        {
            case Key_Left:
                cursorLeft( cc );
                break;

            case Key_Right:
                cursorRight( cc );
                break;

            case Key_Up:
                cursorUp( cc );
                break;

            case Key_Down:
                cursorDown( cc );
                break;

            case Key_Home:
                cursorHome( cc );
                break;

            case Key_End:
                cursorEnd( cc );
                break;

            case Key_Next:
                cursorPageDown( cc );
                break;

            case Key_Prior:
                cursorPageUp( cc );
                break;

            case Key_Insert:
                cursorInsert( cc );
                break;

            default:
                //not used by the widget
                e->ignore();
                return;
        }
    }

    e->accept();
}



/**
 *
 */
void SLHexViewWidget::showOffset( uint offset )
{
    uint now = privateBuffer->cursorOffset();

    if (offset > now || offset < now)
    {
        gotoOffset(offset, 7, false, true);
    }

}

/** No descriptions */
void SLHexViewWidget::showBufferEnd()
{
    gotoOffset( privateBuffer->documentSize(), 7, false, true);

    /* the last attempt to solve the bug#3 */
    changeXPos(0);

}

/*
void SLHexViewWidget::closeEvent(QCloseEvent* e)
{

    QWidget::closeEvent(e);
    delete this;
}
*/
