/*
 *                        slhexviewwidget.h
 *                           -------------------
 *  begin                : Mon Apr 28 2003
 *  copyright            : (C) 2003 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * 
 * $Id: slhexviewwidget.h,v 1.8 2005/01/21 08:40:58 gbertani Exp $
 */


#ifndef SLHEXVIEWWIDGET_H
#define SLHEXVIEWWIDGET_H

#include <qwidget.h>
#include <qgarray.h>
#include <kglobalsettings.h>
#include "khexedit/hexviewwidget.h"

class SLHexBuffer;
struct SDisplayState;

/**
  *@author giuseppe massimo bertani
  */

class SLHexViewWidget : public CHexViewWidget  
{
   Q_OBJECT

public: 
	SLHexViewWidget( QWidget *parent, const char *name, SLHexBuffer *srcBuffer, 
            bool readOnly );
	~SLHexViewWidget();

    /** sets the numerical base of the widget */ 
    void setBase( const QString& base );

    /** sets the numerical base of the offset column */
    void setOffsetBase( bool wantHex );

    
    /** toggle RO flag */ 
    void setMute( bool status );

    /** set number of bytes per line */
    void setBytesPerLine( uint bpl );
    
    /** */
    uint getBytesPerLine();

    /** */
    int print( CHexPrinter &printer );

    /** */
    SLHexBuffer* hexBuffer()
    {
        return privateBuffer;    
    }
   
    /** */
    void setBuffer( SLHexBuffer *hexBuffer );
    
    /** */
    void updateView( bool redraw, bool fixCursor );
    
    /** from CHexViewWidget */
    virtual void paste( void );

    /**  */
    void showOffset( uint offset );
    
    /**  */
    void showBufferEnd();

    /** */
    void selectDataChunk(uint from, uint size);
    
protected slots:
    /** */
    void slotBufferChanged();
    
    /** */
    void slotValueChanged(int);

#if 0
    /** overrides the one in CHexViewWidget */
    void setCursorPosition(int x, int y, bool init, bool cellLevel);
#endif

protected:

    /** from CHexViewWidget */
    virtual void keyPressEvent( QKeyEvent *e );
    
    /** */
    virtual void mousePressEvent( QMouseEvent * e ); 

    /** Applies global colors and fonts from control panel */
    virtual void updateAttributes();
    
    //virtual void closeEvent(QCloseEvent* e);
 
signals:
    
    /** 
     * notifies the receiver that in the buffer 
     * privateBuf were inserted new data 
     */ 
    void updateBuffer(SLHexBuffer* privateBuf);

    /** */
    void mouseRightClick();
    
    /** */
    void scrollBarChanged(uint);

    
private:

    /** true if the widget will be used only to display data
     * instead of entering them */
    bool isMute;

    /** I need a copy of this here, because there is no way to
     * retrieve the current layout from CHexViewWidget */
    SDisplayLayout currLayout;
   
    /** private hex buffer */
    SLHexBuffer* privateBuffer;
    
    /** source hex buffer (shared by SLSession) */
    SLHexBuffer* sharedBuffer;
 
    /** offset of the next byte to enter */
    uint enterOffset;

};


#endif
