/*
 *                        sllogbrowser.cpp  -  description
 *                           -------------------
 *  begin                : gio feb 12 2004
 *  copyright            : (C) 2004 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * 
 */
static char* cvs_id = "$Id";


#include "sllogbrowser.h"

SLLogBrowser::SLLogBrowser( QWidget *parent, const char *name ) :
 QTextEdit( parent, name )
{
    setTextFormat( Qt::PlainText );
    setFont( KGlobalSettings::fixedFont() );
    setMouseTracking( false );
    setWordWrap( NoWrap );
    headingSize = 0;
    
    QScrollBar* sb = verticalScrollBar();
    
    connect(sb, SIGNAL( valueChanged(int) ), this, SLOT( slotValueChanged(int) ));
    ignoreSignals = false;
}

SLLogBrowser::~SLLogBrowser()
{
    qDebug( "%s:%u - destroying SLLogBrowser", __FILE__, __LINE__ );
}

QPopupMenu* SLLogBrowser::createPopupMenu ( const QPoint & pos )
{
    //parameter <pos> intentionally ignored

    emit mouseRightClick();

    // I hope this does not hurt...
    return 0;
}

void SLLogBrowser::slotValueChanged(int value)
{
    //convert scrollbar value to paragraph
    if (value >= 0 && ignoreSignals == false)
    {
        yHeight = contentsHeight();
        totPara = paragraphs();
        currPara = ((value * totPara) / yHeight) + headingSize;
        qDebug( "%s:%u - slotValueChanged(value=%d), para=%u", __FILE__, __LINE__,
            value, currPara);
        emit scrollBarChanged(currPara);

    }
}


int SLLogBrowser::getLineHeight() const
{
    QFontMetrics fm( font() );
    return fm.height();
}

int SLLogBrowser::getLineWidth( const QString& s ) const
{
    QFontMetrics fm( font() );
    return fm.width( s );
}

uint SLLogBrowser::getShowedEvents() const
{
    int events = (paragraphs() - headingSize);
    if (events < 0)
    {
        events = 0;
    }

    return static_cast<uint>(events);
}


void SLLogBrowser::showEvent( uint event )
{
    ignoreSignals = true;

    //the scrollbar works with pixels while
    //the browser works with text lines
    //so we need to numerate starting from zero
    //instead of 1
    currPara = event + 1;

    setSelection(currPara, 0, currPara, paragraphLength(currPara));

    QScrollBar* sb = verticalScrollBar();

    //moves the scrollbar only if the requested event
    //is not visible


    uint firstPara = static_cast<uint>(paragraphAt(QPoint(contentsX(),contentsY())));
    uint lastPara = static_cast<uint>(paragraphAt(QPoint(contentsX(),
        (contentsY() + visibleHeight() - 1))));
    qDebug( "%s:%u - firstPara:%d lastPara %d", __FILE__, __LINE__ ,
        firstPara, lastPara);

    if (firstPara >= currPara || lastPara <= currPara)
    {
        sync();
        yHeight = contentsHeight();
        totPara = paragraphs();
        uint pixelY = (currPara * yHeight) / totPara;
        sb->setValue(pixelY);
        qDebug( "%s:%u - Moving scrollbar: totPara:%u pixelY:%u, yHeight:%d", __FILE__, __LINE__ ,
            totPara, pixelY, yHeight);
    }
    sb = horizontalScrollBar();
    sb->setValue(0);
    
    ignoreSignals = false;
}

void SLLogBrowser::append( const QString& text )
{
    ignoreSignals = true;
    QTextEdit::append(text);
    ignoreSignals = false;
}
