/*
 *                        slserialport.cpp
 *                           -------------------
 *  begin                : Tue Dec 10 2002
 *  copyright            : (C) 2002 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * 
 */

static char* cvs_id = "$Id: slserialport.cpp,v 1.6 2004/12/10 16:19:13 gbertani Exp $";

#include "serlook.h"
#include "slserialport.h"

#define SAMPLE_MS 10

/**
 *
 *
 *
 *
 */ 
SLSerialPort::SLSerialPort(QString& dev, struct termios& opts)
{
    device = dev;
    options = opts;
    fd = 0;
    tmr = 0;
    modem = 0;
    rx_data = false;
    tx_data = false;
    
}

/**
 *
 *
 *
 *
 */ 
uint SLSerialPort::getModemStatus() const
{
    return modem;
}


/**
 *
 *
 *
 *
 */ 
void SLSerialPort::slotCheckModem()
{
unsigned ms;
static ulong calls = 0; //only for debug purposes

    calls++;
    //qDebug("%s:%u - slotCheckModem() call nr. %lu", __FILE__, __LINE__, calls);
    
    if (fd == 0)
    {
        if (modem != 0)
        {
            //qDebug("%s:%u - slotCheckModem() status changed from %08Xh to zero (port closed)", __FILE__, __LINE__, modem);
            modem = 0;
            emit statusChanged();
        }
        return;
    }
        
    ioctl(fd, TIOCMGET, &ms);
    
    //clears the higher bits (17...31) that 
    //should not be used
    //(statserial shows only the lowest 16)
    ms &= 0x0000ffff;

    //ignores the txd and rxd signals from the driver
    ms &= ~TIOCM_ST;
    ms &= ~TIOCM_SR;
    
    if (tx_data == true)
    {
        //txd emulation
        ms |= TIOCM_ST;
    }

    if (rx_data == true)
    {
        //rxd emulation
        ms |= TIOCM_SR;
    }

    if (ms != modem)
    {
        //qDebug("%s:%u - slotCheckModem() status changed from %08Xh to %08Xh", __FILE__, __LINE__, modem, ms);
        modem = ms;
        emit statusChanged();
    }
}

/**
 *
 *
 *
 *
 */ 
void SLSerialPort::slotCheckRx()
{
uint retval;

    rx_data = false;

    if (fd == 0)
    {
        //port closed. It should not happen because
        //the timer is stopped when the port is closed
        //but some events could still be enqueued and they 
        //must be flushed

        modem = 0;
    }
    else
    {
    
        ioctl(fd, FIONREAD, &retval);
        if (retval > 0)
        {
            rx_data = true; 
            emit rxReady();
        }
    }
    slotCheckModem();

}

/**
 *
 *
 *
 *
 */ 
SLSerialPort::~SLSerialPort()
{
    if (fd != 0)
    {
        close();
    }
    modem = 0;
    slotCheckModem();
}

/**
 *
 *
 *
 */ 
bool SLSerialPort::open( int mode )
{
    //NOTE: the mode parameter is not used. It will be
    //always IO_ReadWrite

    setState( 0 );
    
    if (mode != IO_ReadWrite)
    {
        qDebug("%s:%u - SLSerialPort.open() failed, mode=%d not supported", __FILE__, __LINE__, mode);
        setStatus( IO_OpenError );
        return false;
    }
    
    setMode( IO_ReadWrite );
    setFlags( IO_ReadWrite );
   
    //O_NDELAY ignore the DCD line status
    //The reading is always blocking, please
    //check data available with size() before
    //issue readBlock() or getch() calls
    fd = ::open( device.latin1(), O_RDWR|O_NOCTTY|O_NDELAY );
    if(fd < 0)
    {
        qDebug("%s:%u - SLSerialPort.open() failed, fd=%d ", __FILE__, __LINE__, fd);
        setStatus( IO_OpenError );
        fd = 0;
        return false;
    }

    ::tcsetattr( fd, TCSANOW, &options );

    setStatus( IO_Ok );
    setState( IO_Open );
    setFlags( IO_ReadWrite | IO_Open );
    
    tmr = new QTimer(this, 0);
    CHECK_PTR( tmr );
    
    connect (tmr, SIGNAL(timeout(void)), this, SLOT(slotCheckRx(void)));
    
    tmr->start( SAMPLE_MS );
    slotCheckModem();
    return true;
}
    
/**
 *
 *
 *
 */ 
void SLSerialPort::close()
{
    delete tmr;
    tmr = 0;
    
    ::tcdrain(fd);
    int retval = ::close(fd);
    ASSERT(retval >= 0);
    fd = 0;
    slotCheckModem();
    
}
 

/**
 *
 *
 *
 */ 
void SLSerialPort::flush()
{
    if(::tcflush(fd ,TCIOFLUSH) != 0)
    {
        setStatus( IO_WriteError );
    }
    slotCheckModem();
}
   
/**
 *
 *
 *
 *
 */
#if QT_VERSION >= 300
QIODevice::Offset SLSerialPort::size() const
{
    uint retval;

    ioctl(fd, FIONREAD, &retval);
    return static_cast <QIODevice::Offset> (retval);
}

/**
 *
 *
 *
 *
 */
Q_LONG SLSerialPort::readBlock( char *data, Q_ULONG maxlen )
{
    int retval;

    slotCheckModem();
    retval = read(fd, data, maxlen);
    if(retval <= 0)
    {
        if (errno == EAGAIN)
        {
            return 0;
        }
        else
        {
            qDebug("%s:%u - SLSerialPort.readBlock() failed, errno=%d ", __FILE__, __LINE__, errno);
            setStatus( IO_ReadError );
        }
    }

    return static_cast<Q_LONG>(retval);
}

/**
 *
 *
 *
 *
 */
Q_LONG SLSerialPort::writeBlock( const char *data, Q_ULONG len )
{
    int retval = 1;
    int intLen = static_cast <int> (len);
    int i = 0;
    
    tx_data = true; 
    slotCheckModem();
    do
    {
        retval = write(fd, &data[i], intLen);
        if (retval != intLen && retval > 0)
        {
            //device output buffer full
            //waits some mS then tries 
            //to send the remaining bytes
            usleep(1000 * SAMPLE_MS);
            intLen -= retval;
            i += retval;
        }
    }
    while (retval != intLen && retval > 0);
    
    tx_data = false; 
    //slotCheckModem();
    
    if(retval < 0)
    {
        qDebug("%s:%u - SLSerialPort.writeBlock() failed, errno=%d ", __FILE__, __LINE__, errno);
        setStatus( IO_WriteError );
    }

    return static_cast <QIODevice::Offset> (retval);
}

/**
 * not implemented (noncanonical input mode)
 *
 *
 *
 */
Q_LONG SLSerialPort::readLine( char *data, Q_ULONG maxlen )
{

    qDebug("%s:%u - SLSerialPort.readLine(%p, %lu) is not implemented",
        __FILE__, __LINE__, (void*)data, maxlen);
    return -1;
}




#else

uint SLSerialPort::size() const
{
    uint retval;
    
    
    ioctl(fd, FIONREAD, &retval);
    return retval;
}

/**
 *
 *
 *
 *
 */ 
int SLSerialPort::readBlock( char *data, uint maxlen )
{
    int retval;
    
    slotCheckModem();
    retval = read(fd, data, maxlen);
    if(retval <= 0)
    {
        if (errno == EAGAIN)
        {
            return 0;
        }
        else
        {
            qDebug("%s:%u - SLSerialPort.readBlock() failed, errno=%d ", __FILE__, __LINE__, errno);
            setStatus( IO_ReadError );
        }
    }

    return retval;
}

/**
 *
 *
 *
 *
 */ 
int SLSerialPort::writeBlock( const char *data, uint len )
{
    int retval = 1;
    int intLen = static_cast <int> (len);
    int i = 0;
    
    tx_data = true; 
    slotCheckModem();
    do
    {
        retval = write(fd, &data[i], intLen);
        if (retval != intLen && retval > 0)
        {
            //device output buffer full
            //waits some mS then tries 
            //to send the remaining bytes
            usleep(1000 * SAMPLE_MS);
            intLen -= retval;
            i += retval;
        }
    }
    while (retval != intLen && retval > 0);
    
    tx_data = false; 
    //slotCheckModem();
    
    if(retval < 0)
    {
        qDebug("%s:%u - SLSerialPort.writeBlock() failed, errno=%d ", __FILE__, __LINE__, errno);
        setStatus( IO_WriteError );
    }

    return retval;
}

/**
 * not implemented (noncanonical input mode)
 *
 *
 *
 */ 
int SLSerialPort::readLine( char *data, uint maxlen )
{
    
    qDebug("%s:%u - SLSerialPort.readLine() is not implemented", __FILE__, __LINE__);
    return -1;
}

#endif  //QT3


/**
 *
 *
 *
 *
 */ 
int SLSerialPort::getch()
{
int retval;
char data;
    
    slotCheckModem();
    retval = read(fd, &data, 1);
    if(retval < 0)
    {
        if (errno == EAGAIN)
        {
            // no data ready
            return -2;
        }
        else
        {
            qDebug("%s:%u - SLSerialPort.getch() failed, errno=%d", __FILE__, __LINE__, errno);
            setStatus( IO_ReadError );
            return -3;
        }
        
    }

    if (retval == 0)
    {
        //EOF
        return -1;
    }
    
    return data;
}

/**
 *
 *
 *
 *
 */ 
int SLSerialPort::putch(int data)
{
int retval = 1;
    
    tx_data = true;
    slotCheckModem();
    do
    {
        retval = write(fd, &data, 1);
        if (retval <= 0)
        {
            usleep(1000 * SAMPLE_MS);
        }
    }
    while (retval <= 0 && errno == EAGAIN);
    
    tx_data = false;
    //slotCheckModem();
    
    if(retval <= 0)
    {
        qDebug("%s:%u - SLSerialPort.putch() failed, errno=%d", __FILE__, __LINE__, errno);
        setStatus( IO_WriteError );
        return -1;
    }
    
    return data;
}

/**
 *
 *
 *
 *
 */ 
int SLSerialPort::ungetch( int data )
{
    qDebug("%s:%u - SLSerialPort.ungetch(%d) is not implemented", __FILE__, __LINE__, data);
    return -1;
}



