/*
 *                        slserialport.h
 *                           -------------------
 *  begin                : Tue Dec 10 2002
 *  copyright            : (C) 2002 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB.h> skips=4
 *  <TAB.h> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * $Id: slserialport.h,v 1.6 2004/12/10 16:19:13 gbertani Exp $
 * 
 */


#ifndef SLSERIALPORT_H
#define SLSERIALPORT_H

using namespace std;


//linux
#include <termios.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>

//Qt
#include <qtimer.h>
#include <qiodevice.h>

/**Allows low level access to serial 
ports making also easier to configure
termios flags
  *@author giuseppe massimo bertani
  */

class SLSerialPort : public QObject, public QIODevice  
{
    Q_OBJECT 
        
public: 
    SLSerialPort(QString& dev, struct termios& opts);
    
    ~SLSerialPort();
    
    virtual bool open( int mode );
    
    virtual void close();
    
    virtual void flush();


#if QT_VERSION >= 300
    /** returns the number of incoming bytes to be read
     * since serial devices have no size */
    virtual Offset size() const;
    
    virtual Q_LONG readBlock( char *data, Q_ULONG maxlen );
    virtual Q_LONG writeBlock( const char *data, Q_ULONG len );
    virtual Q_LONG readLine( char *data, Q_ULONG maxlen );
#else
    /** returns the number of incoming bytes to be read
     * since serial devices have no size */
    virtual uint size() const;
    
    virtual int readBlock( char *data, uint maxlen );
    virtual int writeBlock( const char *data, uint len );
    virtual int readLine( char *data, uint maxlen );
#endif

    
    virtual int getch();
    virtual int putch( int data );
    virtual int ungetch( int data ); 

    /*
     * Modem lines definitions under linux 2.4:
     * #define TIOCM_LE	    0x001   
     * #define TIOCM_DTR	0x002
     * #define TIOCM_RTS	0x004
     * #define TIOCM_ST	    0x008
     * #define TIOCM_SR	    0x010
     * #define TIOCM_CTS	0x020
     * #define TIOCM_CAR	0x040
     * #define TIOCM_RNG	0x080
     * #define TIOCM_DSR	0x100
     * #define TIOCM_CD	    TIOCM_CAR
     * #define TIOCM_RI	    TIOCM_RNG
     * #define TIOCM_OUT1	0x2000
     * #define TIOCM_OUT2	0x4000
     * #define TIOCM_LOOP	0x8000
    */

    
    bool dsr() const
    {
        return (modem & TIOCM_DSR) != 0;
    }

    bool dtr() const
    {
        return (modem & TIOCM_DTR) != 0;
    }

    bool rts() const
    {
        return (modem & TIOCM_RTS) != 0;
    }


    bool cts() const
    {
        return (modem & TIOCM_CTS) != 0;
    }

    bool dcd() const
    {
        return (modem & TIOCM_CD) != 0;
    }

    bool ring() const
    {
        return (modem & TIOCM_RNG) != 0;
    }
    
    /**
     * It seems that the following signal is not managed by the linux
     * serial driver. It will be driven by this class methods
     */ 
    bool txd() const
    {
        //from serial.pdf: what means "secondary"?
        return (modem & TIOCM_ST) != 0;
    }

    /**
     * It seems that the following signal is not managed by the linux
     * serial driver. It will be driven by this class methods
     */ 
    bool rxd() const
    {
        //from serial.pdf: what means "secondary"?
        return (modem & TIOCM_SR) != 0;
    }

    uint getModemStatus() const;
    
public slots:
    //check if data ready to get
    void slotCheckRx(void);

    //check if modem status lines changed
    void slotCheckModem(void);

signals:
    //data ready to get
    void rxReady(void);

    //modem status lines changed
    void statusChanged(void);

private:    
    
    //serial port device name
    QString device;
    
    //and other parameters: data size, 
    //parity, etc.
    struct termios options;
    
    //serial port file descriptor 
    int fd; 
   
    //modem lines status from device driver
    unsigned modem;

    //emulated txd status
    bool tx_data;
    
    //emulated rxd status
    bool rx_data;
    
    //port sampling timer
    QTimer* tmr;
    
};



#endif
