/*
 *                        slsession.cpp
 *                           -------------------
 *  begin                : mer dic  4 22:54:15 CET 2002
 *  copyright            : (C) 2002 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 * 
 */

static char* cvs_id = "$Id: slsession.cpp,v 1.14 2005/01/21 08:40:58 gbertani Exp $";

#include "slhexbuffer.h"
#include "serlook.h"
#include "sleventrecord.h"
#include "sldumpview.h"
#include "slterminalview.h"
#include "slsessiondialog.h"
#include "slserialport.h"
#include "slsession.h"
#include "slview.h"

//Statics
const uint SLSession::fileFormatVersion = SER_FILE_VER;

SLSession::SLSession(QWidget* const clientArea, const KURL &url)
{
    qDebug("%s:%u - Creating session", __FILE__, __LINE__);

    /*
     * Creates the session settings dialog
     */ 
    dlg = new SLSessionDialog( kapp->mainWidget() );    
    CHECK_PTR( dlg );

    connect( dlg, SIGNAL(commParamsChoice()), this, SLOT(slotChangedViewList()) );
    connect( dlg, SIGNAL(commLineChoice()), this, SLOT(slotCleanBuffers()) );
    
    splitter = new QSplitter(QSplitter::Vertical, clientArea);
    CHECK_PTR ( splitter );
   
    ws = new QWorkspace( splitter );
    CHECK_PTR ( ws );
    
    run = false;
    sendingFile = false;
    viewSyncEnable = true;
    
    // creating buffers
    inBufA = new SLHexBuffer;
    CHECK_PTR ( inBufA );

    outBufA = new SLHexBuffer;
    CHECK_PTR ( outBufA );
    
    inBufB = new SLHexBuffer;
    CHECK_PTR ( inBufB );
    
    getDialog()->slotAllDefaults();
    modified=false;
    doc_url = url;
    

    //serial lines, closed by default
    lineA = 0;
    lineB = 0;
    
    
    //
    // Creating data buffer files
    //

    QDir cwd;
    
    QString baseName = cwd.cleanDirPath( cwd.absFilePath( url.path() ) );
    baseName.truncate( baseName.length() - 4 );

    QString bufName = baseName + ".in_a.buf";
    inBufA->newFile( bufName );

    bufName = baseName + ".out_a.buf";
    outBufA->newFile( bufName );

    bufName = baseName + ".in_b.buf";
    inBufB->newFile( bufName );
    
    bufName = baseName + ".events.buf";
    evLog = new SLEventRecord( bufName );
    CHECK_PTR ( evLog );
    
    splitter->show();
}

SLSession::~SLSession()
{
    
    viewSyncEnable = false;
    slotStopAcquisition();
    
    QWidgetList wList = ws->windowList();

    QWidget* w = 0;
   
    for( w=wList.first(); w!=0;  w=wList.next())
    {
        /*
         * minimizes all the views before
         * destroying them
         */ 
        w->showMinimized();    
    }
    
    if (splitter != 0)
    {
        /*
         * The splitter must be destroyed
         * explicitly because it is owned
         * by the clientArea and not by 
         * this object
         */ 
        delete splitter;
        splitter = 0;
    }

    inBufA->closeFile();
    inBufB->closeFile();
    outBufA->closeFile();
    delete inBufA;
    delete inBufB;
    delete outBufA;
    if ( evLog != 0 )
    {
        delete evLog;
        evLog = 0;
    }
        
}


void SLSession::slotChangedViewList()
{    
    QWidgetList wList = ws->windowList();
   
    QWidget* w = 0;
    SLView *v = 0;
    QString dev, num, enable;
   
    /*
     * NOTE: the view must be showed before, otherwise
     * count() will return 0
     */ 
    

    int i;
    for( i=1, w=wList.first(); w!=0; i++, w=wList.next())
    {
        num.setNum(i);

        v = static_cast<SLView*>(w);
        BufferLine line = v->getLine();
        
        enable = "";
        
        switch( line )
        {
            case Primary_Read:
                dev = dlg->getLineADevice();
                break;

            case Primary_Write:
                dev = dlg->getLineADevice();
                if (dlg->getLineType() == SLSessionDialog::typeSerialTT)
                {
                    enable = i18n(" DISABLED");
                }
                break;
                
            case Secondary_Read:
                dev = dlg->getLineBDevice();
                if (dlg->getLineType() == SLSessionDialog::typeSerialP2P)
                {
                    enable = i18n(" DISABLED");
                }
                break;
        }
        v->setCaption( num, dev, enable );
    }

}

bool SLSession::isLastView() const 
{
    QWidgetList wList = ws->windowList();
    return (wList.count() == 1);
}

bool SLSession::noViews() const 
{
    QWidgetList wList = ws->windowList();
    return (wList.count() == 0);
}


SLView* SLSession::getActiveView() const
{
    QWidgetList wList = ws->windowList();
    
    QWidget* w = 0;
    SLView *v = 0;
    
    for(w=wList.first(); w!=0; w=wList.next())
    {
        if ( w == ws->activeWindow()) 
        {
            v = static_cast<SLView*>(w);

            return v;
        }
    }

    /* no active views */

    if (wList.count() == 1)
    {
        v = static_cast<SLView*>(wList.first());
        return v;
    }
    
    return 0;
}

SLView* SLSession::firstView() const
{ 
    QWidgetList wList = ws->windowList();
    SLView* v = static_cast<SLView*>(wList.first());
    return v; 
}

void SLSession::setURL(const KURL &url)
{
    doc_url=url;
}

const KURL& SLSession::URL() const
{
    return doc_url;
}

SLHexBuffer* const SLSession::getBuffer(BufferLine bl)
{
    switch(bl)
    {
        case Primary_Read:
            CHECK_PTR( inBufA );
            return inBufA;

        case Primary_Write:
            CHECK_PTR( outBufA );
            return outBufA;

        case Secondary_Read:
            CHECK_PTR( inBufB );
            return inBufB;
    
    }
    
    //should never happen
    ASSERT(0);
    return 0;
}

bool SLSession::loadDocument(const KURL &url)
{
   
    //Needed to implement network transparency
    QString tmpfile;
    KIO::NetAccess::download( url, tmpfile );

    if ( !QFile::exists( tmpfile ) )
    {
        KMessageBox::sorry( kapp->mainWidget(), i18n( "the given session file does not exist: " ) + tmpfile );
        return false;
    }
    
    KConfig data(tmpfile, true, false, "");
    
    data.setGroup("Session file info");
    uint version = data.readUnsignedNumEntry("Version");

    if (version != fileFormatVersion)
    {
        int status = KMessageBox::warningContinueCancel( kapp->mainWidget(), 
                QString( i18n( "Session file format version " ) ).arg(version).arg( i18n( 
                        " not supported (should be " ) ).arg(fileFormatVersion).arg( ")" ) , 
                i18n( "Unsupported session file format version" ),
                i18n( "Continue loading" ) );

        if (status == KMessageBox::Cancel)
        {
            return false;
        }
    }

    viewSyncEnable = false;
    QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
    
    data.setGroup("Client Area");
    QValueList<int> sizesList = data.readIntListEntry("I/O splitter");

    data.setGroup("Communication Line");
    dlg->setLineType( (SLSessionDialog::TypeItems) data.readUnsignedNumEntry("Link Type") );
    dlg->setLineADevice( data.readPathEntry("device A") );
    dlg->setLineBDevice( data.readPathEntry("device B") ); 
    data.setGroup("Communication Parameters");
    dlg->setSpeed( (SLSessionDialog::BaudrateItems) data.readUnsignedNumEntry("Standard Speed") );
    dlg->setCustomSpeed( data.readUnsignedNumEntry("Custom Speed") );
    dlg->setSize( (SLSessionDialog::SizeItems) data.readUnsignedNumEntry("Data Size") );
    dlg->setStops( (SLSessionDialog::StopsItems) data.readUnsignedNumEntry("Stop Bits") );
    dlg->setParity( (SLSessionDialog::ParityItems) data.readUnsignedNumEntry("Parity") );
    dlg->setFlow( (SLSessionDialog::FlowControlItems) data.readUnsignedNumEntry("Flow Control") );

    
    data.setGroup("Views Data");
    uint toOpen = data.readUnsignedNumEntry("Opened Views");
    
    SLView* v = 0;
    
    emit setProgressBarRange(toOpen);
    emit updateStatusBar( i18n("Opening views...") );
    emit updateProgressBar(0, this);
        
    for( uint n = 1; n <= toOpen; n++ )
    {
        data.setGroup( QString("View %1").arg(n) );
       
        QString viewClass = data.readEntry("Class");

        // first creates the new view 
        if ( viewClass == QString("SLDumpView") )
        {
            emit createDumpView();
        }
        else if ( viewClass == QString("SLTerminalView") )
        {

            emit createTerminalView();
        }
        else if ( viewClass == QString("SLEventView") )
        {

            emit createEventView();
        }
        
        kapp->processEvents();
        v = static_cast<SLView*>( ws->activeWindow() );
        CHECK_PTR ( v );
        
        v->load( &data );
           
        connect( this, SIGNAL( updateAllViews( BufferLine ) ), v, SLOT( slotUpdateView( BufferLine ) ) );
        
        emit updateProgressBar(n, this);
        
    }
    
    splitter->setSizes( sizesList );


    KIO::NetAccess::removeTempFile( tmpfile );
    doc_url=url;
    emit updateStatusBar( "" );

    //
    //reloading buffers
    //
    
    CProgress p; //not used, needed only because khexedit derived classes want it
    QDir cwd;
    
    emit setProgressBarRange(3);
    emit updateStatusBar( i18n("Reloading log files...") );
    emit updateProgressBar(0, this);
    kapp->processEvents();

    QString baseName = cwd.cleanDirPath( cwd.absFilePath( doc_url.path() ) );
    baseName.truncate( baseName.length() - 4 );

    QString bufName = baseName + ".in_a.buf";
    if (inBufA->loadBuffer( bufName ) == true)
    {
        emit updateAllViews( Primary_Read );
        emit updateProgressBar(1, this);
        kapp->processEvents();
    }
    else
    {
        qDebug( "%s:%u - Primary port input buffer file %s not found, it will be created from scratch", __FILE__, __LINE__, bufName.latin1() );
    }

    bufName = baseName + ".out_a.buf";
    if (outBufA->loadBuffer( bufName ) == true)
    {
        emit updateAllViews( Primary_Write );
        emit updateProgressBar(2, this);
        kapp->processEvents();
    }
    else
    {
        qDebug( "%s:%u - Primary port output buffer file %s not found, it will be created from scratch", __FILE__, __LINE__, bufName.latin1() );
    }
    
    bufName = baseName + ".in_b.buf";
    if (inBufB->loadBuffer( bufName ) == true)
    {
        emit updateAllViews( Secondary_Read );
        emit updateProgressBar(3, this);
    }
    else
    {
        qDebug( "%s:%u - Secondary port input buffer file %s not found, it will be created from scratch", __FILE__, __LINE__, bufName.latin1() );
    }

    bufName = baseName + ".events.buf";
    if (evLog->loadBuffer( bufName ) == true)
    {
        emit updateProgressBar(4, this);
    }
    else
    {
        qDebug( "%s:%u - Event log file %s not found, it will be created from scratch", __FILE__, __LINE__, bufName.latin1() );
    }

    emit updateStatusBar( "" );
    emit updateProgressBar(-1, this);
    modified=false;
    
    QApplication::restoreOverrideCursor();
    viewSyncEnable = true;
    
    return true;
}

bool SLSession::saveDocument(const KURL &url)
{
    //The buffer files are written down again
    //in case the session is saved with a 
    //different name
    QDir cwd;

    viewSyncEnable = false;
    QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
    
    QString baseName = cwd.cleanDirPath( cwd.absFilePath( url.path() ) );
    baseName.truncate( baseName.length() - 4 );
    qDebug("%s:%u - saving current session as %s", __FILE__, __LINE__, baseName.latin1());

    //----------- input buffer primary line -----------
    QString bufName = baseName + ".in_a.buf";
    if (inBufA->saveBuffer( bufName ) == false)
    {
        qDebug("%s:%u - saveBuffer( %s ) failed", __FILE__, __LINE__, bufName.latin1());
        QApplication::restoreOverrideCursor();
        viewSyncEnable = true;
        return false;
    }
    
    //-------- output buffer primary line ---------------            
    bufName = baseName + ".out_a.buf";
    if (outBufA->saveBuffer( bufName ) == false)
    {
        qDebug("%s:%u - saveBuffer( %s ) failed", __FILE__, __LINE__, bufName.latin1());
        QApplication::restoreOverrideCursor();
        viewSyncEnable = true;
        return false;
    }

    //-----input buffer secondary line---------
    bufName = baseName + ".in_b.buf";
    if (inBufB->saveBuffer( bufName ) == false)
    {
        qDebug("%s:%u - saveBuffer( %s ) failed", __FILE__, __LINE__, bufName.latin1());
        QApplication::restoreOverrideCursor();
        viewSyncEnable = true;
        return false;
    }

    //-----event log file --------------
    bufName = baseName + ".events.buf";
    if (evLog->saveBuffer( bufName ) == false)
    {
        qDebug("%s:%u - saveBuffer( %s ) failed", __FILE__, __LINE__, bufName.latin1());
        QApplication::restoreOverrideCursor();
        viewSyncEnable = true;
        return false;
    }
    
    //
    //updating .ser session file
    //
    
    KConfig data(url.path(), false, false, "");
    
    data.setGroup("Session file info");
    data.writeEntry("Version", fileFormatVersion);
    
    data.setGroup("Client Area");
    data.writeEntry("I/O splitter", splitter->sizes());

    data.setGroup("Communication Line");
    data.writeEntry("Link Type", dlg->getLineType()); 
    data.writeEntry("device A", dlg->getLineADevice()); 
    data.writeEntry("device B", dlg->getLineBDevice()); 

    data.setGroup("Communication Parameters");
    data.writeEntry("Standard Speed", dlg->getSpeed()); 
    data.writeEntry("Custom Speed", dlg->getCustomSpeed()); 
    data.writeEntry("Data Size", dlg->getSize()); 
    data.writeEntry("Stop Bits", dlg->getStops()); 
    data.writeEntry("Parity", dlg->getParity()); 
    data.writeEntry("Flow Control",dlg->getParity()); 
    
    QWidgetList wList = ws->windowList();
    
    data.setGroup("Views Data");
    data.writeEntry("Opened Views", wList.count() ); 
    
    QWidget* w = 0;
    SLView *v = 0;
    
    uint n = 1;
    for( w=wList.first(); w!=0; w=wList.next() )
    {
        CHECK_PTR ( w );
        v = static_cast<SLView*>(w);
        data.setGroup( QString("View %1").arg(n) );
        v->save( &data );
        n++;
    }
    
    data.sync();
    
    QApplication::restoreOverrideCursor();
    viewSyncEnable = true;
    modified = false;
    return true;
}

void SLSession::slotCleanBuffers()
{
    QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
    
    inBufA->clear();
    inBufB->clear();
    outBufA->clear();
    evLog->clear();

    modified = true;

    emit buffersCleaned();

    emit updateAllViews( Primary_Read );
    emit updateAllViews( Primary_Write );
    emit updateAllViews( Secondary_Read );

    QApplication::restoreOverrideCursor();
    slotChangedViewList();
}

bool SLSession::canCloseFrame()
{
    return true;
}


SLSessionDialog* const SLSession::getDialog() 
{
    return dlg;
}


void SLSession::slotStartAcquisition()
{
    struct termios t;
    
    ::cfmakeraw( &t );
    dlg->getTermiosInputFlags( t );
    dlg->getTermiosControlFlags( t );

    // opening primary port
    QString deviceName =  dlg->getLineADevice();

    if (lineA != 0)
    {
        delete lineA;
        lineA = 0;
    }

    lineA = new SLSerialPort( deviceName, t );
    CHECK_PTR ( lineA );

    //connecting notification signals
    connect( lineA, SIGNAL(rxReady(void)), this, SLOT(slotInputDataReadyA(void)) );
    connect( lineA, SIGNAL(statusChanged(void)), this, SLOT(slotStatusChangedA(void)) );
    
    if ( lineA->open( IO_ReadWrite ) != true )
    {
        KMessageBox::sorry( kapp->mainWidget(), i18n( "failed opening primary serial port" ) + deviceName );
        slotStopAcquisition();
        return;
    } 
    
    ASSERT(lineA->mode() == IO_ReadWrite);

    if (dlg->getLineType() != SLSessionDialog::typeSerialP2P)
    {
        //Also the line B must be considered. The line B
        //does not transmit - it receives only and the
        //parameters are the same of line A, since in this
        //case we are snooping
        //The device is still opened in read/write mode

        deviceName =  dlg->getLineBDevice();

        if ( lineB != 0 )
        {
            delete lineB;
            lineB = 0;
        }
        lineB = new SLSerialPort( deviceName, t );
        CHECK_PTR ( lineB );
        
        connect( lineB, SIGNAL(rxReady(void)), this, SLOT(slotInputDataReadyB(void)) );
        connect( lineB, SIGNAL(statusChanged(void)), this, SLOT(slotStatusChangedB(void)) );

        if ( lineB->open( IO_ReadWrite ) != true )
        {
            KMessageBox::sorry( kapp->mainWidget(), i18n( "failed opening secondary serial port" ) + deviceName );
            slotStopAcquisition();
            return;
        } 
        
        ASSERT(lineB->mode() == IO_ReadOnly);
    }

    run = true;
}


void SLSession::slotStopAcquisition()
{
    QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
    
    if (lineA != 0)
    {
        lineA->close();
    }

    if ( lineB != 0 )
    {
        lineB->close();
    }

    run = false;
    
    emit updateAllViews( Primary_Write );
    QApplication::restoreOverrideCursor();
}



void SLSession::slotInputDataReadyA()
{
    int rx_count = lineA->size();
    long offset = inBufA->documentSize();
    
    QByteArray chunk( rx_count );
    int retval = lineA->readBlock( chunk.data(), rx_count );
    ASSERT(retval > 0);
    
    appendToBuf( inBufA, &chunk );
    
    uint status = lineA->getModemStatus();
    evLog->addEvent( EV_RX1, offset, rx_count, status );
}


void SLSession::slotStatusChangedA()
{
    long offset;
    
    uint status = lineA->getModemStatus();
    
    //checking for trasmission line status change
    if (lineA->txd() || lineA->rts() || lineA->dtr())
    {
        offset = outBufA->documentSize();
        evLog->addEvent( EV_STAT1OUT, offset, 0, status );
    }
    
    if (lineA->rxd() || lineA->cts() || lineA->dsr() || lineA->dcd()|| lineA->ring())
    {
        offset = inBufA->documentSize();
        evLog->addEvent( EV_STAT1IN, offset, 0, status );
    }
    emit updateLineAStatusLeds( status );
  
}


void SLSession::slotStatusChangedB()
{
    long offset = inBufB->documentSize();
    
    uint status = lineB->getModemStatus();
    emit updateLineBStatusLeds( status );
    evLog->addEvent( EV_STAT2, offset, 0, status );
}




void SLSession::slotInputDataReadyB()
{
    int rx_count = lineB->size();
    long offset = inBufB->documentSize();
    
    QByteArray chunk( rx_count );
    int retval = lineB->readBlock( chunk.data(), rx_count );
    ASSERT(retval > 0);
    
    appendToBuf( inBufB, &chunk );
    
    uint status = lineA->getModemStatus();
    evLog->addEvent( EV_RX2, offset, rx_count, status );
}



bool SLSession::sendFile(const KURL &url)
{
   
    
    //Needed to implement network transparency
  
    QString tmpfile;
    KIO::NetAccess::download( url, tmpfile );
    QFile fileToSend( tmpfile );
    qDebug("%s:%u - sending file %s", __FILE__, __LINE__, tmpfile.latin1() );
    
    fileToSend.open( IO_ReadOnly );

    int byte = -1;
    long size = fileToSend.size();
    int onePerCent = size / 100;
    if (onePerCent == 0)
    {
        onePerCent = 1; //avoids SIGFPE
    }
    
    int perCent = 0;
    
    long i = 0;
    int retval = 0; 

    if (size > 0)
    {
    
        //  emit setProgressBarRange(size);
        QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
       
        sendingFile = true;
        
        long offset = outBufA->documentSize();
        uint status;
        
        //file transfer byte by byte
        byte = fileToSend.getch();
        while (byte != -1 && retval >= 0 && i < size && run == true)
        {
            retval = lineA->putch(byte);
                
            if (retval >= 0)
            {
                
                appendToBuf( outBufA, byte );
                    
                status = lineA->getModemStatus();
                
                
                byte = fileToSend.getch();

                perCent = i / onePerCent;
                if ( i % onePerCent == 0 && perCent > 0 )
                {
                    //updates the UI every 1% sent
                    evLog->addEvent( EV_TX, offset+i, onePerCent, status );
                    emit updateProgressBar( perCent, this );
                    emit updateAllViews( Primary_Write );
                    kapp->processEvents();
                }
                
                i++;
            }
        }
        
        KIO::NetAccess::removeTempFile( tmpfile );
        sendingFile = false;
        emit updateProgressBar(-1, this);
        emit updateAllViews( Primary_Write );
        
        QApplication::restoreOverrideCursor();
        
        if ( i != size )
        {
            if (run == false)
            {
                //acquisition stopped, transfer interrupted
                KMessageBox::sorry( kapp->mainWidget(), i18n( "acquisition stopped, file transfer interrupted" ) );
            }
            else
            {
                // sending failed
                KMessageBox::error(kapp->mainWidget(), i18n("Failure sending the file!"), i18n("I/O Error !"));
                return false;
            }
            return false;
        }
    }   
    return true;
}

void SLSession::appendToBuf( SLHexBuffer* const buf, const char c )
{
    SCursorConfig cc;
    cc.emulateControlButton( true );
    buf->cursorEnd( cc.controlButton() );

    QByteArray data(1);
    data[0] = c;
    int errCode = buf->inputAtCursor( data, 0 );
    CHECK_PTR( errCode == Err_Success  );
    modified = true;
}

void SLSession::appendToBuf( SLHexBuffer* const buf, const QByteArray* data )
{
    SCursorConfig cc;
    cc.emulateControlButton( true );
    buf->cursorEnd( cc.controlButton() );

    int errCode = buf->inputAtCursor( *data, 0 );
    if( errCode == Err_Success  )
    {
        if ( buf == inBufA )
        {
            emit updateAllViews( Primary_Read );
        }
        else if ( buf == outBufA )
        {
            emit updateAllViews( Primary_Write );
        }
        else if ( buf == inBufB )
        {
            emit updateAllViews( Secondary_Read );
        }

        modified = true;

    }

}

void SLSession::slotUpdateOutputBuffer( const QByteArray& ba )
{

    uint status;
    
    // acquisition must run and no file sending running
    if (run == true && sendingFile == false)
    {
        //sends the data

        unsigned long len = ba.size();
        long offset = outBufA->documentSize();
        Q_LONG retval = lineA->writeBlock( ba.data(), len );   
        if (retval == static_cast <Q_LONG> (len))
        {
            //emit updateAllViews( Primary_Write, echo );
            appendToBuf( outBufA, &ba );
            modified = true;
        }

  
        status = lineA->getModemStatus();
        evLog->addEvent( EV_TX, offset, len, status );

        CHECK_PTR (retval >= 0);
    }
}

void SLSession::slotUpdateOutputBuffer( char c )
{
    uint status;

    // acquisition must run and no file sending running
    if (run == true && sendingFile == false)
    {
        //number of bytes to send
        ulong count = 1;

        //puts the char in a temp bytearray
        QByteArray buf(count);
        buf[0] = c;
        
        //sends the data
        unsigned long len = buf.size();
        long offset = outBufA->documentSize();
        Q_LONG retval = lineA->writeBlock( buf.data(), len );   
        if (retval == static_cast <Q_LONG> (len))
        {
            //emit updateAllViews( Primary_Write );
            appendToBuf( outBufA, &buf );
            modified = true;
        }
        status = lineA->getModemStatus();
        evLog->addEvent( EV_TX, offset, len, status );

        CHECK_PTR (retval >= 0);
    }
}


void SLSession::slotUpdateOutputBuffer(SLHexBuffer* privateBuf)
{
    /* 
     * Every time something is entered in the output views, that data is written
     * in the relative outBufA in SLSession. After that, an updateBuffer() signal is 
     * raised directly by the widget, that is handled here. 
     * The outBufA buffer and the views private buffers
     * shares the same data area but there are some auxiliary fields that must
     * be keep aligned, for example the number of lines. So the entering buffer
     * sends dataEntered() to align the outBufA and after that an updateAllViews()
     * is sent to align also the remaining output windows
     */ 
  
    uint status;
    
    // acquisition must run and no file sending running
    if (run == true && sendingFile == false)
    {
        //number of bytes to send    
        long count = privateBuf->documentSize() - outBufA->documentSize();
        
        if ( count > 0 )
        {  
            QByteArray buf(count);
            
            uint i,f;
            for( f = 0, i = outBufA->documentSize(); i < privateBuf->documentSize(); i++, f++)
            {
                buf[f] = (*privateBuf)[i];
            }

            CHECK_PTR(buf.size() > 0);

            //sends the data
            unsigned long len = buf.size();
            long offset = outBufA->documentSize();
            Q_LONG retval = lineA->writeBlock( buf.data(), len );   
            if (retval == static_cast <Q_LONG> (len))
            {
                //emit updateAllViews( Primary_Write );
                appendToBuf( outBufA, &buf );
                //align all the views that shows sending data
                outBufA->alignBuffers( privateBuf );
                modified = true;
            }
            status = lineA->getModemStatus();
            evLog->addEvent( EV_TX, offset, len, status );

            CHECK_PTR (retval >= 0);
        }
    }
    
}

bool SLSession::isSnooper() const
{
    CHECK_PTR(dlg != 0);
    return (dlg->getLineType() == SLSessionDialog::typeSerialTT);
}


void SLSession::forceSynchronizeViews( const SLView* master, BufferLine bl, uint offset )
{
    if (viewSyncEnable == false)
    {
        //skips the synchronization while
        //loading/saving the buffers from/to disk
        return;
    }
    
    if (sendingFile == true)
    {
        //skips the synchronization while
        //loading the buffers from disk
        return;
    }

   
    qDebug("%s:%u - Synchronizing signal from %s:%p, offset=%u", __FILE__, __LINE__, master->className(), (void*)master, offset);
    emit synchronizeViews( master, bl, offset );
}
   
 
void SLSession::forceSelectChunk(BufferLine line, uint offset, uint length)
{
    if (viewSyncEnable == false)
    {
        //skips the synchronization while
        //loading/saving the buffers from/to disk
        return;
    }

    if (sendingFile == true)
    {
        //skips the synchronization while
        //loading the buffers from disk
        return;
    }

    qDebug("%s:%u - Selecting chunk on line %u, offset=%u, length=%u", __FILE__, __LINE__,
        line, offset, length);

    emit selectChunk(line, offset, length);    
}
  
