/*
 *                        slsession.h
 *                           -------------------
 *  begin                : mer dic  4 22:54:15 CET 2002
 *  copyright            : (C) 2002 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB.h> skips=4
 *  <TAB.h> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * $Id: slsession.h,v 1.9 2005/01/21 08:40:58 gbertani Exp $ 
 */

#ifndef SLSESSION_H
#define SLSESSION_H

using namespace std;


// include files for QT
#include <qobject.h>
#include <qstring.h>
#include <qlist.h>
#include <qworkspace.h>
#include <qfile.h>
#include <qdir.h>

// include files for KDE
#include <kurl.h>
#include <kapp.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kio/netaccess.h>



// forward declaration of the SerLooK classes

class SLHexBuffer;
class SerLooKApp;
class SLView;
class SLSerialPort;
class SLSessionDialog;
class SLDumpView;
class SLEventRecord;

/** defined also in sldumpdialog.h as SourceBuffer */
enum BufferLine
{
    Primary_Read,
    Secondary_Read,
    Primary_Write
};


/** 
  * 
  *    SLSession provides a document object for a document-view model.
  *
  * The SLSession class provides a document object that can be used in conjunction with the classes
  * SerLooKApp and SLView to create a document-view model for MDI (Multiple Document Interface)
  * KDE 2 applications based on KApplication and KTMainWindow as main classes and QWorkspace as MDI manager widget.
  * Thereby, the document object is created by the SerLooKApp instance (and kept in a document list) and contains
  * the document structure with the according methods for manipulating the document
  * data by SLView objects. Also, SLSession contains the methods for serialization of the document data
  * from and to files.
  *
  * The document object contains session parameters. Its file structure is similar to KDE configuration files,
  * with the exception that they can be placed everywhere in the filesystem. Keep careful to not exchange
  * session parameters with the GUI options that are instead saved in the appropriate serlookrc file.
  * 
  * 
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team. Customizations for SerLooK (c) Giuseppe Massimo Bertani     
  * @version 
  * 
  */
    
class SLSession : public QObject
{
    Q_OBJECT 

public:
    
    /** Loads the last saved session (default) */
    SLSession(QWidget* const clientArea, const KURL &url);
    
    ~SLSession();

    /** returns the first view instance */
    SLView* firstView() const;
    
    /** returns the active view */
    SLView* getActiveView() const;
   
    /** returns true if there is only one view opened */
    bool isLastView() const;
    
    /** returns true if there are no more opened views */
    bool noViews() const;
    
    /** This method gets called when the user is about to close a frame window. It checks, if more than one view
        * is connected to the document (then the frame can be closed), if ws have only one view and the document is
        * modified, the user gets asked if he wants to save the document.
        */
    bool canCloseFrame();
    
    /** sets the modified flag for the document after a modifying action on the view connected to the document.*/
    void setModified(bool m = true)
    {
        modified = m;
    };
    
    /** returns if the document is modified or not. Use this to determine if your document needs saving by the user on closing.*/
    bool isModified(){ return modified; };

    
    /** loads the document by filename and format and emits the updateViews() signal */
    bool loadDocument(const KURL &url);
    
    /** saves the document under filename and format.*/    
    bool saveDocument(const KURL &url);
    
    /** saves the document under filename and format.*/    
    bool sendFile(const KURL &url);
    
    /** returns the KURL of the document */
    const KURL& URL() const;

    /** sets the URL of the document */
    void setURL(const KURL& url);

    /** returns a reference to the session settings */
    SLSessionDialog* const getDialog();
   
    /** 
     * ws is the MDI frame widget that handles MDI child widgets
     * each view inserts itself in the workspace during creation
     */
    QWorkspace* workspace() const
    {
        return ws;
    }
    

    /**
     * returns the buffer associated to a line.
     */ 
    SLHexBuffer* const getBuffer(BufferLine bl);
   
    /**
     * returns the event recorder.
     */ 
    SLEventRecord* const getEventRecorder()
    {
        return evLog;
    }
   

    /**
     * returns true if in TT mode
     */
    bool isSnooper() const;
    
    /**
     * forces to emit a synchronizeViews() signal
     * that should synchronize all the active views
     * to show the data starting from offset
     * of the buffer bl
     */
    void forceSynchronizeViews( const SLView* master, BufferLine bl, uint offset );

    /**
     * Event views call this method to propagate
     * a chunk selection signal to all data views
     * 
     */
    void forceSelectChunk(BufferLine line, uint offset, uint length);

    
    /** */
    bool acquisition() const
    {
        return run;
    }
    
public slots:

    /** clean all the buffers */
    void slotCleanBuffers();

    /** gets called if a view is removed or added */
    void slotChangedViewList();

    /** keeps the outBufA aligned with the
     * data entered via keyboard in the views
     */ 
    void slotUpdateOutputBuffer(SLHexBuffer* privateBuf);

    /**  NOT USED YET */
    void slotUpdateOutputBuffer( const char c );

    /** used by terminal views to send entered chars */
    void slotUpdateOutputBuffer( const QByteArray& ba );
    
    /** */
    void slotStartAcquisition();

    /** */
    void slotStopAcquisition();

    
private slots:

    /** */
    void slotInputDataReadyA();

    /** */
    void slotInputDataReadyB();

    /** */
    void slotStatusChangedA();

    /** */
    void slotStatusChangedB();
    
signals:

    /** */
    void updateAllViews( BufferLine bl );

    /** */
    void setProgressBarRange( int totalSteps );
    
    /** */
    void updateProgressBar( int progress, void* requester );
    
    /** */
    void updateLineAStatusLeds( uint status );
    
    /** */
    void updateLineBStatusLeds( uint status );
    
    /** */
    void updateStatusBar( const QString& status );

    /** */
    void createDumpView();

    /** */
    void createTerminalView();

    /** */
    void createEventView();
    
    /** */
    void synchronizeViews( const SLView* master, BufferLine bl, uint offset );

    /** */
    void selectChunk(BufferLine line, uint offset, uint length);      

    /** */
    void buffersCleaned();

private:

    /** appends data to the hexbuffer then updates 
     * all the involved views */
    void appendToBuf( SLHexBuffer* const buf, const QByteArray* data );
    
    /** appends a single byte to the hexbuffer
     * without update the views - used for
     * file sending (the views are updated
     * at the end of sending) 
     */
    void appendToBuf( SLHexBuffer* const buf, const char c );
    
    /** input data log buffer line A */
    SLHexBuffer* inBufA;

    /** output data log buffer line A*/
    SLHexBuffer* outBufA;
    
    /** input data log buffer line B */
    SLHexBuffer* inBufB;

    /** event logging file */
    SLEventRecord* evLog;
    
    /** the modified flag of the current document */
    bool modified;

    /** loadDocument() is running */
    bool viewSyncEnable;
    
    /** performing file sending, the output buffer will
     * be updated AFTER sending and only with the bytes
     * effectively sent
     */
    bool sendingFile;
    
    /** */
    KURL doc_url;
    
    /** the session dialog is still present in memory because it
     * holds all the settings
     */ 
    SLSessionDialog* dlg;
    
    /** views workspace */
    QWorkspace *ws;
    
    /** splitter between the the views workspace and
     * the optional widgets like the input box */
    QSplitter* splitter;
    
    /** .ser file format version */
    static const uint fileFormatVersion;
    
    /** primary serial line */
    SLSerialPort*       lineA;
   
    /** secondary serial line */
    SLSerialPort*       lineB;

    /** acquisition running flag */
    bool run;
   
};



#endif // SLSESSION_H
