/*
 *                        slsessiondialog.cpp
 *                           -------------------
 *  begin                : Sun Dec 29 2002
 *  copyright            : (C) 2002 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *  Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * 
 */

static char* cvs_id = "$Id: slsessiondialog.cpp,v 1.7 2004/12/10 16:19:13 gbertani Exp $";

#include "slsessiondialog.h"

SLSessionDialog::SLSessionDialog( QWidget *parent, char *name, bool modal ) 
  : KDialogBase( Tabbed, i18n("Session settings"), Help|Default|Apply|Ok|Cancel, Ok, parent, name, modal, true )
{
    setHelp( "serlook/index.html", QString::null ); // When Help is pressed

    setupCommLinePage();
    setupCommParamsPage();
}

SLSessionDialog::~SLSessionDialog()
{
}

void SLSessionDialog::setupCommLinePage()
{
    QFrame* page = addPage( i18n( "Line" ), i18n( "Communication line selection" ));
    
    QGridLayout* myLayout = new QGridLayout( page ); 
//row 0

    mCommLine.typeLabel = new QLabel( page );
    mCommLine.typeLabel->setText( i18n( "Connection type" ) );
    myLayout->addWidget( mCommLine.typeLabel, 0, 0 );
    
    mCommLine.typeSelector = new QComboBox( false, page );
    mCommLine.typeSelector->insertItem( i18n( "Serial - one line - point to point mode" ) ,    typeSerialP2P );
    mCommLine.typeSelector->insertItem( i18n( "Serial - two lines - snooper mode" ),   typeSerialTT  );
    //mCommLine.typeSelector->insertItem( i18n( "Serial - two lines point to point" ),    typeSerialInMiddle );

    QToolTip::add( mCommLine.typeSelector, i18n( "Warning: changing this setting, the all the data buffers will be cleared" ));
    myLayout->addWidget( mCommLine.typeSelector, 0, 1 );
    
    connect( mCommLine.typeSelector, SIGNAL(activated(int)), this, SLOT(slotEnableBDevice()) );

//row 1
    mCommLine.deviceALabel = new QLabel( page );
    mCommLine.deviceALabel->setText( i18n( "Communication device A" ) );
    myLayout->addWidget( mCommLine.deviceALabel, 1, 0 );
    mCommLine.deviceAEntry = new QLineEdit( page );
    myLayout->addWidget( mCommLine.deviceAEntry, 1, 1 );

//row 2
    mCommLine.deviceBLabel = new QLabel( page );
    mCommLine.deviceBLabel->setText( i18n( "Communication device B (read only)" ) );
    myLayout->addWidget( mCommLine.deviceBLabel, 2, 0 );
    mCommLine.deviceBEntry = new QLineEdit( page );
    myLayout->addWidget( mCommLine.deviceBEntry, 2, 1 );

    mCommLine.deviceBLabel->setEnabled(false);
    mCommLine.deviceBEntry->setEnabled(false);

    int idx = pageIndex( page );
    qDebug( "%s:%u - session dialog comm line page %p index: %d", __FILE__, __LINE__, (void*)page, idx );

}


void SLSessionDialog::setupCommParamsPage()
{
    QFrame* page = addPage( i18n( "Params" ), i18n( "Communication parameters" ));
    
    QGridLayout* myLayout = new QGridLayout( page ); 

//row 0
    mCommParams.speedLabel = new QLabel( page );
    mCommParams.speedLabel->setText( i18n( "Baudrate" ) );
    myLayout->addWidget( mCommParams.speedLabel, 0, 0 );

    mCommParams.speedSelector = new QComboBox( false, page, "mCommParams.speedSelector" );
    mCommParams.speedSelector->insertItem( i18n( "300" ),       baud300 );
    mCommParams.speedSelector->insertItem( i18n( "600" ),       baud600 );
    mCommParams.speedSelector->insertItem( i18n( "1200" ),      baud1200 );
    mCommParams.speedSelector->insertItem( i18n( "1800" ),      baud1800 );
    mCommParams.speedSelector->insertItem( i18n( "2400" ),      baud2400 );
    mCommParams.speedSelector->insertItem( i18n( "4800" ),      baud4800 );
    mCommParams.speedSelector->insertItem( i18n( "9600" ),      baud9600 );
    mCommParams.speedSelector->insertItem( i18n( "19200" ),     baud19200 );
    mCommParams.speedSelector->insertItem( i18n( "38400" ),     baud38400 );
    mCommParams.speedSelector->insertItem( i18n( "57600" ),     baud57600 );
    mCommParams.speedSelector->insertItem( i18n( "115200" ),    baud115200 );
    mCommParams.speedSelector->insertItem( i18n( "230400" ),    baud230400 );
    //mCommParams.speedSelector->insertItem( i18n( "Custom" ),    baudCustom ); NOT PORTABLE 
    myLayout->addWidget( mCommParams.speedSelector, 0, 1 );

    //connect( mCommParams.speedSelector, SIGNAL(activated(int)), this, SLOT(slotEnableCustomSpeed()) );
    
//row 1
    mCommParams.customSpeedLabel = new QLabel( page );
    mCommParams.customSpeedLabel->setEnabled( false );
    mCommParams.customSpeedLabel->setText( i18n( "Custom value" ) );
    myLayout->addWidget( mCommParams.customSpeedLabel, 1, 0 );

    mCommParams.customSpeedEntry = new QSpinBox( page );
    mCommParams.customSpeedEntry->setEnabled( false );
    mCommParams.customSpeedEntry->setMaxValue( 230400L );
    mCommParams.customSpeedEntry->setMinValue( 300 );
    mCommParams.customSpeedEntry->setLineStep( 100 );
    myLayout->addWidget( mCommParams.customSpeedEntry, 1, 1 );
    
//row 2
    mCommParams.sizeLabel = new QLabel( page );
    mCommParams.sizeLabel->setText( i18n( "Character size" ) );
    myLayout->addWidget( mCommParams.sizeLabel, 2, 0 );
    
    mCommParams.sizeSelector = new QComboBox( false, page );
    mCommParams.sizeSelector->insertItem( i18n( "5" ), size5 );
    mCommParams.sizeSelector->insertItem( i18n( "6" ), size6 );
    mCommParams.sizeSelector->insertItem( i18n( "7" ), size7 );
    mCommParams.sizeSelector->insertItem( i18n( "8" ), size8 );
    mCommParams.sizeSelector->setCurrentItem( 3 );
    myLayout->addWidget( mCommParams.sizeSelector, 2, 1 );

//row 3
    mCommParams.stopsLabel = new QLabel( page );
    mCommParams.stopsLabel->setText( i18n( "Stop bits" ) );
    myLayout->addWidget( mCommParams.stopsLabel, 3, 0 );

    mCommParams.stopsSelector = new QComboBox( false, page );
    mCommParams.stopsSelector->insertItem( i18n( "1" ), stop1 );
    mCommParams.stopsSelector->insertItem( i18n( "2" ), stop2 );
    myLayout->addWidget( mCommParams.stopsSelector, 3, 1 );

//row 4
    mCommParams.parityLabel = new QLabel( page );
    mCommParams.parityLabel->setText( i18n( "Parity" ) );
    myLayout->addWidget( mCommParams.parityLabel, 4, 0 );

    mCommParams.paritySelector = new QComboBox( false, page );
    mCommParams.paritySelector->insertItem( i18n( "None" ), parNone );
    mCommParams.paritySelector->insertItem( i18n( "Even" ), parEven );
    mCommParams.paritySelector->insertItem( i18n( "Odd" ),  parOdd );
    myLayout->addWidget( mCommParams.paritySelector, 4, 1 );

//row 5
    mCommParams.flowLabel = new QLabel( page );
    mCommParams.flowLabel->setText( i18n( "Flow control" ) );
    myLayout->addWidget( mCommParams.flowLabel, 5, 0 );

    mCommParams.flowSelector = new QComboBox( false, page );
    mCommParams.flowSelector->insertItem( i18n( "None" ),                   flowNone );
    mCommParams.flowSelector->insertItem( i18n( "Hardware (RTS/CTS)" ),     flowHardware );
    mCommParams.flowSelector->insertItem( i18n( "Software (XON/XOFF)" ),    flowXonXoff );
    mCommParams.flowSelector->insertItem( i18n( "Software (XANY/XOFF)" ),   flowXanyXoff );
    myLayout->addWidget( mCommParams.flowSelector, 5, 1 );

    int idx = pageIndex( page );
    qDebug( "%s:%u - session dialog comm params page %p index: %d", __FILE__, __LINE__, (void*)page, idx );


}


void SLSessionDialog::slotAllDefaults()
{
    mCommLine.typeSelector->setCurrentItem(typeSerialP2P);
    mCommLine.deviceAEntry->setText("/dev/ttyS0");   
    mCommLine.deviceBEntry->setText("/dev/ttyS1");   
    mCommLine.deviceBLabel->setEnabled(false);
    mCommLine.deviceBEntry->setEnabled(false);
    
    mCommParams.speedSelector->setCurrentItem(baud38400);
    mCommParams.sizeSelector->setCurrentItem(size8);
    mCommParams.stopsSelector->setCurrentItem(stop1); 
    mCommParams.paritySelector->setCurrentItem(parNone);
    mCommParams.flowSelector->setCurrentItem(flowNone);
    mCommParams.customSpeedEntry->setValue(0);          //none
    mCommParams.customSpeedLabel->setEnabled(false);
    mCommParams.customSpeedEntry->setEnabled(false);
    
}

void SLSessionDialog::slotDefault()
{
    switch( activePageIndex() )
    {
        case pageCommLine:
            mCommLine.typeSelector->setCurrentItem(typeSerialP2P);
            mCommLine.deviceAEntry->setText("/dev/ttyS0");   
            mCommLine.deviceBEntry->setText("/dev/ttyS1");   
            mCommLine.deviceBLabel->setEnabled(false);
            mCommLine.deviceBEntry->setEnabled(false);
            break;
 
        case pageCommParams:
            mCommParams.speedSelector->setCurrentItem(baud38400);
            mCommParams.sizeSelector->setCurrentItem(size8);
            mCommParams.stopsSelector->setCurrentItem(stop1); 
            mCommParams.paritySelector->setCurrentItem(parNone);
            mCommParams.flowSelector->setCurrentItem(flowNone);
            mCommParams.customSpeedEntry->setValue(0);          //none
            mCommParams.customSpeedLabel->setEnabled(false);
            mCommParams.customSpeedEntry->setEnabled(false);
            break;

        default:
            break;
    }
    
}

void SLSessionDialog::slotOk()
{
    slotApply();
    accept();
}

void SLSessionDialog::slotApply()
{
    int idx = activePageIndex();
    qDebug( "%s:%u - session dialog selected page index: %d", __FILE__, __LINE__, idx );

    switch( idx )
    {
        case pageCommLine:
            emit commLineChoice();
            break;
 
        case pageCommParams:
            emit commParamsChoice();
            break;
            
    }
    
}


void SLSessionDialog::slotEnableCustomSpeed()
{
    mCommParams.customSpeedLabel->setEnabled(false);
    mCommParams.customSpeedEntry->setEnabled(false);

    if (mCommParams.speedSelector->currentText() == QString("Custom"))
    {
        mCommParams.customSpeedLabel->setEnabled(true);
        mCommParams.customSpeedEntry->setEnabled(true);
    }
}


void SLSessionDialog::slotEnableBDevice()
{
    mCommLine.deviceBLabel->setEnabled(false);
    mCommLine.deviceBEntry->setEnabled(false);

    if (mCommLine.typeSelector->currentText().contains("two line", false) == true)
    {
        mCommLine.deviceBLabel->setEnabled(true);
        mCommLine.deviceBEntry->setEnabled(true);
    }
}


uint SLSessionDialog::getBaudRate() const
{
    
    typedef QMap<BaudrateItems,uint> BaudRateMap;
    
    BaudRateMap bmap;
    
    bmap.insert( baud300, 300 );
    bmap.insert( baud600, 600 );
    bmap.insert( baud1200, 1200 );
    bmap.insert( baud2400, 2400 );
    bmap.insert( baud4800, 4800 );
    bmap.insert( baud9600, 9600 );
    bmap.insert( baud19200, 19200 );
    bmap.insert( baud38400, 38400 );
    bmap.insert( baud57600, 57600 );
    bmap.insert( baud115200, 115200 );
    bmap.insert( baud230400, 230400 );
    
    BaudrateItems b = getSpeed();

    return bmap[b];
}

    
void SLSessionDialog::getTermiosControlFlags( struct termios& t )
{
    
    typedef QMap<BaudrateItems,uint> BaudRateMap;
    
    BaudRateMap bmap;
    
    bmap.insert( baud300, B300 );
    bmap.insert( baud600, B600 );
    bmap.insert( baud1200, B1200 );
    bmap.insert( baud2400, B2400 );
    bmap.insert( baud4800, B4800 );
    bmap.insert( baud9600, B9600 );
    bmap.insert( baud19200, B19200 );
    bmap.insert( baud38400, B38400 );
    bmap.insert( baud57600, B57600 );
    bmap.insert( baud115200, B115200 );
    bmap.insert( baud230400, B230400 );
    
    BaudrateItems b = getSpeed();

    t.c_cflag &= ~( CBAUD | CSIZE | PARENB | CSTOPB | PARODD | CRTSCTS );
    
    //t.c_cflag |= bmap[b];
    cfsetispeed( &t, bmap[b] );
    cfsetospeed( &t, bmap[b] );

    typedef QMap<SizeItems,uint> SizeMap;
    
    SizeMap smap;
    
    smap.insert( size5, CS5 );
    smap.insert( size6, CS6 );
    smap.insert( size7, CS7 );
    smap.insert( size8, CS8 );
    
    SizeItems s = getSize();
    
    t.c_cflag |= smap[s];
    
    typedef QMap<StopsItems,uint> StopsMap;
    
    StopsMap zmap;
    
    zmap.insert( stop1, 0 );
    zmap.insert( stop2, CSTOPB );
    
    StopsItems z = getStops();
    t.c_cflag |= zmap[z];

    typedef QMap<ParityItems,uint> ParityMap;
    
    ParityMap pmap;
    
    pmap.insert( parNone, 0 );
    pmap.insert( parEven, PARENB );
    pmap.insert( parOdd, PARENB|PARODD );

    ParityItems p = getParity();
    t.c_cflag |= pmap[p];
    
    typedef QMap<FlowControlItems,uint> FlowMap;
    
    FlowMap fmap;
    
    fmap.insert( flowNone, 0 );
    fmap.insert( flowHardware, CRTSCTS );
    fmap.insert( flowXonXoff, 0 );
    fmap.insert( flowXanyXoff, 0 );

    FlowControlItems f = getFlow();
    
    t.c_cflag |= fmap[f];
    t.c_cflag |= (CLOCAL | CREAD);
}


void SLSessionDialog::getTermiosInputFlags( struct termios& t )
{
    typedef QMap<FlowControlItems,uint> FlowMap;
    
    FlowMap fmap;
    
    fmap.insert( flowNone, 0 );
    fmap.insert( flowHardware, 0 );
    fmap.insert( flowXonXoff, IXON|IXOFF );
    fmap.insert( flowXanyXoff, IXANY|IXOFF );
   
    FlowControlItems f = getFlow();

    t.c_iflag &= ~(INPCK | IGNPAR | IXOFF | IXON | IXANY);
    t.c_iflag |= fmap[f];

    typedef QMap<ParityItems,uint> ParityMap;
    
    ParityMap pmap;
    
    pmap.insert( parNone, IGNPAR );
    pmap.insert( parEven, INPCK );
    pmap.insert( parOdd, INPCK );
    
    ParityItems p = getParity();
    t.c_iflag |= pmap[p];
    
}

SLSessionDialog::TypeItems SLSessionDialog::getLineType() const
{
    return (TypeItems) mCommLine.typeSelector->currentItem();
}

QString SLSessionDialog::getLineADevice() const
{        
    return mCommLine.deviceAEntry->text();
}

QString SLSessionDialog::getLineBDevice() const
{
    return mCommLine.deviceBEntry->text();
}

SLSessionDialog::BaudrateItems SLSessionDialog::getSpeed() const
{
    return (BaudrateItems) mCommParams.speedSelector->currentItem();
}

uint SLSessionDialog::getCustomSpeed() const
{
    return  mCommParams.customSpeedEntry->value();
}

SLSessionDialog::SizeItems SLSessionDialog::getSize() const
{
    return (SizeItems) mCommParams.sizeSelector->currentItem();
}

SLSessionDialog::StopsItems SLSessionDialog::getStops() const
{
    return (StopsItems) mCommParams.stopsSelector->currentItem();
}

SLSessionDialog::ParityItems SLSessionDialog::getParity() const
{
    return (ParityItems) mCommParams.paritySelector->currentItem();
}

SLSessionDialog::FlowControlItems SLSessionDialog::getFlow() const
{
    return (FlowControlItems) mCommParams.flowSelector->currentItem();
}

void SLSessionDialog::setLineType( TypeItems t ) 
{
    mCommLine.typeSelector->setCurrentItem( t );
    if (t != typeSerialP2P)
    {
        slotEnableBDevice();    
    }
}

void SLSessionDialog::setLineADevice( QString p )
{        
    mCommLine.deviceAEntry->setText( p );
}

void SLSessionDialog::setLineBDevice( QString p )
{        
    mCommLine.deviceBEntry->setText( p );
}

void SLSessionDialog::setSpeed ( BaudrateItems b ) 
{
    mCommParams.speedSelector->setCurrentItem( b );
}

void SLSessionDialog::setCustomSpeed( uint c ) 
{
    mCommParams.customSpeedEntry->setValue( c );
}

void SLSessionDialog::setSize( SizeItems s )
{
    mCommParams.sizeSelector->setCurrentItem( s );
}

void SLSessionDialog::setStops( StopsItems s )
{
    mCommParams.stopsSelector->setCurrentItem( s );
}

void SLSessionDialog::setParity( ParityItems p )
{
    mCommParams.paritySelector->setCurrentItem( p );
}

void SLSessionDialog::setFlow( FlowControlItems f )
{
    mCommParams.flowSelector->setCurrentItem( f );
}


bool SLSessionDialog::isLineBEnabled() const
{
    return mCommLine.deviceBEntry->isEnabled();
}

QString SLSessionDialog::getSyntheticParams() const
{
    QString synthetic;
    
    synthetic.setNum( getBaudRate() );
    synthetic += ",";

    switch ( getSize() )
    {
        case size5:
            synthetic += "5,";
            break;

        case size6:
            synthetic += "6,";
            break;

        case size7:
            synthetic += "7,";
            break;

        case size8:
            synthetic += "8,";
            break;
    }

    switch( getParity() )
    {
        case parNone:
            synthetic += "N,";
            break;
            
        case parEven:
            synthetic += "E,";
            break;
            
        case parOdd:
            synthetic += "O,";
            break;
    }

    switch( getStops() )
    {
        case stop1:
            synthetic += "1 ";
            break;
    
        case stop2:
            synthetic += "2 ";
            break;
    
    }

    switch( getFlow() )
    {
        case flowNone:
            synthetic += " - ";
            break;
    
        case flowHardware:
            synthetic += "RTS/CTS";
            break;
    
        case flowXonXoff:
            synthetic += "XON/XOFF";
            break;
    
        case flowXanyXoff:
            synthetic += "XANY/XOFF";
            break;
    
    }
   
    return synthetic;
}
    
uint SLSessionDialog::getTimeToSend() const
{
    //Calculating the byte time
    //
    //byteSize is the overall byte length:
    //1 start + n data bits + parity (if requested) + n stops. 
    //
    //Some size/parity/stops combinations might be not possible in the reality
    //depending from the UART type. This could cause a discrepancy
    //up to 10% from the calculated value.
    
    uint byteSize = 1;

    switch ( getSize() )
    {
        case size5:
            byteSize += 5;
            break;

        case size6:
            byteSize += 6;
            break;

        case size7:
            byteSize += 7;
            break;

        case size8:
            byteSize += 8;
            break;
    }

    switch( getParity() )
    {
        case parNone:
            break;
            
        case parEven:
            byteSize += 1;
            break;
            
        case parOdd:
            byteSize += 1;
            break;
    }

    switch( getStops() )
    {
        case stop1:
            byteSize += 1;
            break;
    
        case stop2:
            byteSize += 2;
            break;
    
    }

    
    double byteTime = (getBaudRate() / byteSize);
    byteTime = 1000000 / byteTime;
   
    return static_cast<uint> (byteTime);

}
