/*
 *                        slsessiondialog.h
 *                           -------------------
 *  begin                : Sun Dec 29 2002
 *  copyright            : (C) 2002 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB.h> skips=4
 *  <TAB.h> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * $Id: slsessiondialog.h,v 1.7 2004/12/10 16:19:13 gbertani Exp $ 
 */

#ifndef SLSESSIONDIALOG_H
#define SLSESSIONDIALOG_H

// linux
#include <termios.h>

// include files for Qt
#include <qcheckbox.h>
#include <qtooltip.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qwidget.h>
#include <qlayout.h>

#if QT_VERSION >= 300
#include <qregexp.h>
#endif //QT3


// include files for KDE
#include <klocale.h>
#include <kconfig.h>
#include <kdialogbase.h>

/**SerLooK session configuration dialog box
This dialog allows to set parameters related to
the communication line and the visual representation
of its traffic (views)

  *@author giuseppe massimo bertani
  */


class SLSessionDialog : public KDialogBase  
{
    Q_OBJECT

public: 
    
    enum Page 
    {
        pageCommLine,
        pageCommParams,
        pageDumpViews,
        pageTerminalViews
    };
   
    enum TypeItems
    {
        typeSerialP2P,
        typeSerialTT
        //typeSerialInMiddle,
        //typeParallelP2P,
        //typeParallelTT,
        //typeParallelInMiddle 
    };

    enum BaudrateItems
    {
        baud300,
        baud600,
        baud1200,
        baud1800,
        baud2400,
        baud4800,
        baud9600,
        baud19200,
        baud38400,
        baud57600,
        baud115200,
        baud230400,
        baudCustom
    };

    enum SizeItems
    {
        size5,
        size6,
        size7,
        size8
    };
   
    enum StopsItems
    {
        stop1,
        stop2
    };

    enum ParityItems
    {
        parNone,
        parEven,
        parOdd
    };

    enum FlowControlItems
    {
        flowNone,
        flowHardware,
        flowXonXoff,
        flowXanyXoff
    }; 
  
    SLSessionDialog( QWidget *parent = 0, char *name = 0, bool modal = false );
    
    ~SLSessionDialog();

//--------------- gets -----------------------------
   

    /** returns the id of the item selected in the dialog */
    TypeItems getLineType() const;
        
    /** returns the id of the item selected in the dialog */
    QString getLineADevice() const;

    /** returns true if line B is enabled */
    bool isLineBEnabled() const;
    
    /** returns the id of the item selected in the dialog */
    QString getLineBDevice() const;
    
    /** returns the id of the item selected in the dialog */
    BaudrateItems getSpeed() const;
    
    /** returns the custom baud rate entered in the dialog */
    uint getCustomSpeed() const;
    
    /** returns the baud rate as unsigned int */
    uint getBaudRate() const;
    
    /** returns the id of the item selected in the dialog */
    SizeItems getSize() const;
    
    /** returns the id of the item selected in the dialog */
    StopsItems getStops() const;
    
    /** returns the id of the item selected in the dialog */
    ParityItems getParity() const;
    
    /** returns the id of the item selected in the dialog */
    FlowControlItems getFlow() const;
   
    /** returns a synthetic string with the comm parameters */
    QString getSyntheticParams() const;
   
    /** returns the time [uS] needed to send amount bytes 
     * using the actual baud rate */
    uint getTimeToSend() const;
    
  
    
//----------sets-------------------------------------------------------------   

    
    /** sets the given item as selected in the dialog */
    void setLineType( TypeItems t ); 
    
    /** sets the given item as selected in the dialog */
    void setLineADevice( QString p );
    
    /** sets the given item as selected in the dialog */
    void setLineBDevice( QString p );
   
    /** sets the given item as selected in the dialog */
    void setSpeed ( BaudrateItems b ); 
    
    /** sets the given custom speed as selected in the dialog */
    void setCustomSpeed( uint c ); 
    
    /** sets the given item as selected in the dialog */
    void setSize( SizeItems s );
    
    /** sets the given item as selected in the dialog */
    void setStops( StopsItems s );
    
    /** sets the given item as selected in the dialog */
    void setParity( ParityItems p );
    
    /** sets the given item as selected in the dialog */
    void setFlow( FlowControlItems f );
    
//---------------------------------------------------
    
    
    /**
     * Fills up the c_iflag field of the given
     * struct termios according to the current
     * serial ports settings (that are the 
     * same for both lines A and B)
     */ 
     void getTermiosInputFlags( struct termios& t );
    
    /**
     * Fills up the c_cflag field of the given
     * struct termios according to the current
     * serial ports settings (that are the 
     * same for both lines A and B)
     */ 
     void getTermiosControlFlags( struct termios& t );
    

public slots:
    virtual void slotAllDefaults();
    virtual void slotDefault();
    virtual void slotOk();
    virtual void slotApply();

signals:
    void commLineChoice();
    void commParamsChoice();

private:
    
    struct SCommLine
    {
        QComboBox   *typeSelector;
        QLabel      *typeLabel;
        QLineEdit   *deviceAEntry;
        QLabel      *deviceALabel;
        QLineEdit   *deviceBEntry;
        QLabel      *deviceBLabel;
        
    };


    struct SCommParams
    {
        QComboBox   *speedSelector;
        QLabel      *speedLabel;
        QComboBox   *sizeSelector;
        QLabel      *sizeLabel;
        QComboBox   *stopsSelector;
        QLabel      *stopsLabel;
        QComboBox   *paritySelector;
        QLabel      *parityLabel;
        QComboBox   *flowSelector;
        QLabel      *flowLabel;
        QSpinBox    *customSpeedEntry;
        QLabel      *customSpeedLabel;
    };

    SCommLine       mCommLine;
    SCommParams     mCommParams;
    
    void setupCommLinePage();
    void setupCommParamsPage();

private slots:
    void slotEnableCustomSpeed();
    void slotEnableBDevice();
    
};


#endif
