/*
 *                        slterminaldialog.cpp
 *                           -------------------
 *  begin                : gio ago 28 2003
 *  copyright            : (C) 2003 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * 
 */
static char* cvs_id = "$Id";


#include "slterminaldialog.h"

const int SLTerminalDialog::HISTORY_MIN = 0;
const int SLTerminalDialog::HISTORY_MAX = 1000000;
const int SLTerminalDialog::HISTORY_STEP = 100;


SLTerminalDialog::SLTerminalDialog(QWidget *parent, const char *name  ) 
    : SLViewDialogBase(i18n("Terminal view properties"), parent, name, false)
{
    setupSourceBufferPage();
    setupEmuParamsPage();
    
}

SLTerminalDialog::~SLTerminalDialog()
{
}

void SLTerminalDialog::setupSourceBufferPage()
{

    QFrame* page = addPage( i18n( "Source" ), i18n( "Data source selection" ));
    QGridLayout* myLayout = new QGridLayout( page ); 
    myLayout->setSpacing(10);
//row 0
    mSourceBuffer.sourceBufferLabel = new QLabel( page );
    mSourceBuffer.sourceBufferLabel->setText( i18n( "Source of the data displayed in the view" ) );
    myLayout->addWidget( mSourceBuffer.sourceBufferLabel, 0, 0 );


    mSourceBuffer.sourceBufferSelector = new QComboBox( false, page );

//Primary_Read omitted, since Primary_Write is used -
//meaning bidirectional behavior (Input+Output)

    mSourceBuffer.sourceBufferSelector->insertItem( i18n( "Input/Output to primary line" ), Primary_Read );
    mSourceBuffer.sourceBufferSelector->insertItem( i18n( "Input from secondary line" ), Secondary_Read );
    myLayout->addWidget( mSourceBuffer.sourceBufferSelector, 0, 1 );

//row 1
    mSourceBuffer.syncViewLabel = new QLabel( page );
    mSourceBuffer.syncViewLabel->setText( i18n( "This view accepts synchronization signals" ) );
    myLayout->addWidget( mSourceBuffer.syncViewLabel, 1, 0 );

    mSourceBuffer.syncViewBox = new QCheckBox( page );
    myLayout->addWidget( mSourceBuffer.syncViewBox, 1, 1 ); 
    
    int idx = pageIndex( page );
    qDebug( "-terminal dialog source buffer page %p index: %d", (void*)page, idx );

}

void SLTerminalDialog::setupEmuParamsPage()
{

    QFrame* page = addPage( i18n( "Emulation" ), i18n( "Terminal emulation parameters" ));
    QGridLayout* myLayout = new QGridLayout( page ); 
    myLayout->setSpacing(10);
//row 0
    mEmuParams.returnLabel = new QLabel( page );
    mEmuParams.returnLabel->setText( i18n( "Return key sends" ) );
    myLayout->addWidget( mEmuParams.returnLabel, 0, 0 );

    mEmuParams.returnSelector = new QComboBox( false, page );
    mEmuParams.returnSelector->insertItem( i18n( "CR" ),      retCR );
    mEmuParams.returnSelector->insertItem( i18n( "LF" ),      retLF );
    mEmuParams.returnSelector->insertItem( i18n( "CR+LF" ),   retCRLF );
    myLayout->addWidget( mEmuParams.returnSelector, 0, 1 );

//row 1
    mEmuParams.bkspLabel = new QLabel( page );
    mEmuParams.bkspLabel->setText( i18n( "Backspace key sends" ) );
    myLayout->addWidget( mEmuParams.bkspLabel, 1, 0 );

    mEmuParams.bkspSelector = new QComboBox( false, page );
    mEmuParams.bkspSelector->insertItem( i18n( "BS" ),      bkspBS );
    mEmuParams.bkspSelector->insertItem( i18n( "DEL" ),     bkspDEL );
    myLayout->addWidget( mEmuParams.bkspSelector, 1, 1 );

//row 4
    mEmuParams.echoLabel = new QLabel( page );
    mEmuParams.echoLabel->setText( i18n( "Local echo " ) );
    myLayout->addWidget( mEmuParams.echoLabel, 4, 0 );

    mEmuParams.echoBox = new QCheckBox( page );
    myLayout->addWidget( mEmuParams.echoBox, 4, 1 );
    
//row 5
    mEmuParams.histSizeLabel = new QLabel( page );
    mEmuParams.histSizeLabel->setText( i18n( "History lines " ) );
    myLayout->addWidget( mEmuParams.histSizeLabel, 5, 0 );

    mEmuParams.histSizeEntry = new QSpinBox( HISTORY_MIN, HISTORY_MAX, HISTORY_STEP, page );
    myLayout->addWidget( mEmuParams.histSizeEntry, 5, 1 );

    int idx = pageIndex( page );
    qDebug( "-terminal dialog emulation parameters page %p index: %d", (void*)page, idx );

}


void SLTerminalDialog::slotAllDefaults()
{
    mSourceBuffer.sourceBufferSelector->setCurrentItem(Primary_Write);
    mSourceBuffer.syncViewBox->setDown( true );
    mEmuParams.bkspSelector->setCurrentItem(bkspBS);
    mEmuParams.returnSelector->setCurrentItem(retCRLF);
    mEmuParams.echoBox->setChecked(false);
    mEmuParams.histSizeEntry->setValue(HISTORY_MIN);
}

void SLTerminalDialog::slotDefault()
{
    switch( activePageIndex() )
    {
        case pageSourceBuffer:
            mSourceBuffer.sourceBufferSelector->setCurrentItem(Primary_Write);
            mSourceBuffer.syncViewBox->setDown( true );
            break;
            
        case pageEmuParams:
            mEmuParams.bkspSelector->setCurrentItem(bkspBS);
            mEmuParams.returnSelector->setCurrentItem(retCR);
            mEmuParams.echoBox->setChecked(false);
            mEmuParams.histSizeEntry->setValue(HISTORY_MIN);
            break;
            
        default:
            break;
    }
    
}

void SLTerminalDialog::slotApply()
{
    SLViewDialogBase::slotApply();
    
    switch( activePageIndex() )
    {
        case pageEmuParams:
            emit emuParamsChoice();
            break;

        default:
            break;
            
    }
    
}

SLTerminalDialog::ReturnItems SLTerminalDialog::getReturn() const
{
    return (ReturnItems) mEmuParams.returnSelector->currentItem();
}

SLTerminalDialog::BkspItems SLTerminalDialog::getBackspace() const
{
    return (BkspItems) mEmuParams.bkspSelector->currentItem();
}

bool SLTerminalDialog::getEcho() const
{
    return mEmuParams.echoBox->isChecked();
}

int SLTerminalDialog::getHistSize() const
{
    return mEmuParams.histSizeEntry->value();
}


void SLTerminalDialog::setBackspace( BkspItems b )
{
    mEmuParams.bkspSelector->setCurrentItem( b );
}

void SLTerminalDialog::setReturn( ReturnItems r )
{
    mEmuParams.returnSelector->setCurrentItem( r );
}

void SLTerminalDialog::setEcho( bool e )
{
    mEmuParams.echoBox->setChecked( e );
}

void SLTerminalDialog::setHistSize( int lines )
{
    mEmuParams.histSizeEntry->setValue( lines );
}

void SLTerminalDialog::save(KConfig* data)
{
    SLViewDialogBase::save(data);
    data->writeEntry("History Size", getHistSize());
    data->writeEntry("Local Echo", getEcho()); 
    data->writeEntry("Return Sends", getReturn()); 
    data->writeEntry("Backspace Sends", getBackspace());

}

void SLTerminalDialog::load(KConfig const* data)
{
    setHistSize( data->readNumEntry("History Size") );
    setEcho( data->readBoolEntry("Local Echo") );
    setReturn( (SLTerminalDialog::ReturnItems) data->readUnsignedNumEntry("Return Sends") );
    setBackspace( (SLTerminalDialog::BkspItems) data->readUnsignedNumEntry("Backspace Sends") );
    emit emuParamsChoice();
    SLViewDialogBase::load(data);
}



