/*
 *                        slterminaldialog.h
 *                           -------------------
 *  begin                : gio ago 28 2003
 *  copyright            : (C) 2003 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * $Id: slterminaldialog.h,v 1.2 2005/01/21 08:40:58 gbertani Exp $
 */

#ifndef SLTERMINALDIALOG_H
#define SLTERMINALDIALOG_H

#include "slviewdialogbase.h"


/**Terminal views parameters dialog
  *@author giuseppe massimo bertani
  */

class SLTerminalDialog : public SLViewDialogBase  
{
   Q_OBJECT

public: 
       
    enum Page 
    {
        pageSourceBuffer,
        pageEmuParams
        //colors, fonts, etc. to be added
    };
    
    enum ReturnItems
    {
        retCR,
        retLF,
        retCRLF
    };

    enum BkspItems
    {
        bkspBS,
        bkspDEL
    };
    
	SLTerminalDialog(QWidget *parent=0, const char *name=0);
	~SLTerminalDialog();

    void save(KConfig* data);
    void load(KConfig const* data);
    
//--------------- gets -----------------------------------------------------

    /** returns the id of the item selected in the dialog */
    BkspItems getBackspace() const;
   
    /** returns the id of the item selected in the dialog */
    ReturnItems getReturn() const;
    
    /** returns the local echo state for terminal views */
    bool getEcho() const;

    /** returns the size (lines) of the history */
    int getHistSize() const;
     
//----------sets-------------------------------------------------------------   

     /** sets the given item as selected in the dialog */
    void setBackspace( BkspItems b );

    /** sets the given item as selected in the dialog */
    void setReturn( ReturnItems r );

    /** sets the local echo state */
    void setEcho( bool e );

    /** sets the history size, in lines
     * if lines <= 0 no history available
     */
    void setHistSize( int lines );

//---------------------------------------------------

public slots:
    virtual void slotAllDefaults();
    virtual void slotDefault();
    virtual void slotApply();
    
signals:
    void emuParamsChoice();

private:

    struct SEmuParams
    {
        QComboBox   *returnSelector;
        QLabel      *returnLabel;
        QComboBox   *bkspSelector;
        QLabel      *bkspLabel;
        QSpinBox    *histSizeEntry;
        QLabel      *histSizeLabel;
        QCheckBox   *echoBox;
        QLabel      *echoLabel;
    };

    SEmuParams  mEmuParams; 

    static const int HISTORY_MIN;
    static const int HISTORY_MAX;
    static const int HISTORY_STEP;
    
    void setupEmuParamsPage();
    virtual void setupSourceBufferPage();
};


#endif
