/*
 *                        slterminalview.cpp
 *                           -------------------
 *  begin                : Mon Aug 4 2003
 *  copyright            : (C) 2003 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * 
 */

static char* cvs_id = "$Id: slterminalview.cpp,v 1.9 2005/01/21 12:55:38 gbertani Exp $";


#include "slsession.h"
#include "slview.h"
#include "slsessiondialog.h"
#include "sltermwidget.h"
#include "slhexbuffer.h"
#include "slterminaldialog.h"
#include "slterminalview.h"

SLTerminalView::SLTerminalView(SLSession* const pDoc, BufferLine line) :
    SLView(pDoc, line)  
{
    qDebug( "%s:%u - building slterminalview", __FILE__, __LINE__ );  
    //----- viewer-------------
   
    QHBoxLayout* layout = new QHBoxLayout(this); 
   
    //When the terminal is connected to the first line,
    //line is "Primary_Read" but it means "Primary Read Write" !!!
    //
    bool isMute = (line == Secondary_Read);

    //first creates a default term widget 
    view = new SLTermWidget( this, 0, doc->getBuffer( currentLine ), doc->getBuffer( Primary_Write ), isMute );
    CHECK_PTR( view );

    layout->addWidget( view );
    
    props = new SLTerminalDialog(this);
    CHECK_PTR( props );
    baseProps = props;
    
    //default props
    props->setHistSize(0);
    props->setSourceBuffer( currentLine );
    props->setBackspace( SLTerminalDialog::bkspBS );
    props->setReturn( SLTerminalDialog::retCRLF );
    props->setEcho(false);

    view->setHistSize( props->getHistSize() );
    
    connect(view, SIGNAL( enteredSequence( const QByteArray& ) ), this, SLOT( slotEnteredSequence( const QByteArray& ) ));
    connect(view, SIGNAL( mouseRightClick() ), this, SLOT( slotPropsDialog() ));
    connect(view, SIGNAL( updateProgressBar(int, void*) ), this, SLOT( slotForwardProgressBar(int, void*) ));
    connect(view, SIGNAL( cursorChanged( int )), this, SLOT( slotCursorChanged( int ) ));

    
    connect(doc,  SIGNAL( synchronizeViews( const SLView*, BufferLine, uint )), this, SLOT( slotGoToOffset( const SLView*, BufferLine, uint ) ));
    connect(props, SIGNAL( sourceBufferChoice() ), this, SLOT( slotChangeBuffer() ));
    connect(props, SIGNAL( emuParamsChoice() ), this, SLOT( slotChangeEmuParams() ));
    connect(doc,  SIGNAL( selectChunk( BufferLine, uint, uint )), this, SLOT( slotSelectChunk( BufferLine, uint, uint ) ));
    connect(doc,  SIGNAL( buffersCleaned()), view, SLOT( slotRedraw() ));

    ready = false;
    isSyncing = false;

}

SLTerminalView::~SLTerminalView()
{
    qDebug( "%s:%u - destroying SLTerminalView", __FILE__, __LINE__ );
    delete props;
    delete view;
}

void SLTerminalView::cut()
{
    //view->cut();    
}


void SLTerminalView::copy()
{
    view->copy();
}

void SLTerminalView::paste()
{
    view->paste();
}


void SLTerminalView::save(KConfig* data)
{
    data->writeEntry("Class", QString("SLTerminalView") );
    
    props->save(data);
    SLView::save(data);
}

void SLTerminalView::load(KConfig const* data)
{
    //The "Class" field is already read
    //by SLSession

    SLView::load(data);
    props->load(data);
}


void SLTerminalView::slotUpdateView(BufferLine line)
{
    if (line == currentLine)
    {
        //updates with incoming data
        view->updateView();
    }
    
    if (line == Primary_Write &&
         currentLine == Primary_Read &&
         props->getEcho() == true)
    {
        //echoes local keystrokes
        view->updateView(true);
    }
}

void SLTerminalView::slotChangeEmuParams()
{
    if ( view->setHistSize( props->getHistSize() ) == true)
    {
        //history size changed
        view->slotRedraw();
    }
    else
    {
        view->updateView();
    }
}


void SLTerminalView::print(CHexPrinter *pPrinter)
{
    if (pPrinter->setup( this ))
    {
        view->print( *pPrinter );
    }
}

void SLTerminalView::setCaption( const QString& num, const QString& dev, 
    const QString& enable ) 
{
    QString caption = "(" + num + ") " + dev + " ";
    
    switch( currentLine )
    {
        
        case Primary_Read:
            if (doc->isSnooper() == true)
            {
                caption += i18n("Read-Only");
            }
            else
            {
                caption += i18n("Read-Write") + enable;
            }
            break;

        case Secondary_Read:
            caption += i18n("Read-Only") + enable;
            break;

        default:
            ASSERT(false);
    }

    QWidget::setCaption(caption);
    
}

void SLTerminalView::slotChangeBuffer()
{
    BufferLine line = props->getSourceBuffer();

    ASSERT(line != Primary_Write);

    if (line != currentLine)
    {
        view->clear();
    }
    
    currentLine = line;
    view->setBuffer( doc->getBuffer( currentLine ), doc->getBuffer( Primary_Write ) );

    bool ro = !(currentLine == Primary_Read);
    view->setMute(ro);
    view->updateView();
    doc->slotChangedViewList();
}


void SLTerminalView::slotEnteredSequence( const QByteArray& toSend )
{
    //special keys mapping
    QByteArray subst( toSend );
    
    //Return key
    int found = 0;
    do
    {
        found = subst.find('\x0d', found);
        if (found != -1)
        {
            switch( props->getReturn() )
            {
                case SLTerminalDialog::retCR :
                    found = -1;
                    break;

                case SLTerminalDialog::retLF :
                    subst[ found ] = LF;
                    found++;
                    break;

                case SLTerminalDialog::retCRLF :
                    int i = subst.size();
                    subst.resize( i + 1 );
                    found++;
                    do
                    {
                        subst[i] = subst[i-1];
                        i--;
                    }
                    while (i > found);
                    subst[found] = LF;
                    break;
            }

        }
    }
    while (found != -1);
    
    //Backspace key
    found = 0;
    do
    {
        found = subst.find('\x7f', found);
        if (found != -1)
        {
            switch( props->getBackspace() )
            {
                case SLTerminalDialog::bkspBS :
                    subst[ found ] = '\x08';
                    found++;
                    break;

                case SLTerminalDialog::bkspDEL :
                    found = -1;
                    break;

            }

        }
    }
    while (found != -1);

    doc->slotUpdateOutputBuffer( subst );
}



void SLTerminalView::slotCursorChanged( int value )
{
    if (ready == true && isSyncing == false)
    {
        qDebug( "%s:%u - slotCursorChanged(value=%u)", __FILE__, __LINE__, value);
        doc->forceSynchronizeViews( this, currentLine, value );
    }
}

void SLTerminalView::slotGoToOffset( const SLView* master, BufferLine line, uint offset )
{
    if (ready == false ||
        line == Primary_Write ||
        props->getSyncView() == false ||
        master == this )
    {
        //qDebug( "%s:%u - slotGoToOffset(offset=%u) nothing to do", __FILE__, __LINE__, offset);
        return;
    }
    
    isSyncing = true;
    qDebug( "%s:%u - slotGoToOffset(offset=%u)", __FILE__, __LINE__, offset);
    view->showOffset( offset );
    isSyncing = false;
}

void SLTerminalView::slotSelectChunk(BufferLine line, uint offset, uint length)
{
    if (line != currentLine ||
        props->getSyncView() == false ||
        ready == false )
    {
        //qDebug( "%s:%u - slotSelectChunk() nothing to do", __FILE__, __LINE__ );
        return;
    }
    isSyncing = true;
    qDebug( "%s:%u - slotSelectChunk(line=%u,offset=%u,length=%u)", __FILE__, __LINE__,
        line, offset, length);


    view->selectDataChunk( offset, length );
    isSyncing = false;
}



void SLTerminalView::slotDocumentLoaded()
{
    ready = true;
}

/*
void SLTerminalView::closeEvent(QCloseEvent* e)
{
    qDebug( "%s:%u - closing SLTerminalView", __FILE__, __LINE__ );
    delete this;
}
*/

