/*
 *                        slterminalview.h
 *                           -------------------
 *  begin                : Mon Aug 4 2003
 *  copyright            : (C) 2003 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * $Id: slterminalview.h,v 1.5 2005/01/21 08:40:59 gbertani Exp $
 * 
 */

#ifndef SLTERMINALVIEW_H
#define SLTERMINALVIEW_H

// include files for Qt
#include <qlayout.h>


// includes for KDE
#include <klocale.h>


#include "slsession.h"
#include "slview.h"

class SLView;
class SLSession;
class SLTermWidget;
class SLTerminalDialog;
class SLHexBuffer;

/**Simple terminal emulation view
 * 
 *@author giuseppe massimo bertani
 */

class SLTerminalView : public SLView  
{
   Q_OBJECT

public: 
    /**
     * Creates a terminal emulation view
     */ 
	SLTerminalView(SLSession* const pDoc, BufferLine line );

    /** */
    virtual ~SLTerminalView();
    
    /** */
    void save(KConfig* data);

    /** */
    void load(KConfig const* data);
   
    /** default window size */
    QSize sizeHint() const
    {
        return QSize(400, 200);
    }
    
    /** */
    void cut();

    /** */
    void copy();

    /** */
    void paste();
    
    /** contains the implementation for printing functionality and gets called by SerLooKApp::slotFilePrint() */
    virtual void print(CHexPrinter *pPrinter);

    /**
     * Sets the caption text, adding a personalization
     * after a base text
     */
    virtual void setCaption( const QString& num, const QString& dev, 
            const QString& enable ); 
    
    
    
public slots:
    /** synchronizes the view with the buffer contents.
     * If the line is different from the actual one,
     * nothing happens.
     */
    virtual void slotUpdateView( BufferLine line );

    /**  */
    virtual void slotGoToOffset( const SLView* master, BufferLine line, uint offset );

    /**  */
    virtual void slotCursorChanged( int value );

    /**  */
    virtual void slotSelectChunk(BufferLine line, uint offset, uint length);

protected slots:
    /** 
     * Updates the terminal widget settings after changing
     * them in the relative properties dialog.
     */
    virtual void slotChangeEmuParams();
   
    /**
     * Synchronizes the view with the buffer contents
     * when the source buffer changes in the relative
     * properties dialog.
     * If the line is different from the actual one,
     * the view must completely refresh and display the
     * new data */
    virtual void slotChangeBuffer();
        
    /**
     * forwards input data from the termwidget to the
     * output buffer, specifying if the local echo is
     * desired or not
     */
    virtual void slotEnteredSequence( const QByteArray& toSend );
    
    /**
     * things to do immediately after a new session
     * is created or loaded
     */
    virtual void slotDocumentLoaded();

    
protected:
    //virtual void closeEvent(QCloseEvent* e);

private:
    
    
    /** */
    SLTermWidget* view;    
    
    /** View properties dialog box */
    SLTerminalDialog* props;

};

#endif
