/*
 *                        sltermwidget.cpp
 *                           -------------------
 *  begin                : Mon Aug 4 2003
 *  copyright            : (C) 2003 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * 
 */
static char* cvs_id = "$Id: sltermwidget.cpp,v 1.7 2005/01/21 12:55:38 gbertani Exp $";

#include "slhexbuffer.h"
#include "sltermwidget.h"


SLTermWidget::SLTermWidget(QWidget *parent, const char *name,
            SLHexBuffer *rxBuffer, SLHexBuffer *txBuffer, bool stayMute, uint historyLines ):
 TEWidget(parent, name) 
{
    mapIndex = 0;
    hm = 0;
    hmFullFlag = false;
    cols = 0;
    txOffset = 0;
    rxOffset = 0;                     
    scrBarLen = 0;

    tmr = new QTimer(this);
    CHECK_PTR(tmr);
       
    //links the session buffers 
    setBuffer ( rxBuffer, txBuffer );

    //calls required to correctly initialize the
    //TEWidget
    KeyTrans::loadAll();
    setScrollbarLocation(TEWidget::SCRRIGHT);

    //creates the emulator
    em = new TEmuVt102(this);
    CHECK_PTR(em);

    //set colors
    updateAttributes();

    //creates and initializes hm:
    setHistSize( historyLines );
    appendDisplayLines(rxOffset);

    connect( em, SIGNAL(sndBlock(const char*,int)), this, SLOT(slotSendBytes(const char*,int)) );
    connect( tmr, SIGNAL(timeout(void)), this, SLOT(slotRedraw(void)) );

    setMute( stayMute );

    //widget cleanup
    clear();

    em->setConnect(true);
}

SLTermWidget::~SLTermWidget()
{
    qDebug( "%s:%u - destroying SLTermWidget", __FILE__, __LINE__ );
    em->setConnect(false);
}



void SLTermWidget::updateAttributes()
{
    setVTFont( KGlobalSettings::fixedFont() );

    ColorEntry *ct = getColorTable();
    ct[DEFAULT_BACK_COLOR] =  ColorEntry(KGlobalSettings::baseColor(), 0, 0 );
    ct[DEFAULT_FORE_COLOR] =  ColorEntry(KGlobalSettings::textColor(), 0, 0 );
    setColorTable(ct);
}

bool SLTermWidget::setHistSize( int newSize )
{
    bool histSizeChanged = false;
    
    if (newSize == -1 || newSize != histSize)
    {
        //force a history rebuild
        if (hm != 0)
        {
            delete hm;
            hm = 0;
            em->setHistory( *new HistoryTypeNone() );
        }

        if (newSize == -1)
        {
            newSize = histSize;
        }
        else
        {
            histSizeChanged = true;
        }
    }
                                 

    if (hm == 0)
    {
        hm = new HistoryMap( newSize );
        CHECK_PTR(hm);
        hm->fill(0);
        mapIndex = 0;            
        hmFullFlag = false;
        scrBarLen = 0;
    }
    else
    {
        hm->resize( newSize );
        if ( histSize > newSize )
        {
            //history shrinked, mapIndex
            //must be corrected
            if (mapIndex > (newSize-1))
            {
                mapIndex = 0;
            }

        }
    }
    histSize = newSize;

    qDebug("%s:%u - setHistSize(), histSize=%u, history map size=%d",
         __FILE__, __LINE__, histSize, hm->size());

    em->setHistory( *new HistoryTypeBuffer( histSize ) );
    setScroll( 0, scrBarLen );

    return histSizeChanged;
}

void SLTermWidget::setBuffer( SLHexBuffer *rxBuffer, SLHexBuffer *txBuffer )
{
    //the tx buffer might be null in case of
    //read only views

    //I must define two *Offset variables because I can't
    //use the shared buffer cursor to keep track of the position
    //slhexviewwidget can do this because it must use a local
    //private buffer with its cursor but termwidget have not
    //a private buffer



    txSharedBuffer = txBuffer;
    if (txSharedBuffer != 0)
    {
        QString b( *txSharedBuffer );
        uint bLen = txSharedBuffer->documentSize();
        b.truncate( bLen );
        txOffset = bLen;
    }


    //the rx buffer MUST be defined
    CHECK_PTR (rxBuffer);
    rxSharedBuffer = rxBuffer;

    if (rxSharedBuffer != 0)
    {
        QString b( *rxSharedBuffer );
        uint bLen = rxSharedBuffer->documentSize();
        b.truncate( bLen );
        rxOffset = bLen;
    }



}

void SLTermWidget::setMute( bool status )
{
    //commented after backport to KDE2.2
    //em->setListenToKeyPress(!status);
    mute = status;
}

void SLTermWidget::slotSendBytes( const char* s, int len )
{
    if (mute == true)
    {
        return;
    }

    QByteArray ba;
    ba.duplicate( s, len );
    emit enteredSequence ( ba );

}

void SLTermWidget::slotRedraw()
{
    clear();
    updateView(); //FIXME: localEcho?
}


void SLTermWidget::appendDisplayLines(uint offset)
{
    if(histSize <= 0)
    {
        return;       
    }
    
    SLHexBuffer* sb = rxSharedBuffer;

    qDebug("%s:%u - appendDisplayLines(), hm size %d, mapIndex=%u",
         __FILE__, __LINE__, hm->size(), mapIndex);

    hm->at(mapIndex) = offset;


    qDebug("%s:%u - appendDisplayLines(), appended line with offset %u at map position [%u]",
         __FILE__, __LINE__, offset, mapIndex);

   
    if (static_cast<uint>(mapIndex) == hm->size())
    {
        //hm works as a circular buffer
        mapIndex = 0;
        hmFullFlag = true;
    }

    if ( hmFullFlag == true )
    {
        scrBarLen = histSize-1;
        setScroll( scrBarLen, scrBarLen );
    }
    else
    {
        if (mapIndex < Lines())
        {
            //screen not yet full                   
            //no scroll needed
            scrBarLen = 0;
            setScroll( scrBarLen, scrBarLen );
        }
        else
        {
            //screen full but hm not yet
            scrBarLen = mapIndex - Lines() + 1;
            setScroll( scrBarLen, scrBarLen );

        }
    }
    mapIndex++;

}


void SLTermWidget::updateView( bool localEcho )
{
    SLHexBuffer* sb = (localEcho == true) ? txSharedBuffer : rxSharedBuffer;
    uint& offset = (localEcho == true) ? txOffset : rxOffset;

    uint bufLen = sb->documentSize();


    if (bufLen == 0)
    {
        //nothing to show
        return;
    }

    if (bufLen == offset)
    {
        //nothing new to show
        return;
    }

    //sb->data()[bufLen] = '\0';
    sb->at(bufLen) = '\0';

    //amount of bytes to be appended in the emulator
    int blockSize = bufLen - offset;

    qDebug("%s:%u - offset=%d, blockSize=%d, bufLen=%d, localEcho=%d ", 
            __FILE__, __LINE__, offset, blockSize, bufLen, localEcho);

    //calculates the number of display lines needed to display the entire
    //block and their initial byte offsets - they are appended to the hm array 
    //then the line is sent to the emulation layer - please note that each
    //time the view is resized, this operation must be repeated
    //
    for( uint i = offset; i < bufLen; i++ )
    {
        char c = sb->at(i);
        cols++;
        
        if (c == CR)
        {
            //carriage return, the next char will be displayed at
            //the beginning of the same line
            cols = 0;
        }
        
        if (c == LF)
        {
            //line feed, the current display line terminates and a new
            //hm element must be created but the column counter remains
            //untouched so the next char will be displayed in the next column
            appendDisplayLines(i);
        }
        
        if (cols >= Columns())
        {
            //line wrap: acts like CR+LF
            cols = 0;
            appendDisplayLines(i);
        }
    }

    //sends the new data to the emulator
    QCString tmp(blockSize);
    tmp.duplicate( &sb->data()[offset], blockSize );
    em->onRcvBlock( tmp, blockSize );
    offset += blockSize;

}



void SLTermWidget::clear()
{
    em->reset();
    txOffset = 0;
    rxOffset = 0;
    cols = 0;
    setHistSize(-1);
    appendDisplayLines(rxOffset);

    em->onRcvBlock( "", 0 );

}

void SLTermWidget::copy()
{
//Does nothing, since the copying
//is managed internally by TEWidget
}

/** */
void SLTermWidget::paste()
{
    //does not use emitSelection() pasting mechanism
    //because it doesn't support pasting from SLDumpView
    //the following code snippet comes from CHexViewWidget::paste()

    QMimeSource *data = QApplication::clipboard()->data();
    if( data != 0 )
    {
        QByteArray buf;
        if( CHexDrag::decode( data, buf ) == true )
        {
            //pasting binary data (I.E. from SLDumpView)
            emit enteredSequence( buf );
            return;
        }

        QString text;
        if( QTextDrag::decode( data, text ) == true )
        {
            //pasting text strings
            QByteArray buf;
            CHexClipboard c;

            if( c.decode( buf, text ) == true )
            {
	            emit enteredSequence( buf );
            }
        }
    }

}

void SLTermWidget::resizeEvent ( QResizeEvent* ev )
{
    //window resized: the hm array is rebuilt,
    //the widget cleared and the data reloaded

    tmr->stop();
    TEWidget::resizeEvent(ev);
    tmr->start(500, true);
        
}

void SLTermWidget::mousePressEvent(QMouseEvent* ev)
{

    if ( possibleTripleClick && (ev->button()==LeftButton) )
    {
        mouseTripleClickEvent(ev);
        return;
    }

    if ( !contentsRect().contains(ev->pos()) )
    {
        return;
    }

    QPoint tL  = contentsRect().topLeft();
    int    tLx = tL.x();
    int    tLy = tL.y();

    line_selection_mode = FALSE;
    word_selection_mode = FALSE;

    if ( ev->button() == LeftButton)
    {
        QPoint pos = QPoint((ev->x()-tLx-blX)/font_w,(ev->y()-tLy-bY)/font_h);

        preserve_line_breaks = !( ev->state() & ControlButton );

        if (mouse_marks || (ev->state() & ShiftButton))
        {
            emit clearSelectionSignal();
            iPntSel = pntSel = pos;
            actSel = 1; // left mouse button pressed but nothing selected yet.
            grabMouse(   /*crossCursor*/  ); // handle with care!
        }
        else
        {
            emit mouseSignal( 0, pos.x() + 1, pos.y() + 1 ); // left button
        }
    }

    if ( ev->button() == MidButton )
    {
        //emitSelection(); gmbertani
        paste();
    }

    if ( ev->button() == RightButton ) // Configure
    {
        //emit configureRequest( this, ev->state()&(ShiftButton|ControlButton), ev->x(), ev->y() );
        emit mouseRightClick();
    }
}

uint SLTermWidget::getLine( uint offset )
{
    uint off1,off2;

    int circularLine = -1;

    int filledLines = (hmFullFlag == true)?
        (histSize-1) : (mapIndex-1);

    for ( int i = 0; i < filledLines; i++ )
    {
        off1 = hm->at(i);
        off2 = hm->at(i+1);

        if (i == (mapIndex-1))
        {
            //the line 0: the range to check is between
            //off1 and document size
            off2 = rxSharedBuffer->documentSize();
            if ( off1 <= offset && off2 >= offset)
            {
                circularLine = i;
                break;
            }

        }
        else
        {
            //other lines
            if ( off1 <= offset && off2 > offset)
            {
                circularLine = i;
                break;
            }
        }

    }

    if (circularLine == -1)
    {
        //offset requested not more present
        //in history (too old)
        return histSize;
    }

    //calculates the real line to show

    int line = 0;
    if (hmFullFlag == false)
    {
        line = mapIndex;
    }
    else
    {
        line = circularLine - mapIndex;
        if (line < 0)
        {
            line += histSize;
        }
    }

    return line;
}


void SLTermWidget::scrollChanged(int value)
{
    if(value >= (histSize-1))
    {
        return;    
    }

    TEWidget::scrollChanged(value);

    emit cursorChanged( getOffset( value ) );
}

int SLTermWidget::print( CHexPrinter& printer )
{
    SLHexBuffer* sb = rxSharedBuffer;

    return sb->printAscii( printer );
}

void SLTermWidget::selectDataChunk(uint from, uint size)
{
    //FIXME: too rough
    return;
    

    em->clearSelection();
    int dispLine = showOffset( from );

    if (size == 0 || dispLine == -1)
    {
        //nothing to select
        qDebug("%s:%u - selectDataChunk(from %u, size %u) clearing pseudoselection",
            __FILE__, __LINE__, from, size);
        return;
    }

    qDebug("%s:%u - selectDataChunk(from %u, size %u) activating pseudoselection",
         __FILE__, __LINE__, from, size);

    //calculate the display line end
    int absLineBeg = getLine(from);
    int absLineEnd = getLine(from+size);
    int lineEnd = dispLine + (absLineEnd - absLineBeg);
    int lineBeg = dispLine;
        
    int colBeg = from - getOffset( absLineBeg );
    em->onSelectionBegin(colBeg /* 0 */, lineBeg);

    int colEnd = (from+size) - getOffset( absLineEnd );
    
    em->onSelectionExtend(colEnd /*Columns()-1*/, lineEnd);

    qDebug("%s:%u - selectDataChunk() from lineBeg=%d, lineEnd=%d",
         __FILE__, __LINE__, lineBeg, lineEnd);

}

int SLTermWidget::showOffset( uint offset )
{
    int line = static_cast<int>( getLine(offset) );
    const int halfPage = Lines()/2;
    int displayLine = -1;
        
    if (line < histSize)
    {
        //passing <line> to setScroll()
        //that line will be showed on the
        //widget's bottom line
        
        if (line > (histSize - halfPage))
        {
            //last half page - the most recent
            //history lines displayed when the
            //scrollbar reaches the bottom side
            displayLine = Lines()-1;
            setScroll( line, scrBarLen );
                       
        }
        else if (line < halfPage)
        {
            //first half page - the oldest history
            //lines displayed when the scrollbar
            //reaches the top side
            displayLine = 0;          
            setScroll( 0, scrBarLen );

        }
        else
        {
            //in other cases, the requested row
            //is scrolled at the center of the
            //client area
            displayLine = halfPage;
            setScroll( line + displayLine, scrBarLen );
        }

        emit changedHistoryCursor(scrollbar->value());
        qDebug("%s:%u - showOffset(%u), line=%d",
             __FILE__, __LINE__, offset,  line);
    }

    return displayLine;
}                                                

uint SLTermWidget::getOffset( uint line )
{
    //the line parameter must be converted because hm
    //is managed in a circular fashion. The
    //lowest line displayed, where the cursor
    //is placed, has value=0 while the oldest (topmost)
    //line is numbered (histSize - 1)
    //
    //mapIndex points to the hm element that will
    //contain the offset of the next line to be
    //inserted. This means that if hm is completely
    //filled, [mapIndex] points also the oldest line
    //that will be overwritten by a newer one,
    //while [mapIndex-1] is the real position
    //of line 0.

    int circularLine = line;
    if (hmFullFlag == true)
    {
        circularLine = mapIndex + line;
        if (circularLine >= histSize)
        {
            circularLine -= histSize;
        }
    }

    int offset = hm->at(circularLine);
    return offset;
}



