/*
 *                        sltermwidget.h
 *                           -------------------
 *  begin                : Mon Aug 4 2003
 *  copyright            : (C) 2003 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * $Id: sltermwidget.h,v 1.8 2005/01/21 12:55:38 gbertani Exp $
 */

#ifndef SLTERMWIDGET_H
#define SLTERMWIDGET_H

#include <qwidget.h>
#include <qevent.h>
#include <qclipboard.h>
#include <kglobalsettings.h>

#include "khexedit/hexclipboard.h"
#include "khexedit/hexdrag.h"
#include "khexedit/hexprinter.h"
#include "konsole/TEmuVt102.h"
#include "konsole/TEHistory.h"
#include "konsole/TEScreen.h"


class SLHexBuffer;


typedef QMemArray<uint>HistoryMap;



/**VT100 terminal emulation widget
  *@author giuseppe massimo bertani
  */

class SLTermWidget : public TEWidget 
{
   Q_OBJECT

public: 
	SLTermWidget(QWidget *parent, const char *name, 
            SLHexBuffer *rxBuffer, SLHexBuffer *txBuffer,
            bool stayMute = false, uint historyLines = 0);
            
	~SLTermWidget();

    /** changes the buffer showed. This will cause
     * the complete redrawing of the widget */
    void setBuffer( SLHexBuffer *rxBuffer, SLHexBuffer *txBuffer );

    /** changes the emulator history size */
    bool setHistSize( int newSize );
    
    /** toggle the mute flag */ 
    void setMute( bool status );
    
    /** This method must be called to notify the widget
    about new data in the input buffer to be displayed.
    When the widget is created, all the data contained
    in the buffer are sent to the VT emulator so it can
    build its history. Those data can be recorded completely
    or not depending of the number of history lines \p histSize set.
    */
    void updateView( bool localEcho = false );
    
    /** clears the widget */
    void clear();

    /** No descriptions */
    void paste();

    /** No descriptions */
    void copy();

    /** printing code */
    int print( CHexPrinter& printer );

    /** Scrolls the widget's history to display
    the required buffer offset. If the offset
    is no more included in the widget's history
    it does nothing. It return the number of
    widget line (between 0 and Lines()) where
    the given offset is shown */
    int showOffset( uint offset );
                       
     /** chunk pseudo-selection */
    void selectDataChunk(uint from, uint size);

    
    

protected:
    /** reimplemented from TEWidget
     * to use paste() instead of
     * emitSelection() while pressing
     * middleMouse
     */ 
    virtual void mousePressEvent( QMouseEvent* ev );

    /**
     * catches resize events in order to rebuild
     * the hm array and redisplay all
     */
    virtual void resizeEvent ( QResizeEvent* ev );

    /** Applies global colors and fonts from control panel */
    virtual void updateAttributes();

    /** Appends a new line in the lines list */
    void appendDisplayLines( uint i );

    /** returns the history line nr. displaying the given offset */
    uint getLine( uint offset );

    /** returns the offset of the first byte of the given line */
    uint getOffset( uint line );

public slots:
     /**  */
    virtual void slotRedraw(void);
  
protected slots:

    /**  */
    virtual void slotSendBytes(const char* s, int len);


    /** redefined from TEWidget */
    virtual void scrollChanged(int value);


signals:
    /** */
    void updateProgressBar( int progress, void* requester );

    /** */
    void enteredSequence( const QByteArray& ba );
    
    /** notifies the cursor position for
     synchronization purposes */
    void cursorChanged( int value );

    /** */
    void mouseRightClick();
    
private:

   
    /** Konsole emulation code access point */
    TEmuVt102*  em;
    
    /** receiving hex buffer (shared by SLSession) */
    SLHexBuffer* rxSharedBuffer;
    
    /** sending hex buffer (shared by SLSession) */
    SLHexBuffer* txSharedBuffer;
   

    /** offset of the rxSharedBuffer that contains new data */
    uint rxOffset;

    /** offset of the txSharedBuffer that contains new data 
     * (to show in case of local echo on) */
    uint txOffset;
    
    /** mute mode, no keystrokes accepted */
    bool mute;

    /**
     * This is a dynamic array containing
     * the buffer offsets of each line showed
     * in the widget history. When the history is
     * full, the oldest line is deleted to leave
     * space to a newest one so this array must behave
     * in the same circular fashion. The array has one item for each
     * history line, so when the history must be "rotated" all
     * the data inside it scroll down dropping out the oldest one.
     */
    HistoryMap* hm;

    /** hm completely filled, overwriting
     * started */
    bool hmFullFlag;
    
    /** history size (lines) */
    int histSize;

    /** history map circular index */
    int mapIndex;

    /** columns counter */
    int cols;

    /** widget redraw delay timer */
    QTimer* tmr;

    /** scrollbar length */
    int scrBarLen;

};


#endif
