/*
 *                        slview.cpp
 *                           -------------------
 *  begin                : Sun Feb 9 2003
 *  copyright            : (C) 2003 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * 
 */

static char* cvs_id = "$Id: slview.cpp,v 1.8 2004/12/10 16:19:14 gbertani Exp $";

#include <qpainter.h>
#include <qprinter.h>
#include "slsession.h"
#include "slviewdialogbase.h"
#include "slview.h"

SLView::SLView(SLSession* const pDoc, BufferLine line) : 
 QWidget(pDoc->workspace()) 
{

    qDebug( "%s:%u - building slview", __FILE__, __LINE__ );
    CHECK_PTR (pDoc);
    doc=pDoc;
    currentLine = line;
    workspace = pDoc->workspace();
    baseProps = 0;
    
    ready = false;

    //derivate views must instantiate their props dialog
    //then connect sourceBufferChoice() to slotChangeBuffer() 
    qDebug( "%s:%u - slview ok", __FILE__, __LINE__ );  
}

SLView::~SLView()
{
    qDebug( "%s:%u - destroying SLView", __FILE__, __LINE__ );
}

void SLView::save(KConfig* data)
{
    /*
     * For managing sizes and positions we
     * must consider the QWorkspaceChild instead
     * of the window itself, because this one does
     * know anything about its border and decorations
     */ 
    QWidget* c = workspaceChild();
    CHECK_PTR (c);
    
    data->writeEntry("Window Position", c->pos());
    data->writeEntry("Window Size", c->size());
    
}

void SLView::load(KConfig const* data)
{
    /*
     * For managing sizes and positions we
     * must consider the QWorkspaceChild instead
     * of the window itself, because this one does
     * know anything about its border and decorations
     */ 
    QWidget* c = workspaceChild();
    CHECK_PTR (c);

    //currentLine = static_cast <BufferLine> ( data->readUnsignedNumEntry("Connected Line") );
    //serlook must then call slotUpdateView() to synchronize the current buffer with
    //the current line.
    
    QSize s = data->readSizeEntry("Window Size");
    QPoint p = data->readPointEntry("Window Position");

    if(!s.isEmpty())
    {
        c->resize( s );
    }
    
    c->move( p );
}

SLSession *SLView::getDocument() const
{
    return doc;
}

QWidget* SLView::workspaceChild()
{
    QWidget* p = parentWidget();
    if ( p->isA("QWorkspaceChild") == true )
    {
        return p;
    }
    
    return 0;
}

void SLView::update(SLView* pSender)
{
	if(pSender != this)
		repaint();
}


QSize SLView::size()
{
    QWidget* c = workspaceChild();
    CHECK_PTR (c);
    return c->size();
}


QPoint SLView::pos() 
{
    QWidget* c = workspaceChild();
    CHECK_PTR (c);
    return c->pos();
}

void SLView::resize(const QSize& s)
{
    QWidget* c = workspaceChild();
    CHECK_PTR (c);
    return c->resize(s);

}

void SLView::move(const QPoint& p)
{
    QWidget* c = workspaceChild();
    CHECK_PTR (c);
    return c->move(p);

}

/*
void SLView::closeEvent(QCloseEvent* e)
{
    qDebug( "%s:%u - closing SLView", __FILE__, __LINE__ );
    QWidget::closeEvent(e);
}
*/

void SLView::slotPropsDialog()
{
    if (baseProps == 0)
    {
        KMessageBox::sorry( kapp->mainWidget(), i18n( "properties dialog not defined" ) );
        return;
    }
    baseProps->show();
    doc->setModified( true );
}

void SLView::slotForwardProgressBar( int progress, void* requester )
{
    emit updateProgressBar( progress, requester );
}



