/*
 *                        slview.h
 *                           -------------------
 *  begin                : Sun Feb 9 2003
 *  copyright            : (C) 2003 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * $Id: slview.h,v 1.8 2004/12/15 08:31:23 gbertani Exp $ 
 */

#ifndef SLVIEW_H
#define SLVIEW_H

#include <qwidget.h>
#include <qapplication.h>
#include <qworkspace.h>

// include files for KDE
#include <kconfig.h>
#include <kprinter.h>
#include <kglobalsettings.h>

#include "slsession.h"
#include "khexedit/hexprinter.h"

class QPainter;
class QPrinter;
class SLSession;
class SLHexBuffer;
class SLViewDialogBase;


/**Base class for all the view widgets
  *@author giuseppe massimo bertani
  */

class SLView : public QWidget  
{
   Q_OBJECT

public: 


	SLView(SLSession* const pDoc, BufferLine line = Primary_Read);
    
	virtual ~SLView();
    
    /** returns a pointer to the document connected to the view*/
    SLSession *getDocument() const;


    /** */
    BufferLine getLine() const
    {
        return currentLine;
    }

    /** */
//Del di KDevelop: 
    
    /** */
    virtual void save(KConfig* data);

    /** */
    virtual void cut() = 0;

    /** */
    virtual void copy() = 0;

    /** */
    virtual void paste() = 0;

    /** */
    virtual void load(KConfig const* data);
    
    /** */
    SLViewDialogBase* getDialog() const
    {
        return baseProps;
    }
    
    /** */
    QWidget* workspaceChild();
    
    /** gets called to redraw the document contents if it has been modified */
    void update(SLView* pSender);
    
    /** contains the implementation for printing functionality and gets called by SerLooKApp::slotFilePrint() */
    virtual void print(CHexPrinter *pPrinter) = 0;
    
    /**
     * For managing sizes and positions we
     * must consider the QWorkspaceChild instead
     * of the window itself, because this one does
     * know anything about its border and decorations
     */ 
    QSize size();

    /**
     * For managing sizes and positions we
     * must consider the QWorkspaceChild instead
     * of the window itself, because this one does
     * know anything about its border and decorations
     */ 
    QPoint pos();

    /**
     * For managing sizes and positions we
     * must consider the QWorkspaceChild instead
     * of the window itself, because this one does
     * know anything about its border and decorations
     */ 
    void resize (const QSize& s);

    /**
     * For managing sizes and positions we
     * must consider the QWorkspaceChild instead
     * of the window itself, because this one does
     * know anything about its border and decorations
     */ 
    void move (const QPoint& p);		
   
   
    /**
     * Sets the caption text, adding a personalization
     * after a base text
     */
    virtual void setCaption( const QString& num, const QString& dev, 
            const QString& enable ) = 0; 
    
public slots:
    
    /** synchronizes the view with the buffer contents.
     * If the line is different from the actual one,
     * nothing happens.
     */
    virtual void slotUpdateView( BufferLine line ) = 0;

     /**
     * pops up the properties dialog
     */
    virtual void slotPropsDialog();

    /**
     * Forwards progress bar updating from owned
     * widgets up to the main widget by issuing
     * updateProgressBar() signal
     */ 
    void slotForwardProgressBar( int progress, void* requester );
    
signals:

    /** */
    void updateProgressBar( int progress, void* requester );
   
    /** */
    void cursorChanged();

protected slots:
    /** 
     * Synchronizes the view with the buffer contents
     * when the source buffer changes in the relative
     * properties dialog.
     * If the line is different from the actual one,
     * the view must completely refresh and display the
     * new data */
    virtual void slotChangeBuffer() = 0;
    
   
    /**
     * things to do immediately after a new session
     * is created or loaded
     */
    virtual void slotDocumentLoaded() = 0;
    
    
protected:
    //virtual void closeEvent(QCloseEvent* e) = 0;
 
    
    /** Points the document connected to the view, specified in the constructor */
    SLSession *doc;
    
    /** */ 
    QWorkspace* workspace;

    /** View properties dialog box */
    SLViewDialogBase* baseProps;

    /** connected buffer */
    BufferLine currentLine;

    /** document loaded */
    bool ready;

    /** sync loop preventer */
    bool isSyncing;
};

#endif
