/*
 *                        slviewdialogbase.h
 *                           -------------------
 *  begin                : lun set 1 2003
 *  copyright            : (C) 2003 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * $Id: slviewdialogbase.h,v 1.4 2004/12/10 16:19:14 gbertani Exp $
 */

#ifndef SLVIEWDIALOGBASE_H
#define SLVIEWDIALOGBASE_H

// include files for Qt
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qwidget.h>
#include <qlayout.h>

// include files for KDE
#include <klocale.h>
#include <kconfig.h>
#include <kdialogbase.h>


#include "slsession.h"

/**Base class for all views properties dialogs
 * It defines the pageSourceBuffer tab that is present
 * for all the views types
 * 
 *@author giuseppe massimo bertani
 */

class SLViewDialogBase : public KDialogBase  
{
    Q_OBJECT


    //BufferLine defined in slsession.h 
    
public:
    static const int pageSourceBuffer;

	SLViewDialogBase(QString title, QWidget *parent=0, const char *name=0, bool modal = false);
	~SLViewDialogBase();
    
    void save(KConfig* data);
    void load(KConfig const* data);

//--------------- gets -----------------------------------------------------

    /** returns the id of the item selected in the dialog */
    BufferLine getSourceBuffer() const;

    /** returns true if the view can accept synchronization signals */
    bool getSyncView() const;

//----------sets-------------------------------------------------------------   
    
    /** sets the given item as selected in the dialog */
    void setSourceBuffer( BufferLine b );

    /** sets if the view can accept synchronization signals */
    void setSyncView( bool sync );
    
public slots:
    virtual void slotAllDefaults() = 0;
    virtual void slotDefault() = 0;
    virtual void slotOk();
    virtual void slotApply();
    
signals:
    void sourceBufferChoice();

protected:
    struct SSourceBuffer
    {
        QComboBox   *sourceBufferSelector;
        QLabel      *sourceBufferLabel;

        QCheckBox   *syncViewBox;
        QLabel      *syncViewLabel;
    };

    
    
    SSourceBuffer   mSourceBuffer; 

    /**
     * to be redefined by derived classes
     */ 
    virtual void setupSourceBufferPage() = 0;
    
};


#endif
